(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    384729,      11292]
NotebookOptionsPosition[    362674,      10606]
NotebookOutlinePosition[    363909,      10647]
CellTagsIndexPosition[    363689,      10639]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["DiracQ Example Book",
 FontSize->18]], "Title",
 CellChangeTimes->{{3.5661377119262004`*^9, 3.5661377278651123`*^9}},
 TextAlignment->Center,
 FontFamily->"Times New Roman",
 FontSize->22],

Cell[CellGroupData[{

Cell[TextData[StyleBox["A Compilation of Problems in Quantum Theory Performed \
Using the DiracQ Package",
 FontSize->16]], "Section",
 CellChangeTimes->{{3.4882178074748583`*^9, 3.4882178344073153`*^9}, {
  3.488221107145529*^9, 3.488221114640195*^9}, {3.4882292085169783`*^9, 
  3.488229211572301*^9}, {3.5661377494673476`*^9, 3.5661377880025516`*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell["\<\
The following problems demonstrate how the DiracQ package can be used to \
solve non-trivial problems. Throughout this notebook ensure that you have the \
relevant operators activated on the DiracQ palette before evaluating any \
input.\
\>", "Text",
 CellChangeTimes->{{3.566137113579977*^9, 3.566137206330282*^9}, {
  3.5661376865137467`*^9, 3.5661377044067707`*^9}, {3.5661378665520444`*^9, 
  3.5661378758605766`*^9}},
 TextAlignment->Left],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Get", "[", "\"\<DiracQV1.m\>\"", "]"}], ";"}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQzcAQpsdg+MqRgaHDGEIbN/wC0z4QumFR6zETEN+j
DUIfkzpkChK/DqY9rti+73Z/5ajy1+cDiI7hbdCR9XjlWKrYpQeiXcM8Du/0
feV46F05mBbt/SNjPuGVI9cVMUUQPS/u8Umvia8cXyjanwbRsxwWxswG0lmR
G8E0sysbywIg7ecqBab1t4o7gmgbay1XEM0XZrB/A5CuTbIC0zY5ty+D6JOz
HK+AaBPr299AdF7aKzD9R1b0N4g+cMkSTIekLNTbCKR7npvpg+hLuazst4D0
xIe+YLpMveL7bSCt2lj0A0SL2P21ZJ30yvGGorQViC6ruaadp/fa8cvle2Aa
AFRYolo=
  "]],

Cell[CellGroupData[{

Cell["(I) Canonical Pairs", "Subsection",
 CellChangeTimes->{{3.489783162183263*^9, 3.4897831662447357`*^9}, {
  3.4999729687494373`*^9, 3.499972970223175*^9}, {3.530412037420274*^9, 
  3.5304120385416727`*^9}},
 FontFamily->"Times"],

Cell["\<\
The package contains canonical position and momentum operators. Their use is \
demonstrated below for some simple examples.\
\>", "Text",
 CellChangeTimes->{{3.489783212968416*^9, 3.489783254299429*^9}, {
   3.499972991137834*^9, 3.4999730051101723`*^9}, 3.5662165659921155`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "p"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"p is the canonical momentum operator. This operator can be \
called with one argument, taken to be site index, or two arguments. The \
second argument will be taken to be coordinate direction. Also included is \
the 3 dimensional canonical momentum vector, represented by OverVector[p], or \
\!\(\*OverscriptBox[\\\"p\\\", \\\"\[RightVector]\\\"]\).\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.648221544477581*^9},
 CellTags->"Info3648196344-2372840"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "q"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"q is the canonical position operator. This operator can be \
called with one argument, taken to be site index, or two arguments. The \
second argument will be taken to be coordinate direction. Also included is \
the 3 dimensional canonical position vector, represented by OverVector[q], or \
\!\(\*OverscriptBox[\\\"q\\\", \\\"\[RightVector]\\\"]\).\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.648221544838644*^9},
 CellTags->"Info3648196344-2372840"]
}, Open  ]],

Cell[TextData[{
 "The physical system can be in arbitrary dimensions and may refer to an \
arbotrary number of particles. The canonical pairs are endowed with indices:  \
either one or two indices can be used. The first index is taken to be site \
label. The second index is taken to be the component. If only one index is \
given, a 1-dimensional system is assumed. This is demonstrated below. \
[Note:",
 StyleBox[" unless you choose to declare q and p as active operators in the \
DiracQ Palette by ticking the boxes in the Palette, (or more aggressively \
turn all symbols on), the following commutators will turn out to vanish. If \
that happens, go back and turn them  \"on\". ",
  FontSlant->"Italic"],
 "]"
}], "Text",
 CellChangeTimes->{{3.4897832885035*^9, 3.489783347750432*^9}, {
  3.499973013300365*^9, 3.49997303217857*^9}, {3.499973064454484*^9, 
  3.499973166110825*^9}, {3.4999732046542873`*^9, 3.499973222328467*^9}, {
  3.5002607880359373`*^9, 3.500260915801588*^9}, {3.500260960778564*^9, 
  3.500261002866165*^9}, {3.5662165794338846`*^9, 3.5662165836651263`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{"p", "[", "j", "]"}], ",", 
   RowBox[{"q", "[", "i", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "\[ImaginaryI]"}], " ", "\[HBar]", " ", 
  RowBox[{"\[Delta]", "[", 
   RowBox[{"i", ",", "j"}], "]"}]}]], "Output",
 CellChangeTimes->{3.565997869781588*^9, 3.648221553797858*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{"p", "[", 
    RowBox[{"j", ",", "\[Alpha]"}], "]"}], ",", 
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "\[ImaginaryI]"}], " ", "\[HBar]", " ", 
  RowBox[{"\[Delta]", "[", 
   RowBox[{"i", ",", "j"}], "]"}], " ", 
  RowBox[{"\[Delta]", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
 CellChangeTimes->{3.565997878945112*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{"p", "[", 
    RowBox[{"j", ",", "y"}], "]"}], ",", 
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "y"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "\[ImaginaryI]"}], " ", "\[HBar]", " ", 
  RowBox[{"\[Delta]", "[", 
   RowBox[{"i", ",", "j"}], "]"}]}]], "Output",
 CellChangeTimes->{3.565997884896453*^9, 3.648221567608316*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5659978909648*^9, 3.64822157876884*^9}]
}, Open  ]],

Cell["\<\
Comment: In the last output, notice that p and q without any indices are \
treated as c-numbers.  This can be a slight nuisance, but one  resolution is \
to treat a set of  harmonic oscillators indexed by a \"i\".\
\>", "Text",
 CellChangeTimes->{{3.4999741075029783`*^9, 3.499974213958342*^9}, {
  3.565271440474296*^9, 3.565271487735922*^9}, {3.565997774471137*^9, 
  3.5659977754481926`*^9}}],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"i", ",", "j"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"H", "=", 
   RowBox[{
    FractionBox[
     SuperscriptBox[
      RowBox[{"p", "[", "i", "]"}], "2"], 
     RowBox[{"2", "m"}]], "+", 
    RowBox[{
     FractionBox[
      SuperscriptBox["m\[Omega]", "2"], "2"], 
     SuperscriptBox[
      RowBox[{"q", "[", "i", "]"}], "2"]}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{"H", ",", 
   RowBox[{"p", "[", "i", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["m\[Omega]", "2"], " ", "\[HBar]"}], ")"}], "**", 
   RowBox[{"q", "[", "i", "]"}]}]}]], "Output",
 CellChangeTimes->{3.5659979189013977`*^9, 3.648221589701807*^9}]
}, Open  ]],

Cell["\<\
Next we construct creation and destruction operators from the canonical pairs.\
\>", "Text",
 CellChangeTimes->{{3.4999742572724133`*^9, 3.499974292180277*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"a", "[", "i_", "]"}], ":=", 
   RowBox[{
    SqrtBox[
     FractionBox[
      RowBox[{"m", " ", "\[Omega]"}], 
      RowBox[{"2", "\[HBar]"}]]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"q", "[", "i", "]"}], "+", 
      RowBox[{"\[ImaginaryI]", " ", 
       FractionBox[
        RowBox[{"p", "[", "i", "]"}], 
        RowBox[{"m", " ", "\[Omega]"}]]}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"a\[Dagger]", "[", "i_", "]"}], ":=", 
   RowBox[{
    SqrtBox[
     FractionBox[
      RowBox[{"m", " ", "\[Omega]"}], 
      RowBox[{"2", "\[HBar]"}]]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"q", "[", "i", "]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", 
       FractionBox[
        RowBox[{"p", "[", "i", "]"}], 
        RowBox[{"m", " ", "\[Omega]"}]]}]}], ")"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5659979392885637`*^9, 3.565997939635584*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{"a", "[", "i", "]"}], ",", 
   RowBox[{"a\[Dagger]", "[", "i", "]"}]}], "]"}]], "Input"],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.5659979485520935`*^9, 3.648221597172402*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"n", "[", "i_", "]"}], ":=", 
  RowBox[{
   RowBox[{"a\[Dagger]", "[", "i", "]"}], "**", 
   RowBox[{"a", "[", "i", "]"}]}]}]], "Input"],

Cell["\<\
Here we show that [n[i], a[i]] = -a[i] and similarly [n[i],a\[Dagger][i]]=a\
\[Dagger][i].  SimplifyQ puts the expressions in a standard form where the \
identity of two expressions becomes clear.\
\>", "Text",
 CellChangeTimes->{{3.489783428470755*^9, 3.489783497469322*^9}, {
   3.51741176162153*^9, 3.517411788002748*^9}, {3.530412466639282*^9, 
   3.530412517080553*^9}, 3.565271652150175*^9, {3.648221667276739*^9, 
   3.648221840950561*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyQ", "[", 
  RowBox[{"a", "[", "i", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    SqrtBox[
     FractionBox[
      RowBox[{"m", " ", "\[Omega]"}], "\[HBar]"]], "**", 
    RowBox[{"q", "[", "i", "]"}]}], 
   SqrtBox["2"]], "+", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     FractionBox["1", 
      RowBox[{
       SqrtBox[
        FractionBox[
         RowBox[{"m", " ", "\[Omega]"}], "\[HBar]"]], " ", "\[HBar]"}]], "**", 
     RowBox[{"p", "[", "i", "]"}]}]}], 
   SqrtBox["2"]]}]], "Output",
 CellChangeTimes->{3.565997965541065*^9, 3.648221610270998*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyQ", "[", 
  RowBox[{
   RowBox[{"Commutator", "[", 
    RowBox[{
     RowBox[{"n", "[", "i", "]"}], ",", 
     RowBox[{"a", "[", "i", "]"}]}], "]"}], "+", 
   RowBox[{"a", "[", "i", "]"}]}], "]"}]], "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.565997974047552*^9, 3.648221610368965*^9}]
}, Open  ]],

Cell["\<\
 (Note  that the command is  more explicitly A1= Commutator[n[i],a[i]] ]; A2= \
  +a[i]; A = SimplifyQ[A1+A2]. Of course the final result A vanishes, as it \
must)\
\>", "Text",
 CellChangeTimes->{{3.64822184672864*^9, 3.648221872336597*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyQ", "[", 
  RowBox[{
   RowBox[{"Commutator", "[", 
    RowBox[{
     RowBox[{"n", "[", "i", "]"}], ",", 
     RowBox[{"a\[Dagger]", "[", "i", "]"}]}], "]"}], "-", 
   RowBox[{"a\[Dagger]", "[", "i", "]"}]}], " ", "]"}]], "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.565997980801938*^9, 3.648221610471315*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["(II) ",
  FontSize->14],
 StyleBox["Hydrogen atom and Runge Lenz Laplace vectors",
  FontFamily->"Times",
  FontSize->14]
}], "Subsection",
 CellChangeTimes->{{3.490039906034565*^9, 3.4900399139846582`*^9}, 
   3.490041233093458*^9, {3.4903670246004477`*^9, 3.490367027717663*^9}, {
   3.4999727128796177`*^9, 3.499972723136116*^9}, {3.530412705295313*^9, 
   3.530412706686549*^9}, {3.565271868301515*^9, 3.5652718705009747`*^9}},
 FontFamily->"Utopia",
 FontSize->16],

Cell["\<\
Here we define the Hamiltonian of the Hydrogen atom. We demonstrate the \
conservation of the Runge-Lenz-Laplace vector by using the package.
We use the three dimensional vector notation for the canonical pair. For \
example\
\>", "Text",
 CellChangeTimes->{{3.490367003344953*^9, 3.490367020677183*^9}, {
   3.499975544453388*^9, 3.499975569091337*^9}, {3.499975968202198*^9, 
   3.499975971429125*^9}, {3.56527180917949*^9, 3.565271859276442*^9}, {
   3.5659978351886096`*^9, 3.5659978404669113`*^9}, 3.5662166057153873`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}]], "Input",
 CellChangeTimes->{{3.522506158428342*^9, 3.522506179730585*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "x"}], "]"}], ",", 
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "y"}], "]"}], ",", 
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "z"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.522506180830659*^9, 3.5235802046034403`*^9, 
  3.524510449241392*^9, 3.524510855848089*^9, 3.527171258365738*^9, 
  3.5304127481601763`*^9, 3.565271925890918*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}], ".", 
  RowBox[{
   OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}]}]], "Input",
 CellChangeTimes->{{3.499975761607192*^9, 3.499975836553149*^9}, {
  3.499975937078608*^9, 3.499975949292981*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "z"}], "]"}], "2"]}]], "Output",
 CellChangeTimes->{{3.499975950144088*^9, 3.4999759787943153`*^9}, 
   3.5174118972244053`*^9, 3.5174168545592527`*^9, 3.5176076312691183`*^9, 
   3.5176560695448103`*^9, 3.517659909410841*^9, 3.517679374726987*^9, 
   3.517687118867612*^9, 3.5177582280625*^9, 3.5225061004011993`*^9, 
   3.523580204702696*^9, 3.524510449341416*^9, 3.5245108559484863`*^9, 
   3.527171258398182*^9, 3.5304127482229023`*^9, 3.565271925956575*^9}]
}, Closed]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "H", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"H", "=", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{
       OverscriptBox["p", "\[RightVector]"], "[", "i", "]"}], ".", 
      RowBox[{
       OverscriptBox["p", "\[RightVector]"], "[", "i", "]"}]}], 
     RowBox[{"2", "m"}]], "-", 
    RowBox[{"Z", " ", 
     FractionBox[
      SuperscriptBox["\[ExponentialE]", "2"], 
      SqrtBox[
       RowBox[{
        RowBox[{
         OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}], ".", 
        RowBox[{
         OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}]}]]]}]}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.490033791326199*^9, 3.490033845906204*^9}, 
   3.490041233096446*^9, 3.490124964045618*^9, {3.4909826788352213`*^9, 
   3.490982715350062*^9}, {3.517657512221197*^9, 3.517657513178385*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "NCcross"}]], "Input",
 CellChangeTimes->{{3.517659922571973*^9, 3.517659933296508*^9}}],

Cell[BoxData[
 StyleBox["\<\"Non Commutative cross product of two 3dimensional vectors \
retaining the order of the operators.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.565271944732335*^9},
 CellTags->"Info3565243144-4018571"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   OverscriptBox["L", "\[RightVector]"], "[", "i_", "]"}], "=", 
  RowBox[{"NCcross", "[", 
   RowBox[{
    RowBox[{
     OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}], ",", " ", 
    RowBox[{
     OverscriptBox["p", "\[RightVector]"], "[", "i", "]"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.490366605072197*^9, 3.490366684164921*^9}, {
   3.490366821076426*^9, 3.490366871477784*^9}, {3.490982741901638*^9, 
   3.49098277489323*^9}, {3.51741275801087*^9, 3.517412758629941*^9}, {
   3.517413442096987*^9, 3.517413449084825*^9}, {3.517416697984407*^9, 
   3.517416722754808*^9}, 3.517416882727839*^9, {3.517659942750853*^9, 
   3.517659949532752*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "y"}], "]"}], "**", 
     RowBox[{"p", "[", 
      RowBox[{"i", ",", "z"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "z"}], "]"}], "**", 
     RowBox[{"p", "[", 
      RowBox[{"i", ",", "y"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "x"}], "]"}], "**", 
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "z"}], "]"}]}]}], "+", 
    RowBox[{
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "z"}], "]"}], "**", 
     RowBox[{"p", "[", 
      RowBox[{"i", ",", "x"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "x"}], "]"}], "**", 
     RowBox[{"p", "[", 
      RowBox[{"i", ",", "y"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "y"}], "]"}], "**", 
     RowBox[{"p", "[", 
      RowBox[{"i", ",", "x"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.517659957537359*^9, 3.517679379742612*^9, {3.517687120617612*^9, 
   3.517687127508237*^9}, 3.517758230328125*^9, 3.517771684375*^9, 
   3.5178465005*^9, 3.52250618937978*^9, 3.523580205039928*^9, 
   3.5245104567320547`*^9, 3.524510862222632*^9, 3.527171275719486*^9, 
   3.530412748568574*^9, 3.565271944883873*^9, 3.5659979950267515`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     OverscriptBox["L", "\[RightVector]"], "[", "i", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}], ",", "H"}], "]"}]], "Input",
 CellChangeTimes->{{3.490366951839465*^9, 3.49036696691115*^9}, {
  3.5174133245775957`*^9, 3.517413332860297*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.490366952742153*^9, 3.490367872044886*^9, 3.4903736627036448`*^9, 
   3.490982793874758*^9, 3.4909837224593687`*^9, 3.491073675096135*^9, 
   3.4999758807413397`*^9, 3.517411903370509*^9, {3.517413318000148*^9, 
   3.5174133340835533`*^9}, 3.5174168985617237`*^9, 3.517607658864749*^9, 
   3.517607853456107*^9, 3.517656084831229*^9, 3.517657558293412*^9, 
   3.517659968197002*^9, 3.517679386211362*^9, 3.517680011742612*^9, 
   3.517687129617612*^9, 3.517758231421875*^9, 3.5177716853125*^9, 
   3.51784650128125*^9, 3.522506207942163*^9, 3.5235802051933193`*^9, 
   3.524510456932485*^9, 3.524510862428787*^9, 3.527171275879191*^9, 
   3.530412748724401*^9, 3.5652719537058372`*^9, 3.5659979964868355`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     OverscriptBox["L", "\[RightVector]"], "[", "i", "]"}], "[", 
    RowBox[{"[", "2", "]"}], "]"}], ",", "H"}], "]"}]], "Input",
 CellChangeTimes->{{3.490366969558968*^9, 3.490366969678658*^9}, {
  3.517413338011722*^9, 3.517413339664465*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.490366975014045*^9, 3.490367873144339*^9, 
  3.490373663833109*^9, 3.4909837241691027`*^9, 3.4910736761760893`*^9, 
  3.499975883607305*^9, 3.517411910927538*^9, 3.517413340508676*^9, 
  3.517416903853644*^9, 3.5176076591260223`*^9, 3.517656091866634*^9, 
  3.517657562366927*^9, 3.5176599707652693`*^9, 3.517679387023862*^9, 
  3.517687130492612*^9, 3.517758232265625*^9, 3.5177716860625*^9, 
  3.517846502046875*^9, 3.522506211564666*^9, 3.5235802053405933`*^9, 
  3.524510457083399*^9, 3.5245108625792933`*^9, 3.527171276029572*^9, 
  3.530412758210129*^9, 3.56527195389275*^9, 3.5659979999150314`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyQ", "[", 
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      OverscriptBox["L", "\[RightVector]"], "[", "i", "]"}], "[", 
     RowBox[{"[", "3", "]"}], "]"}], ",", "H"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.517413345077539*^9, 3.517413346672975*^9}, {
  3.517608265493032*^9, 3.5176082734115477`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.49036697285334*^9, 3.490367874248605*^9, 3.490373664915474*^9, 
   3.4909837254691267`*^9, 3.4910736772644987`*^9, 3.499975885576432*^9, 
   3.517411915447507*^9, 3.517413347752939*^9, 3.517416908058687*^9, 
   3.517607659347221*^9, 3.517608274080049*^9, 3.517656098310256*^9, 
   3.517657565763564*^9, 3.517659974644663*^9, 3.517679387820737*^9, 
   3.517687131242612*^9, 3.517758232875*^9, 3.517771686734375*^9, 
   3.517846502796875*^9, 3.5225062146843853`*^9, 3.5235802054872932`*^9, 
   3.52451045723424*^9, 3.5245108627310553`*^9, 3.527171276181629*^9, 
   3.530412758385993*^9, 3.565271954047139*^9, 3.565998002661188*^9, {
   3.565998044494581*^9, 3.565998051210965*^9}}]
}, Closed]],

Cell["\<\
The answer is so simple that it looks as though we have not performed any \
operation whatsoever. However, if we leave basic commutators unevaluated we \
see that indeed the computation was not entirely trivial (feel free to try it \
by setting \[OpenCurlyDoubleQuote]Apply Definition\[CloseCurlyDoubleQuote] to \
\[OpenCurlyDoubleQuote]False\[CloseCurlyDoubleQuote] using the palette and \
then reevaluate the input). As an example consider the product \
q[i,y]**p[i,z], it is not conserved and we get a messy answer:\
\>", "Text",
 CellChangeTimes->{{3.4903672953662043`*^9, 3.490367348584647*^9}, {
  3.565272002366497*^9, 3.565272047915051*^9}, {3.5659980300777564`*^9, 
  3.565998084076845*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "y"}], "]"}], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}], ",", "H"}], "]"}]], "Input",
 CellChangeTimes->{{3.522506272234885*^9, 3.522506279049835*^9}, {
  3.522506309290587*^9, 3.5225063142313766`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{
    FractionBox["\[HBar]", "m"], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "y"}], "]"}], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "y"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.522506279887308*^9, 3.5225063159701*^9, 
  3.523580205802312*^9, 3.524510461377192*^9, 3.524510870199294*^9, 
  3.52717128366509*^9, 3.530412758657317*^9, 3.5659980203191986`*^9}]
}, Closed]],

Cell[TextData[{
 "We next define the Runge Lenz vector ",
 Cell[BoxData[
  OverscriptBox["A", "\[Rule]"]],
  CellChangeTimes->{{3.490367055069154*^9, 3.4903670992165318`*^9}, {
   3.5174121850998774`*^9, 3.517412203358918*^9}, {3.517412236784143*^9, 
   3.517412279851679*^9}, {3.517412321261448*^9, 3.51741233325264*^9}},
  TextAlignment->Center,
  FontSize->14,
  FontWeight->"Bold"],
 "[i] with a formal definition:"
}], "Text",
 CellChangeTimes->{{3.499998165100471*^9, 3.4999981866131268`*^9}, {
   3.517412337918914*^9, 3.517412343741897*^9}, 3.517412405762785*^9, {
   3.5652720679895267`*^9, 3.5652720706526814`*^9}, {3.565272149953864*^9, 
   3.565272157065065*^9}, 3.565998090234197*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   OverscriptBox["A", "\[RightVector]"], "[", "i", "]"}], "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     OverscriptBox["L", "\[RightVector]"], "[", "i", "]"}], "\[Cross]", 
    RowBox[{
     OverscriptBox["p", "\[RightVector]"], "[", "i", "]"}]}], " ", "+", " ", 
   RowBox[{"Z", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "m", " ", 
    FractionBox[
     RowBox[{
      OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}], 
     SqrtBox[
      RowBox[{
       RowBox[{
        OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}], ".", 
       RowBox[{
        OverscriptBox["q", "\[RightVector]"], "[", "i", 
        "]"}]}]]]}]}]}]], "DisplayFormula",
 CellChangeTimes->{{3.490367055069154*^9, 3.4903670992165318`*^9}, {
  3.5174121850998774`*^9, 3.517412203358918*^9}, {3.517412236784143*^9, 
  3.517412279851679*^9}, {3.517412321261448*^9, 3.51741233325264*^9}, {
  3.517412378794689*^9, 3.517412389909966*^9}, {3.517412553656637*^9, 
  3.517412597240958*^9}, {3.5174135554329863`*^9, 3.5174135629257717`*^9}, {
  3.517413688981851*^9, 3.5174136904291687`*^9}, {3.517415806428276*^9, 
  3.517415810948242*^9}, {3.517679423133237*^9, 3.517679426226987*^9}, {
  3.565272062242218*^9, 3.565272062678892*^9}},
 TextAlignment->Center,
 FontSize->14,
 FontWeight->"Bold"],

Cell["\<\
Using the NCcross function and symmetrizing in the two variables L and p, we \
work with\
\>", "Text",
 CellChangeTimes->{{3.499998165100471*^9, 3.4999981866131268`*^9}, {
   3.517412337918914*^9, 3.517412343741897*^9}, 3.517412405762785*^9, {
   3.5652720679895267`*^9, 3.5652721613530283`*^9}, 3.5659981011478214`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    OverscriptBox["A", "\[RightVector]"], "[", "i", "]"}], "=", 
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], 
     RowBox[{"NCcross", "[", 
      RowBox[{
       RowBox[{
        OverscriptBox["L", "\[RightVector]"], "[", "i", "]"}], ",", " ", 
       RowBox[{
        OverscriptBox["p", "\[RightVector]"], "[", "i", "]"}]}], "]"}]}], " ",
     "-", " ", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"NCcross", "[", " ", 
      RowBox[{
       RowBox[{
        OverscriptBox["p", "\[RightVector]"], "[", "i", "]"}], ",", 
       RowBox[{
        OverscriptBox["L", "\[RightVector]"], "[", "i", "]"}]}], "]"}]}], " ",
     "+", "    ", 
    RowBox[{"Z", " ", 
     SuperscriptBox["\[ExponentialE]", "2"], " ", "m", " ", 
     FractionBox[
      RowBox[{
       OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}], 
      SqrtBox[
       RowBox[{
        RowBox[{
         OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}], ".", 
        RowBox[{
         OverscriptBox["q", "\[RightVector]"], "[", "i", "]"}]}]]]}]}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.490366605072197*^9, 3.490366684164921*^9}, {
   3.490366821076426*^9, 3.490366871477784*^9}, {3.490982741901638*^9, 
   3.49098277489323*^9}, {3.5174123693220997`*^9, 3.517412401027195*^9}, {
   3.517412434045342*^9, 3.5174124635536633`*^9}, {3.517413375970869*^9, 
   3.517413413646001*^9}, 3.517413653279271*^9, 3.517415814483671*^9, {
   3.517417001783682*^9, 3.517417012456702*^9}, {3.517417097284542*^9, 
   3.517417149677537*^9}, {3.5174209554416857`*^9, 3.5174209627650146`*^9}, {
   3.517423023808743*^9, 3.517423027564536*^9}, {3.517657762279406*^9, 
   3.517657762930299*^9}, {3.517657934945793*^9, 3.5176579362719*^9}, 
   3.5176580153504963`*^9, {3.51765924157784*^9, 3.517659241982204*^9}, {
   3.5176599878320513`*^9, 3.517659989718504*^9}, 3.5176600376033363`*^9}],

Cell["We check a component of A to make sure it makes sense.", "Text",
 CellChangeTimes->{{3.517413578491889*^9, 3.517413601631072*^9}, 
   3.566216629658757*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   OverscriptBox["A", "\[RightVector]"], "[", "i", "]"}], "[", 
  RowBox[{"[", "3", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"p", "[", 
        RowBox[{"i", ",", "x"}], "]"}], "**", 
       RowBox[{"(", 
        RowBox[{"-", 
         RowBox[{
          RowBox[{"q", "[", 
           RowBox[{"i", ",", "x"}], "]"}], "**", 
          RowBox[{"p", "[", 
           RowBox[{"i", ",", "z"}], "]"}]}]}], ")"}]}]}], "+", 
     RowBox[{
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "y"}], "]"}], "**", 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "**", 
         RowBox[{"p", "[", 
          RowBox[{"i", ",", "y"}], "]"}]}]}], ")"}]}], "-", 
     RowBox[{
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "x"}], "]"}], "**", 
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "z"}], "]"}], "**", 
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "x"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "y"}], "]"}], "**", 
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "y"}], "]"}], "**", 
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "z"}], "]"}]}]}], ")"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", 
         RowBox[{
          RowBox[{"q", "[", 
           RowBox[{"i", ",", "x"}], "]"}], "**", 
          RowBox[{"p", "[", 
           RowBox[{"i", ",", "z"}], "]"}]}]}], ")"}], "**", 
       RowBox[{"p", "[", 
        RowBox[{"i", ",", "x"}], "]"}]}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "**", 
         RowBox[{"p", "[", 
          RowBox[{"i", ",", "y"}], "]"}]}]}], ")"}], "**", 
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "y"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "y"}], "]"}], "**", 
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "z"}], "]"}], "**", 
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "y"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "z"}], "]"}], "**", 
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "x"}], "]"}], "**", 
      RowBox[{"p", "[", 
       RowBox[{"i", ",", "x"}], "]"}]}]}], ")"}]}], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox["\[ExponentialE]", "2"], " ", "m", " ", "Z", " ", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "z"}], "]"}], "2"]}]]]}]], "Output",
 CellChangeTimes->{
  3.517608207805039*^9, 3.517656127937435*^9, 3.517657587748435*^9, 
   3.517657768933608*^9, 3.517658024210763*^9, 3.517659247501238*^9, 
   3.517660003565284*^9, 3.517660042991693*^9, {3.517679430742612*^9, 
   3.517679448883237*^9}, 3.517758242234375*^9, 3.517771689421875*^9, 
   3.517846510796875*^9, 3.5225063615508842`*^9, {3.523580205872857*^9, 
   3.523580220208013*^9}, 3.5245104758902893`*^9, 3.524510877625043*^9, 
   3.527171294410776*^9, 3.5304127724369164`*^9, 3.565272172506452*^9, 
   3.565998110665366*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"term1", "=", 
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      OverscriptBox["A", "\[RightVector]"], "[", "i", "]"}], "[", 
     RowBox[{"[", "3", "]"}], "]"}], ",", "H"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.517413719793964*^9, 3.517413815623728*^9}, {
   3.5174205581205673`*^9, 3.5174205609286203`*^9}, {3.517608327707341*^9, 
   3.5176083324974546`*^9}, {3.5176562144767313`*^9, 3.51765621614108*^9}, {
   3.5176576165079927`*^9, 3.517657616912509*^9}, 3.517679454695737*^9, {
   3.517679526961362*^9, 3.517679534148862*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}], "**", 
    FractionBox["1", 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "z"}], "]"}], "2"]}]]]}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    FractionBox["1", 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "z"}], "]"}], "2"]}]]], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", 
     SuperscriptBox["\[HBar]", "2"]}], ")"}], "**", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
     RowBox[{"3", "/", "2"}]]], "**", 
   RowBox[{"q", "[", 
    RowBox[{"i", ",", "z"}], "]"}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "x"}], "]"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "x"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "y"}], "]"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "y"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "x"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "x"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "y"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "y"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "x"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "y"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "y"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]"}], 
     ")"}], "**", 
    FractionBox["1", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"q", "[", 
          RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]], "**", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}], "**", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{
  3.5174137236866407`*^9, 3.517413780140129*^9, 3.5174138181011887`*^9, 
   3.517415838080408*^9, 3.5174170403633223`*^9, 3.51741716462951*^9, 
   3.517420565949786*^9, 3.517420976310416*^9, 3.517423047073341*^9, 
   3.517608336016076*^9, 3.5176562219530077`*^9, 3.5176567066078577`*^9, 
   3.5176576199680853`*^9, 3.5176577904586897`*^9, 3.517657953980998*^9, 
   3.517658026167521*^9, 3.517659263188802*^9, {3.517660021356082*^9, 
   3.517660045476359*^9}, {3.517679435273862*^9, 3.517679458820737*^9}, {
   3.517679524195737*^9, 3.517679539180112*^9}, 3.51775824671875*^9, 
   3.517771693671875*^9, {3.517846491421875*^9, 3.517846519328125*^9}, 
   3.522506380548902*^9, 3.523580229862625*^9, 3.524510484010973*^9, 
   3.524510880281764*^9, 3.52717130182661*^9, 3.530412775096901*^9, 
   3.56527218235087*^9, 3.5659981162176833`*^9}]
}, Closed]],

Cell["\<\
With this expression, SimplifyQ does not produce a vanishing result since the \
expression on right locates p[i,z] differently in various terms and one needs \
to push this term to one side in ALL terms. Therefore at this stage it is \
better to push  all the momentum type variables to the right extreme ( the \
left extreme works equally well), so that all operators are ordered in a \
similar  way. After that  we can set all non commutative products to simple \
products and simplify.  The function pp and ppp achieve this ordering, and \
sim does the job of simplifying the expressions after replacing the **\
\[CloseCurlyQuote]s with simple *\[CloseCurlyQuote]s. \
\>", "Text",
 CellChangeTimes->{{3.530412788519991*^9, 3.530412983580804*^9}, {
  3.565272227469846*^9, 3.565272374879787*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pp", "[", 
   RowBox[{"a_", ",", "m_"}], "]"}], ":=", 
  RowBox[{"PushOperatorRight", "[", 
   RowBox[{"a", ",", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "m"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.522584557855955*^9, 3.52258460425848*^9}, {
  3.522584635938699*^9, 3.522584646289259*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ppp", "[", "a_", "]"}], ":=", 
  RowBox[{"pp", "[", 
   RowBox[{
    RowBox[{"pp", "[", 
     RowBox[{
      RowBox[{"pp", "[", 
       RowBox[{"a", ",", "x"}], "]"}], ",", "y"}], "]"}], ",", "z"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5225847320154657`*^9, 3.52258474938431*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sim", "[", "a_", "]"}], ":=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"ppp", "[", "a", "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{"NonCommutativeMultiply", "\[Rule]", "Times"}], "}"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sim", "[", "term1", "]"}]], "Input",
 CellChangeTimes->{{3.52451090262297*^9, 3.524510912051221*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5245109136164427`*^9, 3.527171316659869*^9, 
  3.530412992233914*^9, 3.565272397227397*^9, 3.5659981395240164`*^9}]
}, Closed]],

Cell["\<\
This can be examined more closely by examining the intermediate terms:\
\>", "Text",
 CellChangeTimes->{{3.530413008433671*^9, 3.530413036433495*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ppp", "[", "term1", "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{"NonCommutativeMultiply", "\[Rule]", "Times"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5245109504689407`*^9, 3.524510963429295*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"3", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", 
    SuperscriptBox["\[HBar]", "2"], " ", 
    SuperscriptBox[
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "x"}], "]"}], "2"], " ", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
     RowBox[{"5", "/", "2"}]]}]], "+", 
  FractionBox[
   RowBox[{"3", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", 
    SuperscriptBox["\[HBar]", "2"], " ", 
    SuperscriptBox[
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "y"}], "]"}], "2"], " ", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
     RowBox[{"5", "/", "2"}]]}]], "+", 
  FractionBox[
   RowBox[{"3", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", 
    SuperscriptBox["\[HBar]", "2"], " ", 
    SuperscriptBox[
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "z"}], "]"}], "3"]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
     RowBox[{"5", "/", "2"}]]}]], "-", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]", " ", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}], " ", 
    SuperscriptBox[
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "x"}], "]"}], "2"]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
    RowBox[{"3", "/", "2"}]]], "-", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]", " ", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}], " ", 
    SuperscriptBox[
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "y"}], "]"}], "2"]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
    RowBox[{"3", "/", "2"}]]], "-", 
  FractionBox[
   RowBox[{"3", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", 
    SuperscriptBox["\[HBar]", "2"], " ", 
    RowBox[{"q", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"q", "[", 
         RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
     RowBox[{"3", "/", "2"}]]}]], "-", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]", " ", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}], " ", 
    SuperscriptBox[
     RowBox[{"q", "[", 
      RowBox[{"i", ",", "z"}], "]"}], "2"]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"q", "[", 
        RowBox[{"i", ",", "z"}], "]"}], "2"]}], ")"}], 
    RowBox[{"3", "/", "2"}]]], "+", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[ExponentialE]", "2"], " ", "Z", " ", "\[HBar]", " ", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "z"}], "]"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "x"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "y"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{"q", "[", 
       RowBox[{"i", ",", "z"}], "]"}], "2"]}]]]}]], "Output",
 CellChangeTimes->{3.524510966235914*^9, 3.5271713758832417`*^9, 
  3.530413044236773*^9, 3.5652724051645813`*^9, 3.5659981549688997`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.524510981841392*^9, 3.524510985458981*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.524510985930258*^9, 3.527171379684979*^9, 
  3.530413044300889*^9, 3.565272414110013*^9, 3.5659981578100624`*^9}]
}, Closed]],

Cell[TextData[StyleBox["The same scheme is demonstrated by pushing operators \
to the left (rather than right) here.",
 FontWeight->"Plain"]], "Text",
 CellChangeTimes->{{3.5304130516087923`*^9, 3.5304130691028023`*^9}, {
  3.565272418993918*^9, 3.5652724297801657`*^9}},
 FontWeight->"Bold"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ppl", "[", 
   RowBox[{"a_", ",", "m_"}], "]"}], ":=", 
  RowBox[{"PushOperatorLeft", "[", 
   RowBox[{"a", ",", 
    RowBox[{"p", "[", 
     RowBox[{"i", ",", "m"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.522584557855955*^9, 3.52258460425848*^9}, {
  3.522584635938699*^9, 3.522584646289259*^9}, {3.527171396112197*^9, 
  3.5271714049761477`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pppl", "[", "a_", "]"}], ":=", 
  RowBox[{"ppl", "[", 
   RowBox[{
    RowBox[{"ppl", "[", 
     RowBox[{
      RowBox[{"ppl", "[", 
       RowBox[{"a", ",", "x"}], "]"}], ",", "y"}], "]"}], ",", "z"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5225847320154657`*^9, 3.52258474938431*^9}, {
  3.527171410680108*^9, 3.527171418542798*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"siml", "[", "a_", "]"}], ":=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"pppl", "[", "a", "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{"NonCommutativeMultiply", "\[Rule]", "Times"}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5271714235389214`*^9, 3.527171428522789*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"siml", "[", "term1", "]"}]], "Input",
 CellChangeTimes->{{3.52451090262297*^9, 3.524510912051221*^9}, 
   3.5271714311087093`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5245109136164427`*^9, 3.527171316659869*^9, 
  3.527171437736123*^9, 3.530413098365654*^9, 3.565272437936005*^9, 
  3.5659981817604322`*^9}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["(III)  Fermions at work", "Subsection",
 CellChangeTimes->{{3.488217858244334*^9, 3.4882179022404833`*^9}, {
  3.499972803462637*^9, 3.49997280814253*^9}, {3.530412052412681*^9, 
  3.530412055529067*^9}, {3.5652751439401493`*^9, 3.565275146112541*^9}},
 FontFamily->"Times"],

Cell["\<\
For this Section Ensure that Fermionic Operators as well as Bra and Ket \
Vectors are Activated.\
\>", "Item",
 CellChangeTimes->{{3.565998927131065*^9, 3.565998958127838*^9}},
 FontWeight->"Bold"],

Cell["\<\
Here we use 1 to represent spin up and - 1 to represent spin down. We now \
perform some test operations in the context of three problems using the \
Hubbard model Hamiltonian, given below.\
\>", "Text",
 CellChangeTimes->{{3.487690411844791*^9, 3.4876904323045807`*^9}, {
   3.487690657876333*^9, 3.487690689755822*^9}, {3.565275556893036*^9, 
   3.565275566415707*^9}, 3.566216653091097*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"i", ",", "j", ",", "n", ",", "H"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"H", " ", "=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      UnderscriptBox["\[Sum]", "i"], 
      RowBox[{
       UnderscriptBox["\[Sum]", "j"], 
       RowBox[{
        UnderscriptBox["\[Sum]", "\[Sigma]"], 
        RowBox[{
         RowBox[{"t", "[", 
          RowBox[{"i", ",", " ", "j"}], "]"}], " ", 
         RowBox[{
          RowBox[{"f\[Dagger]", "[", 
           RowBox[{"i", ",", " ", "\[Sigma]"}], "]"}], "**", 
          RowBox[{"f", "[", 
           RowBox[{"j", ",", " ", "\[Sigma]"}], "]"}]}]}]}]}]}]}], "+", 
    RowBox[{"U", " ", 
     RowBox[{
      UnderscriptBox["\[Sum]", "i"], 
      RowBox[{
       RowBox[{
        SubscriptBox["n", "f"], "[", 
        RowBox[{"i", ",", " ", "1"}], "]"}], "**", 
       RowBox[{
        SubscriptBox["n", "f"], "[", 
        RowBox[{"i", ",", " ", 
         RowBox[{"-", "1"}]}], "]"}]}]}]}]}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Problem", " ", "1"}], ":", " ", 
  RowBox[{"[", 
   RowBox[{
    SubsuperscriptBox["f", "i", "\[Sigma]"], ",", "H"}], 
   "]"}]}]], "Subsubsection",
 CellChangeTimes->{{3.488225466882596*^9, 3.488225500857769*^9}},
 FontFamily->"Times",
 FontSize->14],

Cell["\<\
    These examples require the user to input expressions carefully. For \
example, notice that we have a sum over \[Sigma] in the first term of H \
above. We therefore cannot use an operator that has a spin specified by \
\[Sigma], because the package does not differentiate that one sigma is inside \
the sum and the other is outside. It will attempt to sum over both \
\[Sigma]'s. This is not so much a restriction as it is a warning against \
sloppy notation that could easily be understood by a human but will be \
misinterpreted by the package. The same restriction applies to site indices, \
such as i and j in this example. 
The example below demonstrates incorrect usage of indices; we are taking the \
commutator of a fermi operator with a site index that is identical to the \
indice of summation in the Hamiltonian.\
\>", "Text",
 CellChangeTimes->{{3.4882222779887342`*^9, 3.488222513021722*^9}, 
   3.489179049874955*^9, 3.5652756254188957`*^9, {3.565275699178018*^9, 
   3.565275717065517*^9}, {3.565998273472678*^9, 3.565998342673636*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"i", ",", "\[Sigma]"}], "]"}], ",", "H"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"U", " ", 
      RowBox[{"\[Delta]", "[", 
       RowBox[{"1", ",", "\[Sigma]"}], "]"}]}], ")"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"i", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"U", " ", 
      RowBox[{"\[Delta]", "[", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "\[Sigma]"}], "]"}]}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"i", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    UnderscriptBox["\[Sum]", "j"], 
    RowBox[{
     UnderscriptBox["\[Sum]", "\[Sigma]"], 
     RowBox[{
      RowBox[{"t", "[", 
       RowBox[{"i", ",", "j"}], "]"}], "**", 
      RowBox[{"f", "[", 
       RowBox[{"j", ",", "\[Sigma]"}], "]"}]}]}]}]}]}]], "Output",
 CellChangeTimes->{3.5659982684103885`*^9, 3.5659983455388002`*^9, 
  3.64822206862922*^9}]
}, Open  ]],

Cell[TextData[{
 "The above answer is ",
 StyleBox["not",
  FontWeight->"Bold"],
 " correct."
}], "Text",
 CellChangeTimes->{{3.488222518632819*^9, 3.488222537187161*^9}}],

Cell["\<\
Trying same the problem again using proper notation where the external \
indices are different from the ones used in H, for example evaluate\
\>", "Text",
 CellChangeTimes->{{3.488222539007514*^9, 3.488222563517468*^9}, {
  3.489244135713903*^9, 3.489244147449548*^9}, {3.5652758023660107`*^9, 
  3.565275820966617*^9}, {3.648222096975768*^9, 3.648222102523707*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"k", ",", " ", "\[Sigma]1"}], "]"}], ",", " ", "H"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"U", " ", 
     RowBox[{"\[Delta]", "[", 
      RowBox[{"1", ",", "\[Sigma]1"}], "]"}]}], ")"}], "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"U", " ", 
     RowBox[{"\[Delta]", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "\[Sigma]1"}], "]"}]}], ")"}], "**", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"k", ",", "1"}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", "\[Sigma]1"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.5659983682180967`*^9, 3.648222108381524*^9}]
}, Open  ]],

Cell[TextData[{
 "we get the correct answer.  We can simplify the answer further by \
specifying a value for ",
 Cell[BoxData[
  FormBox["\[Sigma]1", TraditionalForm]]],
 "."
}], "Text",
 CellChangeTimes->{{3.487690699154665*^9, 3.487690791781266*^9}, {
  3.488222086499817*^9, 3.488222207963977*^9}, {3.4882225682468987`*^9, 
  3.488222571231144*^9}, {3.4882226396427*^9, 3.488222708395677*^9}, {
  3.4892441416019993`*^9, 3.489244141953821*^9}, {3.565275870968403*^9, 
  3.5652758782422743`*^9}, {3.565275918618987*^9, 3.565275922991993*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"k", ",", " ", "\[Sigma]1"}], "]"}], ",", " ", "H"}], "]"}], "/.", 
  RowBox[{"\[Sigma]1", "\[Rule]", "1"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"U", "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", "1"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.5659983945186014`*^9, 3.648222117429859*^9}]
}, Open  ]],

Cell["\<\
The best way of performing this calculation is just to start with the spin \
specified. In general this method is preferred because the package is then \
able to organize the operators using the value of the spins. It may therefore \
be necessary to reorder the output again if you specify the value of a spin \
after calculation.\
\>", "Text",
 CellChangeTimes->{{3.488222794776705*^9, 3.4882228818743277`*^9}, {
   3.488223001167288*^9, 3.488223104933601*^9}, {3.489177014835631*^9, 
   3.4891770289474688`*^9}, {3.565275965205864*^9, 3.5652759697678022`*^9}, 
   3.566216667677932*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"k", ",", " ", "1"}], "]"}], ",", " ", "H"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"U", "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", "1"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.565998425835393*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"k", ",", " ", 
     RowBox[{"-", "1"}]}], "]"}], ",", " ", "H"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"U", "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"k", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.565998432456771*^9}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Problem", " ", "2"}], ":", " ", 
  RowBox[{"[", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["f", "i", "\[Sigma]"], " ", 
     SubsuperscriptBox["n", "i", 
      RowBox[{"-", "\[Sigma]"}]]}], ",", "H"}], "]"}]}]], "Subsubsection",
 CellChangeTimes->{{3.488225542934918*^9, 3.488225601089362*^9}, 
   3.488225667121132*^9},
 FontFamily->"Times",
 FontSize->14],

Cell["\<\
In this example we see that the answer is different in terms of operator \
ordering, depending on which method we use to perform the calculation.\
\>", "Text",
 CellChangeTimes->{{3.488223108736401*^9, 3.4882231533917627`*^9}, {
  3.565276061281176*^9, 3.565276086043964*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"k", ",", "\[Sigma]1"}], "]"}], "**", 
     RowBox[{
      SubscriptBox["n", "f"], "[", 
      RowBox[{"k", ",", 
       RowBox[{"-", "\[Sigma]1"}]}], "]"}]}], ",", "H"}], "]"}], "/.", 
  RowBox[{"\[Sigma]1", "\[Rule]", "1"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"U", "**", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "-", 
  RowBox[{"U", "**", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"k", ",", "1"}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "k"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.5659984479896593`*^9, 3.648222147744631*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}]}], ",", "H"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"U", "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "k"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.565998457196186*^9, 3.648222148262301*^9}]
}, Open  ]],

Cell["\<\
The two results are not identical. However, if we reorder the operators in \
the top output we replicate the bottom output, which has the operators \
correctly ordered.\
\>", "Text",
 CellChangeTimes->{{3.488223166664863*^9, 3.488223250844434*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardOrderQ", "[", 
  RowBox[{
   RowBox[{"Commutator", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"k", ",", "\[Sigma]1"}], "]"}], "**", 
      RowBox[{
       SubscriptBox["n", "f"], "[", 
       RowBox[{"k", ",", 
        RowBox[{"-", "\[Sigma]1"}]}], "]"}]}], ",", "H"}], "]"}], "/.", 
   RowBox[{"\[Sigma]1", "\[Rule]", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"U", "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"k", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"f", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "k"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{
  3.5659984770023193`*^9, {3.6482221484948597`*^9, 3.648222161851424*^9}}]
}, Open  ]],

Cell["\<\
Performing the same calculation with a spin down particle we obtain the same \
answer with the spins reversed.\
\>", "Text",
 CellChangeTimes->{{3.488225630220169*^9, 3.488225657264285*^9}, {
  3.489244168361532*^9, 3.489244171993539*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"k", ",", "1"}], "]"}]}], ",", "H"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"U", "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"k", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "k"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"k", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "j"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"k", ",", "j"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"k", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"k", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"j", ",", "1"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{
  3.565998816969764*^9, {3.648222148761117*^9, 3.64822216990548*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Problem", " ", "3"}], ":", " ", 
    RowBox[{"H", " ", 
     SubsuperscriptBox[
      SuperscriptBox["f", "\[Dagger]"], "1", "\[UpArrow]"], 
     SubsuperscriptBox[
      SuperscriptBox["f", "\[Dagger]"], "2", "\[DownArrow]"]}]}], " ", 
   "\[VerticalSeparator]", "Vacuum"}], 
  "\[RightAngleBracket]"}]], "Subsubsection",
 CellChangeTimes->{{3.48822569509464*^9, 3.488225760135529*^9}, {
   3.488225791662828*^9, 3.488225817269445*^9}, {3.4882258773424788`*^9, 
   3.488225892765924*^9}, 3.489179520151845*^9, {3.565278868517702*^9, 
   3.5652789346035547`*^9}, {3.565998835163805*^9, 3.565998835639832*^9}},
 FontFamily->"Times",
 FontSize->14],

Cell["\<\
This demonstrates the properties of the Ket and Bra states, which need to be \
activated on the palette in addition to the Fermi, Bose or other relevant \
operators.
We can perform the product directly. \
\>", "Text",
 CellChangeTimes->{{3.487690551218411*^9, 3.487690605374506*^9}, {
   3.487690849746355*^9, 3.487690891126054*^9}, 3.488227925249449*^9, {
   3.48917706286092*^9, 3.4891770643069687`*^9}, {3.5659988396330605`*^9, 
   3.565998840591115*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"H", "\[CircleTimes]", 
  RowBox[{
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"1", ",", " ", "1"}], "]"}], " ", "**", " ", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"2", ",", " ", 
     RowBox[{"-", "1"}]}], "]"}], " ", "**", " ", 
   RowBox[{"Ket", "[", "Vacuum", "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "2"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}]}]], "Output",
 CellChangeTimes->{3.565998853715866*^9, 3.648222191032524*^9}]
}, Open  ]],

Cell["\<\
We see that the destruction operators annihilate the vacuum state\
\>", "Text",
 CellChangeTimes->{{3.5652789925276127`*^9, 3.5652790128898277`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   RowBox[{"i", ",", "\[Sigma]"}], "]"}], "**", 
  RowBox[{"Ket", "[", "Vacuum", "]"}]}]], "Input",
 CellChangeTimes->{{3.565278663971428*^9, 3.56527871127732*^9}, 
   3.565278980767901*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.565278682897112*^9, 3.5652787123195333`*^9}, 
   3.565278802551951*^9, 3.565278981717791*^9, 3.5659988594961967`*^9, 
   3.6482221913215237`*^9, 3.6482223462138453`*^9}]
}, Open  ]],

Cell["\<\
We could use the ProductQ operator here, or equivalently use the symbol \
\[CircleTimes] between the operator and the ket.\
\>", "Text",
 CellChangeTimes->{{3.565279024542292*^9, 3.565279092440311*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ProductQ", "[", 
  RowBox[{"H", ",", " ", 
   RowBox[{"Ket", "[", "Vacuum", "]"}]}], "]"}]], "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.5662167192398806`*^9, 3.5662167313835754`*^9}, 
   3.6482221913751583`*^9, 3.6482223564647903`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Ket", "[", "1", "]"}], "=", " ", 
  RowBox[{
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"1", ",", " ", "1"}], "]"}], " ", "**", " ", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"2", ",", " ", 
     RowBox[{"-", "1"}]}], "]"}], " ", "**", " ", 
   RowBox[{"Ket", "[", "Vacuum", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.565279318755941*^9, 3.565279341770423*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"1", ",", "1"}], "]"}], "**", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"2", ",", 
    RowBox[{"-", "1"}]}], "]"}], "**", 
  TemplateBox[{"Vacuum"},
   "Ket"]}]], "Output",
 CellChangeTimes->{3.565279343505042*^9, 3.5659990127539625`*^9, 
  3.6482221914277773`*^9, 3.648222360938582*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ProductQ", "[", 
  RowBox[{"H", ",", 
   RowBox[{
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", " ", "1"}], "]"}], " ", "**", " ", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", " ", 
      RowBox[{"-", "1"}]}], "]"}], " ", "**", " ", 
    RowBox[{"Ket", "[", "Vacuum", "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "2"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}]}]], "Output",
 CellChangeTimes->{3.5659990465528955`*^9, 3.648222191504096*^9, 
  3.648222388647716*^9}]
}, Open  ]],

Cell["\<\
We thereby see that all destruction operators have been killed and the \
correct answer is obtained.
Clearly the same answer is found if we use Ket[1]\
\>", "Text",
 CellChangeTimes->{{3.487690619615499*^9, 3.487690629062038*^9}, {
   3.4876909155773277`*^9, 3.4876909265288353`*^9}, 3.48822607396999*^9, 
   3.6482225072479877`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ProductQ", "[", 
  RowBox[{"H", ",", 
   RowBox[{"Ket", "[", "1", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "2"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}]}]], "Output",
 CellChangeTimes->{3.5659990509121447`*^9, 3.648222191555106*^9, 
  3.648222332921792*^9, 3.648222388957694*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"H", "\[CircleTimes]", " ", 
  RowBox[{"Ket", "[", "1", "]"}]}]], "Input",
 CellChangeTimes->{{3.648222559470645*^9, 3.648222567681199*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "i"], 
   RowBox[{
    RowBox[{"t", "[", 
     RowBox[{"i", ",", "2"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}]}]], "Output",
 CellChangeTimes->{3.648222568586388*^9}]
}, Open  ]],

Cell["\<\
Here we see that the symbol \[CircleTimes] could be used in place of \
explicitly typing out ProductQ[H,Ket[1]]. 
 Trying the above problem w/o \[CircleTimes],\
\>", "Text",
 CellChangeTimes->{{3.4876912402441053`*^9, 3.487691258822884*^9}, 
   3.493488267929723*^9, 3.565279221230756*^9, {3.6482225217122183`*^9, 
   3.6482225274479628`*^9}, {3.648222589561528*^9, 3.6482226557720127`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"H", " ", "**", " ", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"1", ",", " ", "1"}], "]"}], " ", "**", " ", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"2", ",", " ", 
    RowBox[{"-", "1"}]}], "]"}], " ", "**", " ", 
  RowBox[{"Ket", "[", "Vacuum", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"U", " ", 
     RowBox[{
      UnderscriptBox["\[Sum]", "i"], 
      RowBox[{
       RowBox[{"f\[Dagger]", "[", 
        RowBox[{"i", ",", "1"}], "]"}], "**", 
       RowBox[{"f", "[", 
        RowBox[{"i", ",", "1"}], "]"}], "**", 
       RowBox[{"f\[Dagger]", "[", 
        RowBox[{"i", ",", 
         RowBox[{"-", "1"}]}], "]"}], "**", 
       RowBox[{"f", "[", 
        RowBox[{"i", ",", 
         RowBox[{"-", "1"}]}], "]"}]}]}]}], ")"}], "**", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "**", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"2", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   TemplateBox[{"Vacuum"},
    "Ket"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", 
     RowBox[{
      UnderscriptBox["\[Sum]", "i"], 
      RowBox[{
       UnderscriptBox["\[Sum]", "j"], 
       RowBox[{
        UnderscriptBox["\[Sum]", "\[Sigma]"], 
        RowBox[{
         RowBox[{
          RowBox[{"f\[Dagger]", "[", 
           RowBox[{"i", ",", "\[Sigma]"}], "]"}], "**", 
          RowBox[{"f", "[", 
           RowBox[{"j", ",", "\[Sigma]"}], "]"}]}], " ", 
         RowBox[{"t", "[", 
          RowBox[{"i", ",", "j"}], "]"}]}]}]}]}]}], ")"}], "**", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "**", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"2", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   TemplateBox[{"Vacuum"},
    "Ket"]}]}]], "Output",
 CellChangeTimes->{3.56599907075528*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "Note", " ", "that", " ", "we", " ", "would", " ", "not", " ", "get", " ",
      "the", " ", "correct", " ", "answer", " ", "if", " ", "we", " ", "just",
      " ", "used", " ", "NonCommutativeMultiply", " ", "instead", " ", 
     RowBox[{"of", " ", "\[CircleTimes]", 
      RowBox[{".", " ", "Where"}]}], " ", "it", " ", "is", " ", "appropriate",
      " ", "to", " ", "use", " ", "NCM", " ", 
     RowBox[{"(*", "*)"}], "rather", " ", "than", "  ", 
     RowBox[{"(", "\[CircleTimes]", ")"}], " ", "might", " ", "be", " ", 
     "confusing", " ", "initially"}], ",", " ", 
    RowBox[{
    "but", " ", "generally", " ", "it", " ", "is", " ", "easy", " ", "to", 
     " ", "deduce", " ", "which", " ", "one", " ", "is", " ", 
     RowBox[{"appropriate", ".", " ", "A"}], " ", "rule", " ", "of", " ", 
     "thumb", " ", "is", " ", "to", " ", "useNCM", " ", "in", " ", "between", 
     " ", "single", " ", "terms"}], ",", "  ", 
    RowBox[{"and", " ", "to", " ", 
     RowBox[{"use", " ", "\[CircleTimes]", "  ", "in"}], " ", "between", " ", 
     "larger", " ", "expression", " ", "involving", " ", "many", " ", 
     "terms"}], ",", " ", "Plus", " ", ",", " ", "Minus", ",", " ", 
    RowBox[{"or", " ", "Sum", " ", "functions"}]}], ")"}], "."}]], "Text",
 CellChangeTimes->{{3.487691006284635*^9, 3.4876912043765173`*^9}, {
   3.48822807403509*^9, 3.488228075154471*^9}, 3.4891706666928787`*^9, {
   3.493488267874476*^9, 3.493488267895692*^9}, {3.5652791331195107`*^9, 
   3.565279210419754*^9}, {3.565280021987823*^9, 3.565280032787656*^9}, {
   3.5659990809258614`*^9, 3.565999081738908*^9}, {3.5662167580421*^9, 
   3.5662167694287515`*^9}},
 FontFamily->"Times"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["(IV A) Hubbard Model k - space Moment Calculations", "Subsection",
 CellChangeTimes->{{3.490636316125078*^9, 3.4906363289785748`*^9}, {
   3.530412059286461*^9, 3.530412060861039*^9}, 3.565275151437326*^9, {
   3.565836434349271*^9, 3.565836435299198*^9}},
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "H", "]"}], ";", 
  RowBox[{
   RowBox[{"H", "[", 
    RowBox[{"l_", ",", "\[Sigma]_", ",", "p_", ",", "q_", ",", "r_"}], "]"}], 
   ":=", 
   RowBox[{
    RowBox[{
     UnderscriptBox["\[Sum]", "l"], 
     RowBox[{
      UnderscriptBox["\[Sum]", "\[Sigma]"], 
      RowBox[{
       RowBox[{"\[Xi]", "[", "l", "]"}], 
       RowBox[{
        RowBox[{"f\[Dagger]", "[", 
         RowBox[{"l", ",", "\[Sigma]"}], "]"}], "**", 
        RowBox[{"f", "[", 
         RowBox[{"l", ",", "\[Sigma]"}], "]"}]}]}]}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"U", "/", "Ns"}], ")"}], 
     RowBox[{
      UnderscriptBox["\[Sum]", "p"], 
      RowBox[{
       UnderscriptBox["\[Sum]", "q"], 
       RowBox[{
        UnderscriptBox["\[Sum]", "r"], 
        RowBox[{
         RowBox[{"f\[Dagger]", "[", 
          RowBox[{
           RowBox[{"p", "+", "q"}], ",", "1"}], "]"}], "**", 
         RowBox[{"f", "[", 
          RowBox[{"p", ",", "1"}], "]"}], "**", 
         RowBox[{"f\[Dagger]", "[", 
          RowBox[{
           RowBox[{"r", "-", "q"}], ",", 
           RowBox[{"-", "1"}]}], "]"}], "**", 
         RowBox[{"f", "[", 
          RowBox[{"r", ",", 
           RowBox[{"-", "1"}]}], "]"}]}]}]}]}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.565999303675602*^9, 3.565999305399701*^9}}],

Cell["\<\
This is the Hubbard Hamiltonian in k space, with Ns as the number of sites, \
and the labels p,q,l, r as the wavevectors. and \[Xi]=\[Epsilon](k)- \[Mu], \
with \[Mu] as the chemical potential and band dispersion \[Epsilon](k). The \
dummy variables summed over in the right hand side (p,q,r,l,\[Sigma])  are \
declared on the left hand side explicitly.

Throughout this notebook it is advisable to explicitly define the spin of the \
operators we are commuting with the Hamiltonian. This allows delta functions \
to be cancelled and thereby simplifies the result. Below we have computed the \
first  moment using both spin up and spin down operators (1 for spin up, -1 \
for spin down). Also remember we must use new variables every time we commute \
with another Hamiltonian.\
\>", "Text",
 CellChangeTimes->{{3.490557909044394*^9, 3.49055801902087*^9}, {
   3.490558105297234*^9, 3.490558158168373*^9}, {3.4905581961381407`*^9, 
   3.490558244399015*^9}, {3.565280085049972*^9, 3.5652801170114927`*^9}, 
   3.565280169962261*^9, {3.565280251590481*^9, 3.565280310425411*^9}, {
   3.565284077975807*^9, 3.565284176377542*^9}}],

Cell[TextData[{
 "We now compute the symmetric moments of the Hubbard model- these are \
defined\nas ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Omega]", "m"], "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "<", 
     RowBox[{"{", 
      RowBox[{"[", 
       RowBox[{"H", ",", 
        RowBox[{
         RowBox[{"[", 
          RowBox[{"H", ",", "...", 
           RowBox[{"[", 
            RowBox[{"H", ",", "f"}], "]"}], ",", "f\[Dagger]"}], "}"}], ">", 
         " "}]}]}]}]}]}], TraditionalForm]]],
 "with m nested commutators. Thus explicitly the first moment is the \
expectation: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Omega]", "1"], "(", "k", ")"}], "=", " ", 
    RowBox[{
     RowBox[{"(", "-", ")"}], " ", "<", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"[", 
        RowBox[{"H", ",", 
         RowBox[{"f", "(", 
          RowBox[{"k", ",", "\[Sigma]"}], ")"}]}], "]"}], ",", 
       RowBox[{"f\[Dagger]", "(", 
        RowBox[{"k", ",", "\[Sigma]"}], ")"}]}], "}"}], ">"}]}], 
   TraditionalForm]]],
 ". Below we work out the operators that emerge from this nested commutator \
and the final anticommutator. (Clearly the expectation of these requires the \
solution of the eigensystem that we do not address). \nA good reference for \
these moments is W. Nolting, Z. Physik 255, 25-- 39 (1972), his Eqs(8-11) \
correspond to our moments below after shifting ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Omega]", "m"], "=", 
    SuperscriptBox["M", 
     RowBox[{"(", 
      RowBox[{"m", "+", "1"}], ")"}]]}], TraditionalForm]]],
 ". "
}], "Text",
 CellChangeTimes->{{3.490557909044394*^9, 3.49055801902087*^9}, {
   3.490558105297234*^9, 3.490558158168373*^9}, {3.4905581961381407`*^9, 
   3.490558244399015*^9}, {3.565280085049972*^9, 3.5652801170114927`*^9}, 
   3.565280169962261*^9, {3.565280251590481*^9, 3.565280310425411*^9}, {
   3.5652839649129887`*^9, 3.56528406953773*^9}, {3.565284207413972*^9, 
   3.565284392802964*^9}, {3.565284677420977*^9, 3.5652847015303373`*^9}, {
   3.565284809918147*^9, 3.565284843148493*^9}, {3.56528494151797*^9, 
   3.565285013589649*^9}, {3.565285065076985*^9, 3.56528509062493*^9}, {
   3.565285525789749*^9, 3.5652855355224943`*^9}}],

Cell[CellGroupData[{

Cell["First Moment", "Subsubsection",
 CellChangeTimes->{{3.490473227402898*^9, 3.490473230632613*^9}},
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    RowBox[{"Commutator", "[", 
     RowBox[{
      RowBox[{"H", "[", 
       RowBox[{"l", ",", "\[Sigma]", ",", "c", ",", "q", ",", "r"}], "]"}], 
      ",", 
      RowBox[{"f", "[", 
       RowBox[{"k", ",", "1"}], "]"}]}], "]"}], ",", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"k", ",", "1"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   UnderscriptBox["\[Sum]", "r"], 
   RowBox[{
    FractionBox["U", "Ns"], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"r", ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}], "+", 
  RowBox[{"\[Xi]", "[", "k", "]"}]}]], "Output",
 CellChangeTimes->{3.565999350796297*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Second Moment", "Subsubsection",
 CellChangeTimes->{{3.490473237846386*^9, 3.490473240280992*^9}},
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiCommutator", "[", 
  RowBox[{
   RowBox[{"Commutator", "[", 
    RowBox[{
     RowBox[{"H", "[", 
      RowBox[{
       SubscriptBox["l", "1"], ",", 
       SubscriptBox["\[Sigma]", "1"], ",", "\[Chi]", ",", 
       SubscriptBox["\[Theta]", "1"], ",", 
       SubscriptBox["\[Rho]", "1"]}], "]"}], ",", 
     RowBox[{"Commutator", "[", 
      RowBox[{
       RowBox[{"H", "[", 
        RowBox[{"l", ",", "\[Sigma]", ",", "c", ",", "q", ",", "r"}], "]"}], 
       ",", 
       RowBox[{"f", "[", 
        RowBox[{"k", ",", "1"}], "]"}]}], "]"}]}], "]"}], ",", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"k", ",", "1"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   UnderscriptBox["\[Sum]", "r"], 
   RowBox[{
    FractionBox[
     RowBox[{"U", " ", 
      RowBox[{"\[Xi]", "[", "k", "]"}]}], "Ns"], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"r", ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", 
    SubscriptBox["\[Rho]", "1"]], 
   RowBox[{
    FractionBox[
     RowBox[{"U", " ", 
      RowBox[{"\[Xi]", "[", "k", "]"}]}], "Ns"], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{
      SubscriptBox["\[Rho]", "1"], ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", 
    SubscriptBox["\[Theta]", "1"]], 
   RowBox[{
    UnderscriptBox["\[Sum]", 
     SubscriptBox["\[Rho]", "1"]], 
    RowBox[{
     FractionBox[
      SuperscriptBox["U", "2"], 
      SuperscriptBox["Ns", "2"]], "**", 
     RowBox[{
      SubscriptBox["n", "f"], "[", 
      RowBox[{
       SubscriptBox["\[Rho]", "1"], ",", 
       RowBox[{"-", "1"}]}], "]"}]}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "r"], 
   RowBox[{
    UnderscriptBox["\[Sum]", 
     SubscriptBox["\[Theta]", "1"]], 
    RowBox[{
     UnderscriptBox["\[Sum]", 
      SubscriptBox["\[Rho]", "1"]], 
     RowBox[{
      FractionBox[
       SuperscriptBox["U", "2"], 
       SuperscriptBox["Ns", "2"]], "**", 
      RowBox[{"f\[Dagger]", "[", 
       RowBox[{
        RowBox[{"r", "+", 
         SubscriptBox["\[Theta]", "1"]}], ",", 
        RowBox[{"-", "1"}]}], "]"}], "**", 
      RowBox[{"f\[Dagger]", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["\[Theta]", "1"]}], "+", 
         SubscriptBox["\[Rho]", "1"]}], ",", 
        RowBox[{"-", "1"}]}], "]"}], "**", 
      RowBox[{"f", "[", 
       RowBox[{"r", ",", 
        RowBox[{"-", "1"}]}], "]"}], "**", 
      RowBox[{"f", "[", 
       RowBox[{
        SubscriptBox["\[Rho]", "1"], ",", 
        RowBox[{"-", "1"}]}], "]"}]}]}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", "r"], 
   RowBox[{
    UnderscriptBox["\[Sum]", 
     SubscriptBox["\[Theta]", "1"]], 
    RowBox[{
     UnderscriptBox["\[Sum]", 
      SubscriptBox["\[Rho]", "1"]], 
     RowBox[{
      FractionBox[
       SuperscriptBox["U", "2"], 
       SuperscriptBox["Ns", "2"]], "**", 
      RowBox[{"f\[Dagger]", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["\[Theta]", "1"]}], "+", 
         SubscriptBox["\[Rho]", "1"]}], ",", 
        RowBox[{"-", "1"}]}], "]"}], "**", 
      RowBox[{"f\[Dagger]", "[", 
       RowBox[{
        RowBox[{"k", "+", 
         SubscriptBox["\[Theta]", "1"]}], ",", "1"}], "]"}], "**", 
      RowBox[{"f", "[", 
       RowBox[{"r", ",", 
        RowBox[{"-", "1"}]}], "]"}], "**", 
      RowBox[{"f", "[", 
       RowBox[{
        RowBox[{"k", "-", "r", "+", 
         SubscriptBox["\[Rho]", "1"]}], ",", "1"}], "]"}]}]}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", "q"], 
   RowBox[{
    UnderscriptBox["\[Sum]", 
     SubscriptBox["\[Theta]", "1"]], 
    RowBox[{
     UnderscriptBox["\[Sum]", 
      SubscriptBox["\[Rho]", "1"]], 
     RowBox[{
      FractionBox[
       SuperscriptBox["U", "2"], 
       SuperscriptBox["Ns", "2"]], "**", 
      RowBox[{"f\[Dagger]", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "q"}], "-", 
         SubscriptBox["\[Theta]", "1"], "+", 
         SubscriptBox["\[Rho]", "1"]}], ",", 
        RowBox[{"-", "1"}]}], "]"}], "**", 
      RowBox[{"f\[Dagger]", "[", 
       RowBox[{
        RowBox[{"k", "+", 
         SubscriptBox["\[Theta]", "1"]}], ",", "1"}], "]"}], "**", 
      RowBox[{"f", "[", 
       RowBox[{
        SubscriptBox["\[Rho]", "1"], ",", 
        RowBox[{"-", "1"}]}], "]"}], "**", 
      RowBox[{"f", "[", 
       RowBox[{
        RowBox[{"k", "-", "q"}], ",", "1"}], "]"}]}]}]}]}], "+", 
  SuperscriptBox[
   RowBox[{"\[Xi]", "[", "k", "]"}], "2"]}]], "Output",
 CellChangeTimes->{3.565999360430848*^9}]
}, Closed]],

Cell["\<\
Clearly a relabeling of variables would help combine terms here; it is left \
as an exercise for the reader. \
\>", "Text",
 CellChangeTimes->{{3.565285352959876*^9, 3.565285399309161*^9}, {
   3.565285450550474*^9, 3.565285490509974*^9}, 3.5660456864608994`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Third Moment", "Subsubsection",
 CellChangeTimes->{{3.490549985444573*^9, 3.490549990178722*^9}},
 FontSize->14],

Cell["\<\
The third moment can be calculated, but it requires further simplification. \
Due to the length of the result it is not shown below. Remove the semicolon \
at the end to see the large output. This calculation is long, and it takes \
around ten seconds to complete.\
\>", "Text",
 CellChangeTimes->{{3.5660456893930674`*^9, 3.5660457482974367`*^9}, {
  3.5660457807902946`*^9, 3.5660458016994905`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"AntiCommutator", "[", 
    RowBox[{
     RowBox[{"Commutator", "[", 
      RowBox[{
       RowBox[{"H", "[", 
        RowBox[{
        "\[Lambda]", ",", "\[SZ]", ",", "\[Zeta]", ",", "\[Omega]", ",", 
         "\[Iota]"}], "]"}], ",", 
       RowBox[{"Commutator", "[", 
        RowBox[{
         RowBox[{"H", "[", 
          RowBox[{
           SubscriptBox["l", "1"], ",", 
           SubscriptBox["\[Sigma]", "1"], ",", "\[Chi]", ",", 
           SubscriptBox["\[Theta]", "1"], ",", 
           SubscriptBox["\[Rho]", "1"]}], "]"}], ",", 
         RowBox[{"Commutator", "[", 
          RowBox[{
           RowBox[{"H", "[", 
            RowBox[{"l", ",", "\[Sigma]", ",", "p", ",", "q", ",", "r"}], 
            "]"}], ",", 
           RowBox[{"f", "[", 
            RowBox[{"k", ",", "1"}], "]"}]}], "]"}]}], "]"}]}], "]"}], ",", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"k", ",", "1"}], "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.5660083204339685`*^9, {3.5660083608762817`*^9, 3.5660083657355595`*^9}, {
   3.5660084049007998`*^9, 3.566008407144928*^9}, 3.5660422724176273`*^9, {
   3.5660455997559404`*^9, 3.56604561758496*^9}, {3.5660456563731785`*^9, 
   3.566045667136794*^9}, {3.566045764929388*^9, 3.566045768939617*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["(IV B) Hubbard Model r - space Moment Calculations ", "Subsection",
 CellChangeTimes->{{3.490636316125078*^9, 3.4906363289785748`*^9}, {
   3.490733848474121*^9, 3.4907338577699347`*^9}, {3.5304120635953283`*^9, 
   3.530412065440935*^9}, 3.565275156350396*^9, {3.565642336336234*^9, 
   3.565642342332491*^9}, {3.565835929272135*^9, 3.565835931170238*^9}, {
   3.565836428036236*^9, 3.5658364305862083`*^9}},
 FontFamily->"Times"],

Cell[TextData[{
 StyleBox["Here the same calculation as above can be done in real \
space-finally taking Fourier transforms over the two fixed site indices r1,r2 \
with wave vector k gives the moments ",
  FontWeight->"Plain"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["\[Omega]", "m"], "(", "k", ")"}], ".", " ", "The"}], " ",
      "results", " ", "are", " ", "equivalent", " ", "to", " ", "those", " ", 
     "in", "  ", "k", " ", "space", " ", "shown", " ", 
     RowBox[{"above", ".", " ", "It"}], " ", "is", " ", "interesting", " ", 
     "to", " ", "see", " ", "these", " ", "since", " ", "the", " ", "Pauli", 
     " ", "principle", " ", "identities", " ", "such", " ", "as", " ", 
     RowBox[{"n", "**", "n"}]}], "=", 
    RowBox[{
    "n", " ", "are", " ", "utilized", " ", "here", " ", "and", " ", "we", " ",
      "can", " ", "finally", "  ", "take", "  ", "Fourier", " ", "transforms",
      " ", "and", " ", "sum", " ", "over", " ", "the", " ", "fixed", " ", 
     "external", " ", "sites", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"r1", ",", "r2"}], " ", ")"}], "  ", "."}]}]}], 
   TraditionalForm]],
  FontWeight->"Plain"],
 StyleBox["  ",
  FontWeight->"Plain"]
}], "Text",
 CellChangeTimes->{{3.5652855919486*^9, 3.565285688479673*^9}, {
  3.565285826812307*^9, 3.565285928306389*^9}, {3.565836113266986*^9, 
  3.5658362467384357`*^9}},
 FontWeight->"Bold"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "H", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"H", "[", "j_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "*", 
    RowBox[{
     UnderscriptBox["\[Sum]", 
      RowBox[{"k", "[", "j", "]"}]], 
     RowBox[{
      UnderscriptBox["\[Sum]", 
       RowBox[{"m", "[", "j", "]"}]], " ", 
      RowBox[{
       UnderscriptBox["\[Sum]", 
        RowBox[{"u", "[", "j", "]"}]], " ", 
       RowBox[{
        SubscriptBox["t", 
         RowBox[{
          RowBox[{"k", "[", "j", "]"}], ",", 
          RowBox[{"m", "[", "j", "]"}]}]], 
        RowBox[{
         RowBox[{"f\[Dagger]", "[", 
          RowBox[{
           RowBox[{"k", "[", "j", "]"}], ",", 
           RowBox[{"u", "[", "j", "]"}]}], "]"}], "**", 
         RowBox[{"f", "[", 
          RowBox[{
           RowBox[{"m", "[", "j", "]"}], ",", 
           RowBox[{"u", "[", "j", "]"}]}], "]"}]}]}]}]}]}]}], "-", " ", 
   RowBox[{"\[Mu]", " ", 
    RowBox[{
     UnderscriptBox["\[Sum]", 
      RowBox[{"s", "[", "j", "]"}]], 
     RowBox[{
      UnderscriptBox["\[Sum]", 
       RowBox[{"v", "[", "j", "]"}]], 
      RowBox[{
       RowBox[{"f\[Dagger]", "[", 
        RowBox[{
         RowBox[{"s", "[", "j", "]"}], ",", 
         RowBox[{"v", "[", "j", "]"}]}], "]"}], "**", 
       RowBox[{"f", "[", 
        RowBox[{
         RowBox[{"s", "[", "j", "]"}], ",", 
         RowBox[{"v", "[", "j", "]"}]}], "]"}]}]}]}]}], "-", 
   RowBox[{"U", 
    RowBox[{
     UnderscriptBox["\[Sum]", 
      RowBox[{"t", "[", "j", "]"}]], 
     RowBox[{
      RowBox[{
       SubscriptBox["n", "f"], "[", 
       RowBox[{
        RowBox[{"t", "[", "j", "]"}], ",", "1"}], "]"}], "**", 
      RowBox[{
       SubscriptBox["n", "f"], "[", 
       RowBox[{
        RowBox[{"t", "[", "j", "]"}], ",", 
        RowBox[{"-", "1"}]}], "]"}]}]}]}]}]}]}], "Input"],

Cell[CellGroupData[{

Cell["First Moment", "Subsubsection",
 CellChangeTimes->{{3.490473227402898*^9, 3.490473230632613*^9}},
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    RowBox[{"Commutator", "[", 
     RowBox[{
      RowBox[{"H", "[", "1", "]"}], ",", 
      RowBox[{"f", "[", 
       RowBox[{"r1", ",", "1"}], "]"}]}], "]"}], ",", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"r2", ",", "1"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"U", " ", 
      RowBox[{"\[Delta]", "[", 
       RowBox[{"r1", ",", "r2"}], "]"}]}], ")"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"r1", ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}], "-", 
  SubscriptBox["t", 
   RowBox[{"r1", ",", "r2"}]], "-", 
  RowBox[{"\[Mu]", " ", 
   RowBox[{"\[Delta]", "[", 
    RowBox[{"r1", ",", "r2"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.566045856567629*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Second Moment ", "Subsubsection",
 CellChangeTimes->{{3.490473237846386*^9, 3.490473240280992*^9}, {
   3.5656423112689123`*^9, 3.565642328470355*^9}, 3.56700813468815*^9},
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiCommutator", "[", 
  RowBox[{
   RowBox[{"Fold", "[", 
    RowBox[{"Commutator", ",", 
     RowBox[{"f", "[", 
      RowBox[{"r1", ",", "1"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"H", "[", "1", "]"}], ",", 
       RowBox[{"H", "[", "2", "]"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"f\[Dagger]", "[", 
    RowBox[{"r2", ",", "1"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"U", " ", 
     SubscriptBox["t", 
      RowBox[{"r1", ",", "r2"}]]}], ")"}], "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"r1", ",", 
     RowBox[{"-", "1"}]}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"U", " ", 
     SubscriptBox["t", 
      RowBox[{"r1", ",", "r2"}]]}], ")"}], "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"r2", ",", 
     RowBox[{"-", "1"}]}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["U", "2"], " ", 
     RowBox[{"\[Delta]", "[", 
      RowBox[{"r1", ",", "r2"}], "]"}]}], ")"}], "**", 
   RowBox[{
    SubscriptBox["n", "f"], "[", 
    RowBox[{"r1", ",", 
     RowBox[{"-", "1"}]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"U", " ", "\[Mu]", " ", 
      RowBox[{"\[Delta]", "[", 
       RowBox[{"r1", ",", "r2"}], "]"}]}], ")"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"r1", ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}], "+", 
  RowBox[{"2", " ", "\[Mu]", " ", 
   SubscriptBox["t", 
    RowBox[{"r1", ",", "r2"}]]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", 
    RowBox[{"m", "[", "2", "]"}]], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"U", " ", 
      SubscriptBox["t", 
       RowBox[{"r1", ",", 
        RowBox[{"m", "[", "2", "]"}]}]], " ", 
      RowBox[{"\[Delta]", "[", 
       RowBox[{"r1", ",", "r2"}], "]"}]}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"r1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{
      RowBox[{"m", "[", "2", "]"}], ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}], "-", 
  RowBox[{
   UnderscriptBox["\[Sum]", 
    RowBox[{"k", "[", "2", "]"}]], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"U", " ", 
      SubscriptBox["t", 
       RowBox[{
        RowBox[{"k", "[", "2", "]"}], ",", "r1"}]], " ", 
      RowBox[{"\[Delta]", "[", 
       RowBox[{"r1", ",", "r2"}], "]"}]}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{
      RowBox[{"k", "[", "2", "]"}], ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"r1", ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}], "+", 
  RowBox[{
   UnderscriptBox["\[Sum]", 
    RowBox[{"m", "[", "1", "]"}]], 
   RowBox[{
    SubscriptBox["t", 
     RowBox[{"r1", ",", 
      RowBox[{"m", "[", "1", "]"}]}]], " ", 
    SubscriptBox["t", 
     RowBox[{
      RowBox[{"m", "[", "1", "]"}], ",", "r2"}]]}]}], "+", 
  RowBox[{
   SuperscriptBox["\[Mu]", "2"], " ", 
   RowBox[{"\[Delta]", "[", 
    RowBox[{"r1", ",", "r2"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.566045871577488*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Third Moment", "Subsubsection",
 CellChangeTimes->{{3.490473237846386*^9, 3.490473240280992*^9}, {
  3.490734933800988*^9, 3.490734936838067*^9}},
 FontSize->14],

Cell["(* Warning: Produces a large output *)", "Text",
 CellChangeTimes->{{3.5660459060684605`*^9, 3.56604591951923*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"AntiCommutator", "[", 
    RowBox[{
     RowBox[{"Fold", "[", 
      RowBox[{"Commutator", ",", 
       RowBox[{"f", "[", 
        RowBox[{"r1", ",", "1"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"H", "[", "1", "]"}], ",", 
         RowBox[{"H", "[", "2", "]"}], ",", 
         RowBox[{"H", "[", "3", "]"}]}], "}"}]}], "]"}], ",", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"r2", ",", "1"}], "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{3.566045887789415*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
(V)  Spins at work: Eight Vertex Model Yang - Baxter Equation\
\>", "Subsection",
 CellChangeTimes->{{3.488217865420987*^9, 3.4882178789210777`*^9}, 
   3.488220883050235*^9, {3.490453574610146*^9, 3.490453583101441*^9}, {
   3.499972826818653*^9, 3.499972846426502*^9}, 3.499972952887064*^9, {
   3.530412068377191*^9, 3.530412070738722*^9}, 3.565275159774921*^9, 
   3.565836424436309*^9, 3.566045942969571*^9},
 FontFamily->"Times"],

Cell["Activate the Pauli matrix symbol \"\[Sigma]\"", "Item",
 CellChangeTimes->{{3.4929686704746027`*^9, 3.492968699776744*^9}},
 FontSize->14,
 FontWeight->"Bold"],

Cell["\<\
   We use DiracQ to simplify some algebra that arises in  the well known work \
of Baxter on the two dimensional 8 vertex model (R. J. Baxter, Ann. Phys. vol \
76, p1 (1973)) .  The transfer matrix of the model with a parameter u is \
written as: \
\>", "Text",
 CellChangeTimes->{{3.4834614814375*^9, 3.483461699609375*^9}, 
   3.483461928421875*^9, 3.488220883062964*^9, {3.4891789184652243`*^9, 
   3.489178928580625*^9}, {3.489179304353652*^9, 3.489179306601502*^9}, {
   3.565643530875175*^9, 3.5656435999722137`*^9}, {3.5656444753756123`*^9, 
   3.565644552583987*^9}},
 LineSpacing->{1.5, 3}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"T", 
    RowBox[{"(", "u", ")"}]}], "=", 
   RowBox[{
    UnderscriptBox["Tr", "g"], "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["L", 
        RowBox[{"N", ",", "g"}]], 
       RowBox[{"(", "u", ")"}], " ", 
       SubscriptBox["L", 
        RowBox[{
         RowBox[{"N", "-", "1"}], ",", "g"}]], 
       RowBox[{"(", "u", ")"}]}], "..."}], 
     SubscriptBox["L", 
      RowBox[{"1", ",", "g"}]], 
     RowBox[{"(", "u", ")"}]}], "]"}]}], " ", ","}]], "DisplayFormulaNumbered",\

 CellChangeTimes->{{3.48087219565625*^9, 3.48087226559375*^9}, {
   3.480872909875*^9, 3.480872910734375*^9}, {3.480873165296875*^9, 
   3.480873169609375*^9}, {3.480874312625*^9, 3.480874313609375*^9}, 
   3.481480744078125*^9, 3.483461928421875*^9, 3.488220883063929*^9, {
   3.490546998172565*^9, 3.490546999679702*^9}, {3.565644559649129*^9, 
   3.5656445725497437`*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell[TextData[{
 "where each of the objects ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["L", 
       RowBox[{"n", ",", "g"}]], "(", "u", ")"}], " ", "is", "  ", "a", "  ", 
     "scattering", " ", "matrix"}], " ", "\[IndentingNewLine]"}], 
   TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.4834620271875*^9, 3.483462150609375*^9}, 
   3.488220883064567*^9, {3.489178920619316*^9, 3.4891789363017483`*^9}, {
   3.5656445790849733`*^9, 3.56564469734898*^9}, {3.565644732606492*^9, 
   3.565644759358626*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["L", 
    RowBox[{"n", ",", "g"}]], 
   RowBox[{"(", "u", ")"}]}], "=", 
  RowBox[{
   FractionBox[
    RowBox[{"a", "+", "b"}], "2"], "+", 
   RowBox[{
    FractionBox[
     RowBox[{"a", "-", "b"}], "2"], " ", 
    SubsuperscriptBox["\[Sigma]", "n", "z"], 
    SubsuperscriptBox["\[Sigma]", "g", "z"]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{"c", "+", "d"}], "2"], 
    SubsuperscriptBox["\[Sigma]", "n", "x"], 
    SubsuperscriptBox["\[Sigma]", "g", "x"]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{"c", "-", "d"}], "2"], 
    SubsuperscriptBox["\[Sigma]", "n", "y"], 
    RowBox[{
     SubsuperscriptBox["\[Sigma]", "g", "y"], 
     "."}]}]}]}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.48381038815625*^9, 3.483810452125*^9}, 
   3.48381053290625*^9, 3.488220883068331*^9, {3.565645210357758*^9, 
   3.565645211256579*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell[TextData[{
 "  Here the Pauli matrices are defined for sites n=1,N and an extra \
\[OpenCurlyDoubleQuote]ghost\[CloseCurlyDoubleQuote] site \
\[OpenCurlyDoubleQuote]g\[CloseCurlyDoubleQuote]. The Boltzmann weights a, b, \
c, and d are functions of the spectral parameter u. The fundamental \
commutation relation of the transfer matrices [T(u),T(v)]=0 defines the \
integrability of this model and also is the key to its solution. Baxter shows \
that this commuting transfer matrix relation is satisfied if the local \
relation\n",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"n", ",", 
      SubscriptBox["g", "2"]}]], "[", "u", "]"}], "\[CircleTimes]", 
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"n", ",", 
      SubscriptBox["g", "1"]}]], "[", 
    RowBox[{"u", "'"}], "]"}], "\[CircleTimes]", 
   RowBox[{
    SubscriptBox["R", 
     RowBox[{
      SubscriptBox["g", "1"], ",", 
      SubscriptBox["g", "2"]}]], "[", 
    RowBox[{"u", "''"}], "]"}]}]],
  CellChangeTimes->{{3.48131508190625*^9, 3.481315110828125*^9}, 
    3.481315178375*^9, {3.481464232578125*^9, 3.48146423375*^9}, {
    3.48146481253125*^9, 3.48146481321875*^9}, 3.48148074409375*^9, {
    3.48149059275*^9, 3.481490648734375*^9}, {3.4814907199375*^9, 
    3.48149072078125*^9}, 3.481490803953125*^9, {3.481490982640625*^9, 
    3.48149100590625*^9}, {3.481491046*^9, 3.481491049640625*^9}, {
    3.48149158234375*^9, 3.48149159790625*^9}, {3.481492437359375*^9, 
    3.48149246359375*^9}, {3.48164799421875*^9, 3.481647998796875*^9}, 
    3.4834619284375*^9, {3.48382326021875*^9, 3.483823262515625*^9}, {
    3.4838249889375*^9, 3.483824991828125*^9}, 3.483825467078125*^9, {
    3.48433555446875*^9, 3.484335567484375*^9}, 3.488220883114931*^9, {
    3.488220913238639*^9, 3.488220935820991*^9}},
  FontFamily->"Times",
  FontSize->14],
 "= ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox["R", 
     RowBox[{
      SubscriptBox["g", "1"], ",", 
      SubscriptBox["g", "2"]}]], "[", 
    RowBox[{"u", "''"}], "]"}], "\[CircleTimes]", 
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"n", ",", 
      SubscriptBox["g", "1"]}]], "[", 
    RowBox[{"u", "'"}], "]"}], "\[CircleTimes]", 
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"n", ",", 
      SubscriptBox["g", "2"]}]], "[", "u", "]"}]}]],
  CellChangeTimes->{{3.481315113609375*^9, 3.48131514640625*^9}, 
    3.481315209140625*^9, {3.481464452078125*^9, 3.481464453515625*^9}, 
    3.481480744109375*^9, 3.481491067125*^9, {3.481491629109375*^9, 
    3.481491643*^9}, {3.481492498671875*^9, 3.4814925239375*^9}, 
    3.4814925773125*^9, 3.48346192846875*^9, 3.483825473484375*^9, {
    3.484335569671875*^9, 3.48433558253125*^9}, 3.488220883117433*^9, {
    3.488220943950284*^9, 3.488221008660214*^9}},
  FontFamily->"Times",
  FontSize->14],
 " (Baxter\[CloseCurlyQuote]s equation)\nis satisfied, where explicitly"
}], "Text",
 CellChangeTimes->{{3.483811067078125*^9, 3.483811073765625*^9}, {
   3.48381895421875*^9, 3.483819146359375*^9}, {3.483819181015625*^9, 
   3.483819226375*^9}, {3.48433524628125*^9, 3.48433525428125*^9}, 
   3.48822088306855*^9, 3.4891789444743967`*^9, {3.489179339129788*^9, 
   3.489179347105639*^9}, {3.5656447706624823`*^9, 3.565645180666456*^9}, {
   3.565645216538967*^9, 3.565645271118154*^9}, {3.565645430960746*^9, 
   3.565645441557267*^9}, {3.5657036569975147`*^9, 3.565703666109322*^9}, {
   3.565703765291155*^9, 3.565703766837775*^9}, {3.5662168411688547`*^9, 
   3.566216856523733*^9}},
 LineSpacing->{1.5, 3}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["L", 
    RowBox[{"n", ",", 
     SubscriptBox["g", "2"]}]], 
   RowBox[{"(", "u", ")"}]}], "=", 
  RowBox[{
   FractionBox[
    RowBox[{"a", "+", "b"}], "2"], "+", 
   RowBox[{
    FractionBox[
     RowBox[{"a", "-", "b"}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "2"], "z"], 
    SubsuperscriptBox["\[Sigma]", "n", "z"]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{"c", "+", "d"}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "2"], "x"], 
    SubsuperscriptBox["\[Sigma]", "n", "x"]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{"c", "-", "d"}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "2"], "y"], 
    SubsuperscriptBox["\[Sigma]", "n", "y"]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["L", 
    RowBox[{"n", ",", 
     SubscriptBox["g", "1"]}]], 
   RowBox[{"(", 
    RowBox[{"u", "'"}], ")"}]}], "=", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"a", "'"}], "+", 
     RowBox[{"b", "'"}]}], "2"], "+", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"a", "'"}], "-", 
      RowBox[{"b", "'"}]}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "1"], "z"], 
    SubsuperscriptBox["\[Sigma]", "n", "z"]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"c", "'"}], "+", 
      RowBox[{"d", "'"}]}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "1"], "x"], 
    SubsuperscriptBox["\[Sigma]", "n", "x"]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"c", "'"}], "-", 
      RowBox[{"d", "'"}]}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "1"], "y"], 
    SubsuperscriptBox["\[Sigma]", "n", "y"]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["R", 
    RowBox[{
     SubscriptBox["g", "2"], ",", 
     SubscriptBox["g", "1"]}]], 
   RowBox[{"(", 
    RowBox[{"u", "''"}], ")"}]}], "=", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"a", "''"}], "+", 
     RowBox[{"b", "''"}]}], "2"], "+", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"a", "''"}], "-", 
      RowBox[{"b", "''"}]}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "1"], "z"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "2"], "z"]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"c", "''"}], "+", 
      RowBox[{"d", "''"}]}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "1"], "x"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "2"], "x"]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"c", "''"}], "-", 
      RowBox[{"d", "''"}]}], "2"], 
    SubsuperscriptBox["\[Sigma]", 
     SubscriptBox["g", "1"], "y"], 
    RowBox[{
     SubsuperscriptBox["\[Sigma]", 
      RowBox[{
       SubscriptBox["g", "2"], " "}], "y"], 
     "."}]}]}]}]}], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.480877655453125*^9, 3.48087788103125*^9}, {
   3.48087815284375*^9, 3.48087816834375*^9}, {3.480878583171875*^9, 
   3.4808786910625*^9}, 3.48148074409375*^9, {3.481491220203125*^9, 
   3.4814912765625*^9}, {3.481648220296875*^9, 3.4816482505*^9}, 
   3.4834619284375*^9, {3.483822537265625*^9, 3.4838225415*^9}, {
   3.483825338765625*^9, 3.483825352671875*^9}, 3.4882208830782757`*^9, {
   3.5656452769390707`*^9, 3.565645298646356*^9}, {3.5657036719519587`*^9, 
   3.565703672564575*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell[TextData[{
 "\n    Let us use the functions of the DiracQ package to see if we can find \
the requirements on the u's such that (10) is satisfied. For convenience let \
us set the three distinct sites in the problem explicitly as:",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{Cell[TextData[{
     "n=3, ",
     Cell[BoxData[
      FormBox[
       RowBox[{
        SubscriptBox["g", "2"], "=", "1"}], TraditionalForm]]],
     ", ",
     Cell[BoxData[
      FormBox[
       RowBox[{
        SubscriptBox["g", "1"], "=", "2"}], TraditionalForm]]]
    }]], "."}]}]]],
 " This allows the code to automatically solve the Kronecker \[Delta] \
functions, as shown below. For more information on use of the Kronecker \
\[Delta] see the Tutorial."
}], "Text",
 CellChangeTimes->{{3.480878731859375*^9, 3.48087878678125*^9}, 
   3.480878844890625*^9, {3.480879001125*^9, 3.48087902034375*^9}, {
   3.481464277546875*^9, 3.481464306671875*^9}, {3.481464371140625*^9, 
   3.481464436421875*^9}, 3.48148074409375*^9, 3.4834619284375*^9, {
   3.483822547046875*^9, 3.483822719203125*^9}, {3.48433546190625*^9, 
   3.484335486296875*^9}, 3.4882208830792627`*^9, {3.489178900807693*^9, 
   3.4891789069015923`*^9}, 3.4891789498238163`*^9, {3.489179393250736*^9, 
   3.489179416754603*^9}, {3.489179758734226*^9, 3.489179783453691*^9}, {
   3.489243667625464*^9, 3.489243686449993*^9}, {3.490547039861609*^9, 
   3.490547041800569*^9}, {3.5656453093843184`*^9, 3.565645390896338*^9}, {
   3.5660459882821627`*^9, 3.5660459901402693`*^9}, {3.566216865526248*^9, 
   3.566216874230746*^9}},
 LineSpacing->{1.5, 3}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]", "[", 
  RowBox[{
   SubscriptBox["g", "1"], ",", 
   SubscriptBox["g", "2"]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"\[Delta]", "[", 
  RowBox[{
   SubscriptBox["g", "1"], ",", 
   SubscriptBox["g", "2"]}], "]"}]], "Output",
 CellChangeTimes->{3.5660459978787117`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]", "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5660460047891073`*^9}]
}, Closed]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"n", "=", "3"}], ";", 
  RowBox[{
   SubscriptBox["g", "2"], "=", "1"}], ";", 
  RowBox[{
   SubscriptBox["g", "1"], "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"n", ",", 
      SubscriptBox["g", "2"]}]], "[", "W", "]"}], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"a", "+", "b"}], "2"], "+", 
    RowBox[{
     FractionBox[
      RowBox[{"a", "-", "b"}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "2"], ",", "z"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{"n", ",", "z"}], "]"}]}]}], "+", 
    RowBox[{
     FractionBox[
      RowBox[{"c", "+", "d"}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "2"], ",", "x"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{"n", ",", "x"}], "]"}]}]}], "+", 
    RowBox[{
     FractionBox[
      RowBox[{"c", "-", "d"}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "2"], ",", "y"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{"n", ",", "y"}], "]"}]}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"n", ",", 
      SubscriptBox["g", "1"]}]], "[", 
    RowBox[{"W", "'"}], "]"}], "=", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"a", "'"}], "+", 
      RowBox[{"b", "'"}]}], "2"], "+", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"a", "'"}], "-", 
       RowBox[{"b", "'"}]}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "1"], ",", "z"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{"n", ",", "z"}], "]"}]}]}], "+", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"c", "'"}], "+", 
       RowBox[{"d", "'"}]}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "1"], ",", "x"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{"n", ",", "x"}], "]"}]}]}], "+", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"c", "'"}], "-", 
       RowBox[{"d", "'"}]}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "1"], ",", "y"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{"n", ",", "y"}], "]"}]}]}]}]}], ";", 
  RowBox[{
   RowBox[{
    SubscriptBox["R", 
     RowBox[{
      SubscriptBox["g", "1"], ",", 
      SubscriptBox["g", "2"]}]], "[", 
    RowBox[{"W", "''"}], "]"}], "=", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"a", "''"}], "+", 
      RowBox[{"b", "''"}]}], "2"], "+", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"a", "''"}], "-", 
       RowBox[{"b", "''"}]}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "1"], ",", "z"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "2"], ",", "z"}], "]"}]}]}], "+", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"c", "''"}], "+", 
       RowBox[{"d", "''"}]}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "1"], ",", "x"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "2"], ",", "x"}], "]"}]}]}], "+", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"c", "''"}], "-", 
       RowBox[{"d", "''"}]}], "2"], 
     RowBox[{
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "1"], ",", "y"}], "]"}], "**", 
      RowBox[{"\[Sigma]", "[", 
       RowBox[{
        SubscriptBox["g", "2"], ",", "y"}], "]"}]}]}]}]}], ";"}]}], "Input"],

Cell["\<\
Let us first compute the product of the LHS Baxter\[CloseCurlyQuote]s \
equation. For this we will use the function, which will apply the algebra of \
Pauli matrices and collect c terms. Because there are 8 terms in each \
scattering matrix the number of terms on both the right and left hand sides \
of equation is large (128 terms in all). Because of this we will suppress the \
output.\
\>", "Text",
 CellChangeTimes->{{3.481482330578125*^9, 3.48148235084375*^9}, {
   3.481482567328125*^9, 3.481482687890625*^9}, {3.48148289028125*^9, 
   3.4814828924375*^9}, 3.4834619284375*^9, {3.483822792375*^9, 
   3.483822807875*^9}, {3.483825011796875*^9, 3.483825032234375*^9}, {
   3.483825068484375*^9, 3.48382510815625*^9}, {3.4838253625*^9, 
   3.483825452953125*^9}, 3.4882208830951977`*^9, {3.4891654102279453`*^9, 
   3.4891654204887123`*^9}, {3.489176847395503*^9, 3.489176882660986*^9}, {
   3.4891797903239517`*^9, 3.489179791467649*^9}, 3.493488267677726*^9, {
   3.565645455497027*^9, 3.5656454674704723`*^9}, {3.565645499636696*^9, 
   3.565645516434991*^9}, {3.565703797926659*^9, 3.565703830798236*^9}, {
   3.566216885025363*^9, 3.5662168944189005`*^9}},
 LineSpacing->{1.5, 3}],

Cell[BoxData[
 RowBox[{
  RowBox[{"LHS", "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["L", 
      RowBox[{"n", ",", 
       SubscriptBox["g", "2"]}]], "[", "W", "]"}], "\[CircleTimes]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["L", 
        RowBox[{"n", ",", 
         SubscriptBox["g", "1"]}]], "[", 
       RowBox[{"W", "'"}], "]"}], "\[CircleTimes]", 
      RowBox[{
       SubscriptBox["R", 
        RowBox[{
         SubscriptBox["g", "1"], ",", 
         SubscriptBox["g", "2"]}]], "[", 
       RowBox[{"W", "''"}], "]"}]}], ")"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5660460308385973`*^9, 3.566046031744649*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"RHS", "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["R", 
      RowBox[{
       SubscriptBox["g", "1"], ",", 
       SubscriptBox["g", "2"]}]], "[", 
     RowBox[{"W", "''"}], "]"}], "\[CircleTimes]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["L", 
        RowBox[{"n", ",", 
         SubscriptBox["g", "1"]}]], "[", 
       RowBox[{"W", "'"}], "]"}], "\[CircleTimes]", 
      RowBox[{
       SubscriptBox["L", 
        RowBox[{"n", ",", 
         SubscriptBox["g", "2"]}]], "[", "W", "]"}]}], ")"}]}]}], 
  ";"}]], "Input",
 CellChangeTimes->{3.566046048597613*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Length", "[", "LHS", "]"}], ",", 
   RowBox[{"Length", "[", "RHS", "]"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"128", ",", "128"}], "}"}]], "Output",
 CellChangeTimes->{
  3.5660460601622744`*^9, {3.5660460942222223`*^9, 3.566046109062071*^9}}]
}, Closed]],

Cell[TextData[StyleBox["We can take a peek at the output:",
 FontWeight->"Plain"]], "Text",
 CellChangeTimes->{{3.565645594521987*^9, 3.565645612984967*^9}},
 FontWeight->"Bold"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{"LHS", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.565645565300479*^9, 3.565645571146969*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", 
       SuperscriptBox["a", "\[Prime]",
        MultilineFunction->None], " ", 
       SuperscriptBox["a", "\[Prime]\[Prime]",
        MultilineFunction->None]}], ")"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"1", ",", "z"}], "]"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"2", ",", "z"}], "]"}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", 
       SuperscriptBox["a", "\[Prime]",
        MultilineFunction->None], " ", 
       SuperscriptBox["a", "\[Prime]\[Prime]",
        MultilineFunction->None]}], ")"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"1", ",", "z"}], "]"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", 
       SuperscriptBox["a", "\[Prime]",
        MultilineFunction->None], " ", 
       SuperscriptBox["a", "\[Prime]\[Prime]",
        MultilineFunction->None]}], ")"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"2", ",", "z"}], "]"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
   RowBox[{"\[LeftSkeleton]", "170", "\[RightSkeleton]"}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "d", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "d", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "c", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}],
  Short[#, 3]& ]], "Output",
 CellChangeTimes->{
  3.565645572455522*^9, 3.565703991342012*^9, {3.5660460851417027`*^9, 
   3.566046112308257*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{"RHS", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.565645580938924*^9, 3.5656455853982697`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", 
       SuperscriptBox["a", "\[Prime]",
        MultilineFunction->None], " ", 
       SuperscriptBox["a", "\[Prime]\[Prime]",
        MultilineFunction->None]}], ")"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"1", ",", "z"}], "]"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"2", ",", "z"}], "]"}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", 
       SuperscriptBox["a", "\[Prime]",
        MultilineFunction->None], " ", 
       SuperscriptBox["a", "\[Prime]\[Prime]",
        MultilineFunction->None]}], ")"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"1", ",", "z"}], "]"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", 
       SuperscriptBox["a", "\[Prime]",
        MultilineFunction->None], " ", 
       SuperscriptBox["a", "\[Prime]\[Prime]",
        MultilineFunction->None]}], ")"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"2", ",", "z"}], "]"}], "**", 
     RowBox[{"\[Sigma]", "[", 
      RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
   RowBox[{"\[LeftSkeleton]", "170", "\[RightSkeleton]"}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "d", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "d", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "c", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}],
  Short[#, 3]& ]], "Output",
 CellChangeTimes->{3.565645586217031*^9, 3.5657039986915007`*^9, 
  3.5660461177915707`*^9}]
}, Closed]],

Cell["\<\
We now need to collect terms which involve the same set of operators. We can \
then set the sum of the coefficients of all the terms with identical \
operators to zero. We will now use the function TakeQPart,  to extract the \
string of operators from each term. By using the Union function we can also \
remove duplicate expressions. We are thereby left with a list of all the \
different strings of operators that appear anywhere throughout the expression.\
\>", "Text",
 CellChangeTimes->{{3.481483439265625*^9, 3.481483551046875*^9}, 
   3.483461928515625*^9, {3.48382565275*^9, 3.483825653765625*^9}, {
   3.4843356030625*^9, 3.4843356035625*^9}, 3.488220883151244*^9, 
   3.4891794185748587`*^9, {3.4918373343820677`*^9, 3.491837427561595*^9}, {
   3.5656456439978123`*^9, 3.565645665811219*^9}, {3.565704014688328*^9, 
   3.565704021949626*^9}, {3.5662169014993057`*^9, 3.566216903605426*^9}},
 LineSpacing->{1.5, 3}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OperatorTerms", "=", 
  RowBox[{"Union", "[", 
   RowBox[{"TakeQPart", "[", 
    RowBox[{"LHS", "-", "RHS"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.566046173186739*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "OperatorTerms", "]"}]], "Input"],

Cell[BoxData["6"], "Output",
 CellChangeTimes->{3.5660461429530096`*^9}]
}, Closed]],

Cell["\<\
We can now use the QCoefficient function to find the coefficients of a string \
of operators. The use of this function is demonstrated below.\
\>", "Text",
 CellChangeTimes->{{3.4918374819234953`*^9, 3.4918375213524313`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QCoefficient", "[", 
  RowBox[{
   RowBox[{"LHS", "-", "RHS"}], ",", 
   RowBox[{"OperatorTerms", "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
   SuperscriptBox["c", "\[Prime]",
    MultilineFunction->None], " ", 
   SuperscriptBox["a", "\[Prime]\[Prime]",
    MultilineFunction->None]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
   SuperscriptBox["d", "\[Prime]",
    MultilineFunction->None], " ", 
   SuperscriptBox["a", "\[Prime]\[Prime]",
    MultilineFunction->None]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
   SuperscriptBox["c", "\[Prime]",
    MultilineFunction->None], " ", 
   SuperscriptBox["b", "\[Prime]\[Prime]",
    MultilineFunction->None]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
   SuperscriptBox["d", "\[Prime]",
    MultilineFunction->None], " ", 
   SuperscriptBox["b", "\[Prime]\[Prime]",
    MultilineFunction->None]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
   SuperscriptBox["a", "\[Prime]",
    MultilineFunction->None], " ", 
   SuperscriptBox["c", "\[Prime]\[Prime]",
    MultilineFunction->None]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
   SuperscriptBox["b", "\[Prime]",
    MultilineFunction->None], " ", 
   SuperscriptBox["c", "\[Prime]\[Prime]",
    MultilineFunction->None]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
   SuperscriptBox["a", "\[Prime]",
    MultilineFunction->None], " ", 
   SuperscriptBox["d", "\[Prime]\[Prime]",
    MultilineFunction->None]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
   SuperscriptBox["b", "\[Prime]",
    MultilineFunction->None], " ", 
   SuperscriptBox["d", "\[Prime]\[Prime]",
    MultilineFunction->None]}]}]], "Output",
 CellChangeTimes->{3.5660461795441027`*^9}]
}, Closed]],

Cell["\<\
The function has found all of the terms with operators that match the pattern \
of OperatorTerms[[1]] and summed their coefficients. This term must go to \
zero for the Yang - Baxter relation to hold. We can work this function into a \
loop and find all such expressions at once, as shown below.\
\>", "Text",
 CellChangeTimes->{{3.491837550946856*^9, 3.4918376471486387`*^9}, {
  3.491837754053212*^9, 3.4918377849241333`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"expression", "[", "n", "]"}], "=", 
     RowBox[{"QCoefficient", "[", 
      RowBox[{
       RowBox[{"LHS", "-", "RHS"}], ",", 
       RowBox[{"OperatorTerms", "[", 
        RowBox[{"[", "n", "]"}], "]"}]}], "]"}]}], ";", 
    RowBox[{"Print", "[", 
     RowBox[{
      RowBox[{"expression", "[", "n", "]"}], "\[Equal]", "0"}], "]"}]}], ",", 
   
   RowBox[{"{", 
    RowBox[{"n", ",", 
     RowBox[{"Length", "[", "OperatorTerms", "]"}]}], "}"}]}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Print",
 CellChangeTimes->{3.566046195183997*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Print",
 CellChangeTimes->{3.5660461952269993`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Print",
 CellChangeTimes->{3.566046195276002*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Print",
 CellChangeTimes->{3.566046195325005*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Print",
 CellChangeTimes->{3.566046195376008*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "b", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "a", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "d", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", "c", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Print",
 CellChangeTimes->{3.5660461954210105`*^9}]
}, Closed]]
}, Closed]],

Cell["\<\
These are the final equations. By recombining them and simplifying we \
reproduce Baxter's equations.\
\>", "Text",
 CellChangeTimes->{{3.48165262309375*^9, 3.48165264975*^9}, 
   3.481653450421875*^9, 3.48346192853125*^9, {3.484336117953125*^9, 
   3.4843361323125*^9}, 3.48822088317485*^9, {3.491837709316001*^9, 
   3.4918377170444202`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"expression", "[", "2", "]"}], "+", 
      RowBox[{"expression", "[", "4", "]"}]}], ")"}], "/", "\[ImaginaryI]"}], 
   "]"}], "\[Equal]", "0"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"expression", "[", "1", "]"}], "+", 
      RowBox[{"expression", "[", "3", "]"}]}], ")"}], "/", "\[ImaginaryI]"}], 
   "]"}], "\[Equal]", "0"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"expression", "[", "5", "]"}], "+", 
      RowBox[{"expression", "[", "6", "]"}]}], ")"}], "/", "\[ImaginaryI]"}], 
   "]"}], "\[Equal]", "0"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"expression", "[", "2", "]"}], "-", 
      RowBox[{"expression", "[", "4", "]"}]}], ")"}], "/", "\[ImaginaryI]"}], 
   "]"}], "\[Equal]", "0"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"expression", "[", "1", "]"}], "-", 
      RowBox[{"expression", "[", "3", "]"}]}], ")"}], "/", "\[ImaginaryI]"}], 
   "]"}], "\[Equal]", "0"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"expression", "[", "5", "]"}], "-", 
      RowBox[{"expression", "[", "6", "]"}]}], ")"}], "/", "\[ImaginaryI]"}], 
   "]"}], "\[Equal]", "0"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", "d"}], " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{"d", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{"a", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"a", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.566046213366037*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"c", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{"a", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"b", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.566046213368037*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", "b"}], " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{"a", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{"d", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"c", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.566046213370037*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"c", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"c", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"b", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{"b", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.5660462133710375`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"c", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"d", " ", 
    SuperscriptBox["d", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{"b", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"a", " ", 
    SuperscriptBox["b", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.5660462133730373`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"a", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["a", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"b", " ", 
    SuperscriptBox["c", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["b", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "-", 
   RowBox[{"c", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["c", "\[Prime]\[Prime]",
     MultilineFunction->None]}], "+", 
   RowBox[{"d", " ", 
    SuperscriptBox["a", "\[Prime]",
     MultilineFunction->None], " ", 
    SuperscriptBox["d", "\[Prime]\[Prime]",
     MultilineFunction->None]}]}], "\[Equal]", "0"}]], "Output",
 CellChangeTimes->{3.5660462133740377`*^9}]
}, Closed]],

Cell["\<\
These are six homogeneous equations relating the twelve parameters (a,b,c,d) \
etc. Baxter proceeds to give an elegant analysis of these by  \
parameterization  in terms of the spectral parameters u, u\[CloseCurlyQuote], \
u\[CloseCurlyQuote]\[CloseCurlyQuote]. The solution is well described in the \
original paper; we see above that DiracQ  helps in automating the tedious \
calculations.  \
\>", "Text",
 CellChangeTimes->{{3.483825819296875*^9, 3.48382604509375*^9}, {
   3.484336149078125*^9, 3.4843361495*^9}, 3.4882208831934*^9, {
   3.491838152849648*^9, 3.4918381538774*^9}, {3.565645917389517*^9, 
   3.5656461065164547`*^9}, {3.565646137844739*^9, 3.565646273493104*^9}, {
   3.5657038818623533`*^9, 3.565703942599605*^9}, {3.5657040720882387`*^9, 
   3.565704076162386*^9}, {3.5660462346812563`*^9, 3.566046254703401*^9}, 
   3.566216922399501*^9},
 LineSpacing->{1.5, 3}],

Cell["", "PageBreak",
 PageBreakBelow->True,
 FontSize->14,
 FontWeight->"Bold"]
}, Open  ]],

Cell[CellGroupData[{

Cell["(VI) 1D Hubbard Model SSS Matrix Problem", "Subsection",
 CellChangeTimes->{{3.488217837197317*^9, 3.488217855896879*^9}, 
   3.4892451203135433`*^9, {3.490453627298954*^9, 3.490453629595949*^9}, {
   3.530412075206212*^9, 3.530412078186618*^9}, 3.565275165224691*^9, 
   3.565836419998775*^9},
 FontFamily->"Times"],

Cell[CellGroupData[{

Cell["Activate the Pauli matrix symbol \"\[Sigma]\"", "Item",
 CellChangeTimes->{{3.4929686704746027`*^9, 3.492968699776744*^9}, {
  3.5657133356011677`*^9, 3.565713335604238*^9}},
 FontSize->14,
 FontWeight->"Bold"],

Cell[CellGroupData[{

Cell["\<\
We next demonstrate the use of DiracQ in the context of the 1-d Hubbard \
model. It satisfies the Yang Baxter relation, analogous to the 8 vertex model \
discussed in the previous example, and the proof of this is given in B. S. \
Shastry, Phys. Rev. Letts. vol 56, 1529, 2453 (1986).  A relatively simple \
analytical  proof is also to be found in B. S. Shastry, J. Stat. Phys. vol \
50, 57 (1988), using the decorated star triangle relations.  We recall this \
result before proceeding further. It may be noted that the R matrix of this \
model, defined below,  is currently of great interest in connection with   \
the problem of the ADS/CFT correspondence for N = 4 SUSY Yang-Mills theory in \
four dimensions following the work of  N. Beisert, J.Stat.Mech. 0701 (2007) \
P01017, arXiv:nlin/0610017.\
\>", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 15001.},
 CellChangeTimes->{{3.483738652595137*^9, 3.483738744751387*^9}, {
   3.483738896173262*^9, 3.483738925767012*^9}, {3.483739028938887*^9, 
   3.483739113001387*^9}, {3.483739156157637*^9, 3.483739356548262*^9}, {
   3.483739711923262*^9, 3.483739817329512*^9}, {3.483971477953125*^9, 
   3.483971489875*^9}, {3.483971692546875*^9, 3.483971709765625*^9}, {
   3.48397179759375*^9, 3.483971806125*^9}, {3.483971851234375*^9, 
   3.483971904640625*^9}, {3.48397242009375*^9, 3.4839724281875*^9}, {
   3.484336166390625*^9, 3.484336188078125*^9}, 3.4891789656271772`*^9, {
   3.4892439399826736`*^9, 3.4892439469302177`*^9}, {3.489331150679132*^9, 
   3.4893311514197493`*^9}, {3.565713289853262*^9, 3.5657133113383913`*^9}, {
   3.565713349578238*^9, 3.5657134768650427`*^9}, {3.565713515716501*^9, 
   3.565713728029298*^9}, {3.565713766255362*^9, 3.565713793769643*^9}, {
   3.5657138537948837`*^9, 3.565713961134357*^9}, {3.565713998758895*^9, 
   3.565714027780023*^9}, {3.565714059860623*^9, 3.565714510314746*^9}, {
   3.5657146098536673`*^9, 3.5657146240297537`*^9}, {3.565714654149926*^9, 
   3.565714888389523*^9}, {3.565715313295261*^9, 3.565715328407378*^9}, {
   3.5657154341703243`*^9, 3.5657156320469837`*^9}, {3.565715662919568*^9, 
   3.565715779684214*^9}, {3.565715834261265*^9, 3.565715858138712*^9}, 
   3.565715902162878*^9, 3.565716556205474*^9, 3.5657166236760683`*^9, {
   3.565716696407639*^9, 3.565716697356455*^9}, 3.5657209474589148`*^9, {
   3.565729779239767*^9, 3.565729796402397*^9}, {3.5662169475359383`*^9, 
   3.5662169519531913`*^9}, {3.56700824776301*^9, 3.567008313064756*^9}, {
   3.567010514886923*^9, 3.567010576742921*^9}, {3.567010640832296*^9, 
   3.5670106751318617`*^9}},
 LineSpacing->{1.5, 3}],

Cell["\<\
The 1-dimensional Hubbard model Hamiltonian using the Jordan Wigner \
transformation and with suitable boundary conditions can be written in terms \
of Pauli spin matrices as\
\>", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 15001.},
 CellChangeTimes->{{3.483971917828125*^9, 3.483971949671875*^9}, {
   3.48397206509375*^9, 3.483972241015625*^9}, {3.48397228584375*^9, 
   3.48397240334375*^9}, {3.483972530078125*^9, 3.483972533515625*^9}, {
   3.483995892390625*^9, 3.483995900203125*^9}, {3.484335656109375*^9, 
   3.48433565740625*^9}, {3.484336208875*^9, 3.48433622209375*^9}, {
   3.488906586359375*^9, 3.488906601328125*^9}, {3.488906835140625*^9, 
   3.488907044515625*^9}, {3.488908607515625*^9, 3.4889086080625*^9}, {
   3.488908681296875*^9, 3.488908685140625*^9}, {3.4889094539375*^9, 
   3.4889094559375*^9}, 3.4891690145283823`*^9, 3.489169191485353*^9, 
   3.4891794367980413`*^9, 3.489243960736864*^9, {3.4893311653188353`*^9, 
   3.489331266836211*^9}, 3.490458403333181*^9, {3.4905470962766237`*^9, 
   3.4905470968798857`*^9}, 3.5657133246649218`*^9, {3.565715858138801*^9, 
   3.565715974273974*^9}, {3.565728489832163*^9, 3.565728518120768*^9}, 
   3.566216958142545*^9},
 LineSpacing->{1.5, 3}]
}, Closed]]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"H", "=", 
   RowBox[{
    UnderscriptBox["\[Sum]", "n"], 
    SubscriptBox["H", 
     RowBox[{
      RowBox[{"n", "+", "1"}], ",", "n"}]]}]}], ";", "   ", 
  RowBox[{
   SubscriptBox["H", 
    RowBox[{
     RowBox[{"n", "+", "1"}], ",", "n"}]], "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        SubscriptBox["\[Sigma]", "n"], "+"], " ", 
       SuperscriptBox[
        SubscriptBox["\[Sigma]", 
         RowBox[{"n", "+", "1"}]], "-"]}], "+", 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["\[Sigma]", 
         RowBox[{"n", "+", "1"}]], "+"], " ", 
       SuperscriptBox[
        SubscriptBox["\[Sigma]", "n"], "-"]}]}], ")"}], "+", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        SubscriptBox["\[Tau]", "n"], "+"], " ", 
       SuperscriptBox[
        SubscriptBox["\[Tau]", 
         RowBox[{"n", "+", "1"}]], "-"]}], "+", 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["\[Tau]", 
         RowBox[{"n", "+", "1"}]], "+"], " ", 
       SuperscriptBox[
        SubscriptBox["\[Tau]", "n"], "-"]}]}], ")"}], "+", 
    RowBox[{
     FractionBox["1", "4"], "U", " ", 
     SubsuperscriptBox["\[Sigma]", "n", "z"], 
     RowBox[{
      SubsuperscriptBox["\[Tau]", "n", "z"], " ", ".", 
      "     "}]}]}]}]}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.4893312786287603`*^9, 3.489331292916258*^9}, {
  3.565721082274253*^9, 3.5657210925827303`*^9}, {3.5657303615731497`*^9, 
  3.565730410210541*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell["\<\
This corresponds to the quantum Hamiltonian, commuting with the transfer \
matrix of two Free fermi models coupled in a  specific way. We start with a \
single Pauli species scattering matrix :\
\>", "Text",
 CellChangeTimes->{{3.4891693477180557`*^9, 3.489169348594466*^9}, 
   3.4891794510058193`*^9, {3.489331457417654*^9, 3.489331553568712*^9}, {
   3.565716026052948*^9, 3.5657160941097307`*^9}, 3.565716129826479*^9, {
   3.565717428512968*^9, 3.565717474988731*^9}, {3.565719009275399*^9, 
   3.565719010119937*^9}, {3.5657211111804113`*^9, 3.565721118696289*^9}, {
   3.565728143552685*^9, 3.565728151537649*^9}, {3.565729811789937*^9, 
   3.5657298296525393`*^9}, {3.56621696329284*^9, 3.5662169659589925`*^9}}],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["l", 
     RowBox[{"i", " ", "j"}], 
     RowBox[{"(", "\[Sigma]", ")"}]], 
    RowBox[{"(", "\[Theta]", 
     StyleBox[")",
      FontSize->14]}]}], " ", "=", " ", 
   RowBox[{
    FormBox[
     RowBox[{" ", 
      StyleBox[
       FractionBox[
        RowBox[{"a", "+", "b"}], "2"],
       FontSize->14]}],
     TraditionalForm], 
    StyleBox["+",
     FontSize->14], 
    StyleBox[" ",
     FontSize->14], 
    RowBox[{
     FormBox[
      RowBox[{" ", 
       StyleBox[
        FractionBox[
         RowBox[{"a", "-", "b"}], "2"],
        FontSize->14]}],
      TraditionalForm], 
     StyleBox["   ",
      FontSize->14], 
     FormBox[
      SuperscriptBox[
       SubscriptBox["\[Sigma]", "i"], "z"],
      TraditionalForm], 
     FormBox[
      RowBox[{
       SuperscriptBox[
        SubscriptBox["\[Sigma]", "j"], "z"], "  ", "+", " "}],
      TraditionalForm], 
     FractionBox["c", "2"], 
     StyleBox[" ",
      FontSize->14], 
     RowBox[{
      StyleBox["(",
       FontSize->14], 
      StyleBox[" ",
       FontSize->14], 
      RowBox[{
       FormBox[
        SuperscriptBox[
         SubscriptBox["\[Sigma]", "i"], "x"],
        TraditionalForm], 
       FormBox[
        RowBox[{
         SuperscriptBox[
          SubscriptBox["\[Sigma]", "j"], "x"], "  ", "+", " "}],
        TraditionalForm], 
       FormBox[
        SuperscriptBox[
         SubscriptBox["\[Sigma]", "i"], "y"],
        TraditionalForm], 
       FormBox[
        RowBox[{
         RowBox[{
          SuperscriptBox[
           SubscriptBox["\[Sigma]", "j"], "y"], ")"}], "   "}],
        TraditionalForm]}]}]}]}]}],
  FontFamily->"Times"]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.565716133756404*^9, 3.565716167730626*^9}, {
  3.565716198833499*^9, 3.565716239482613*^9}, {3.565716272779652*^9, 
  3.565716497069412*^9}, {3.5657173975882874`*^9, 3.565717400938023*^9}, {
  3.565730502937625*^9, 3.5657305029391603`*^9}, {3.5657306059111223`*^9, 
  3.56573065023615*^9}, {3.5657306884237413`*^9, 3.565730694685401*^9}, {
  3.56573138847994*^9, 3.565731449103373*^9}},
 TextAlignment->Center],

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox["Here", "Text"], 
      StyleBox[" ", "Text"], 
      StyleBox["the", "Text"], 
      StyleBox[" ", "Text"], 
      StyleBox["parameters", "Text"], 
      StyleBox[" ", "Text"], 
      StyleBox["a", "Text"], 
      StyleBox[" ", "Text",
       FontSize->14], "b", 
      StyleBox[" ", "Text",
       FontSize->14], "c", 
      StyleBox[" ", "Text"], 
      StyleBox["satisfy", "Text"], 
      StyleBox[" ", "Text"], 
      StyleBox["the", "Text"], 
      StyleBox[" ", "Text"], 
      StyleBox["Free", "Text"], 
      StyleBox[" ", "Text"], 
      StyleBox["Fermi", "Text"], 
      StyleBox[" ", "Text"], 
      StyleBox["condition", "Text"], 
      StyleBox[" ", "Text"], 
      StyleBox[
       SuperscriptBox["a", "2"], "Text"]}], 
     StyleBox["+", "Text"], 
     StyleBox[
      SuperscriptBox[
       StyleBox["b", "Text"], "2"], "Text"]}], 
    StyleBox["=", "Text"], 
    StyleBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["c", "2"], ".", 
       StyleBox[" ", "Text",
        FontSize->14], "This"}], " ", "is", " ", "conveniently", " ", 
      "parametrized", " ", "by", " ", "setting"}], "Text"]}], 
   StyleBox[" ", "Text"]}], "Text",
  FontFamily->"Times"], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{
    StyleBox["a", "Text"], 
    StyleBox["=", "Text"], 
    StyleBox[" ", "Text",
     FontSize->14], 
    RowBox[{
     StyleBox[
      RowBox[{
       StyleBox[
        StyleBox[
         StyleBox[
          StyleBox["c", "Text",
           FontSize->14], "Text",
          FontSize->14], "Text",
         FontSize->14],
        FontSize->14], " ", 
       StyleBox[
        RowBox[{"Co", 
         StyleBox[
          StyleBox[
           StyleBox["s", "Text",
            FontSize->14], "Text",
           FontSize->14], "Text",
          FontSize->14]}]], 
       RowBox[{"(", "\[Theta]", ")"}], "  ", 
       StyleBox["b",
        FontSize->14]}], "Text"], 
     StyleBox["=", "Text",
      FontSize->14], 
     RowBox[{
      StyleBox[
       StyleBox["c", "Text",
        FontSize->14], "Text",
       FontSize->14], 
      StyleBox[" ", "Text",
       FontSize->14], 
      StyleBox["Sin", "Text",
       FontSize->14], 
      RowBox[{
       StyleBox[
        RowBox[{"(", "\[Theta]", 
         StyleBox[
          StyleBox[
           StyleBox[
            StyleBox[")", "Text",
             FontSize->14], "Text",
            FontSize->14], "Text",
           FontSize->14], "Text",
          FontSize->14]}], "Text",
        FontSize->14], 
       StyleBox[".", "Text",
        FontSize->14], 
       StyleBox[" ", "Text",
        FontSize->14], "For"}], " ", "the", " ", "two", " ", "sites", " ", 
      "1", " ", "and", " ", "2", " ", "we", 
      StyleBox[" ", "Text",
       FontSize->14], "define"}]}]}], " "}], "Text",
  FontFamily->"Times"], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{
    SubscriptBox["a", "1"], 
    StyleBox[
     StyleBox[
      StyleBox[
       StyleBox["=", "Text",
        FontSize->14], "Text",
       FontSize->14], "Text",
      FontSize->14], "Text",
     FontSize->14], 
    RowBox[{"Cos", "[", 
     SubscriptBox["\[Theta]", "1"], "]"}]}], 
   StyleBox[
    StyleBox[
     StyleBox[",", "Text",
      FontFamily->"Times"], "Text",
     FontFamily->"Times"], "Text",
    FontFamily->"Times"], " ", 
   RowBox[{
    SubscriptBox["b", "1"], "=", 
    RowBox[{"Sin", "[", 
     SubscriptBox["\[Theta]", "1"], "]"}]}], 
   StyleBox[
    StyleBox[
     StyleBox[
      StyleBox[",", "Text",
       FontFamily->"Times"], "Text",
      FontFamily->"Times"], "Text",
     FontFamily->"Times"], "Text",
    FontFamily->"Times"], " ", 
   RowBox[{
    SubscriptBox["a", "2"], "=", 
    RowBox[{"Cos", "[", 
     SubscriptBox["\[Theta]", "2"], "]"}]}], 
   StyleBox[
    StyleBox[
     StyleBox[
      StyleBox[",", "Text",
       FontFamily->"Times"], "Text",
      FontFamily->"Times"], "Text",
     FontFamily->"Times"], "Text",
    FontFamily->"Times"], " ", 
   RowBox[{
    SubscriptBox["b", "2"], "=", 
    RowBox[{
     RowBox[{"Sin", "[", 
      SubscriptBox["\[Theta]", "2"], "]"}], 
     StyleBox[" ", "Text",
      FontSize->14], 
     StyleBox[
      RowBox[{
       StyleBox[
        StyleBox[
         StyleBox[
          StyleBox[
           StyleBox[
            StyleBox["a", "Text",
             FontSize->14], "Text",
            FontSize->14], "Text",
           FontSize->14], "Text",
          FontSize->14], "Text",
         FontSize->14], "Text",
        FontSize->14], "nd"}]], 
     StyleBox[" ", "Text",
      FontSize->14], 
     StyleBox[
      RowBox[{
       StyleBox[
        StyleBox[
         StyleBox[
          StyleBox[
           StyleBox[
            StyleBox["d", "Text",
             FontSize->14], "Text",
            FontSize->14], "Text",
           FontSize->14], "Text",
          FontSize->14], "Text",
         FontSize->14], "Text",
        FontSize->14], "enote"}]], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["W", "j"], "\[Congruent]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Theta]", "j"], ",", 
          SubscriptBox["h", "j"]}], ")"}]}], ")"}], ".", 
      StyleBox[
       StyleBox[
        StyleBox[
         StyleBox[
          StyleBox[
           StyleBox["\[IndentingNewLine]", "Text",
            FontSize->14], "Text",
           FontSize->14], "Text",
          FontSize->14], "Text",
         FontSize->14], "Text",
        FontSize->14], "Text",
       FontSize->14], 
      StyleBox[
       StyleBox[
        StyleBox[
         StyleBox["\[IndentingNewLine]", "Text",
          FontSize->14], "Text",
         FontFamily->"Times"], "Text",
        FontFamily->"Times"], "Text",
       FontFamily->"Times"], "We"}], " ", "may", " ", "write", " ", "two", 
     " ", "sets", " ", "of", " ", "scattering", " ", "matrices", 
     FormBox[
      SubsuperscriptBox["l", 
       RowBox[{"i", " ", "j"}], 
       RowBox[{"(", "\[Sigma]", ")"}]],
      TraditionalForm], "and", 
     FormBox[
      SubsuperscriptBox["l", 
       RowBox[{"i", " ", "j"}], 
       RowBox[{"(", "\[Tau]", ")"}]],
      TraditionalForm], "for", " ", "the", " ", "two", " ", "species", " ", 
     "at", " ", "the", " ", "same", " ", 
     RowBox[{"\[Theta]", ":"}]}]}]}], "Text",
  FontFamily->"Times"]}], "Text",
 CellChangeTimes->{{3.565717004870986*^9, 3.5657170304111233`*^9}, {
   3.565717135655394*^9, 3.5657171710189543`*^9}, 3.565717213659102*^9, {
   3.5657172446363897`*^9, 3.565717304725998*^9}, {3.565717381701796*^9, 
   3.565717432599729*^9}, {3.5657175662551107`*^9, 3.565717610446063*^9}, {
   3.565718540702201*^9, 3.5657185663243523`*^9}, {3.565719117218995*^9, 
   3.565719150613709*^9}, {3.565721217600267*^9, 3.565721223297142*^9}, {
   3.565728166164818*^9, 3.565728272747405*^9}, 3.565728343250338*^9, {
   3.565729081192615*^9, 3.5657291150125017`*^9}, {3.5662169742434664`*^9, 
   3.566216979880789*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["l", 
    RowBox[{"i", " ", "j"}]], 
   RowBox[{"(", "\[Theta]", 
    StyleBox[")",
     FontSize->14]}]}], "=", 
  RowBox[{
   SubsuperscriptBox["l", 
    RowBox[{"i", " ", "j"}], 
    RowBox[{"(", "\[Sigma]", ")"}]], 
   RowBox[{
    RowBox[{"(", "\[Theta]", 
     StyleBox[")",
      FontSize->14]}], "\[CircleTimes]", 
    SubsuperscriptBox["l", 
     RowBox[{"i", " ", "j"}], 
     RowBox[{"(", "\[Tau]", ")"}]]}], 
   RowBox[{"(", "\[Theta]", 
    StyleBox[")",
     FontSize->14]}]}]}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.489331557141803*^9, 3.4893315837453957`*^9}, {
  3.5657291297618523`*^9, 3.565729134781657*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell[BoxData[
 StyleBox[
  RowBox[{
  "The", "  ", "scattering", " ", "operator", "  ", "for", " ", "the", " ", 
   "Hubbard", " ", "model", " ", "is", " ", "constucted", "  ", "as"}], 
  "Text"]], "Text",
 CellChangeTimes->{{3.488913023015625*^9, 3.488913024703125*^9}, {
  3.48891416265625*^9, 3.488914193109375*^9}, {3.48891422921875*^9, 
  3.4889143821875*^9}, {3.489179453721313*^9, 3.489179458467177*^9}, {
  3.489244105501914*^9, 3.489244105925639*^9}, {3.489331743217264*^9, 
  3.4893317589196997`*^9}, {3.565717725057376*^9, 3.5657177607553377`*^9}, {
  3.5657191801803627`*^9, 3.565719184242489*^9}, {3.565721135132325*^9, 
  3.565721184299007*^9}, {3.565728373609076*^9, 3.565728377746031*^9}, {
  3.565728448858405*^9, 3.565728451970222*^9}},
 FontFamily->"Times"],

Cell[BoxData[
 StyleBox[
  RowBox[{
   StyleBox[Cell[TextData[Cell[BoxData[
     StyleBox[
      RowBox[{
       RowBox[{
        SubscriptBox["L", 
         RowBox[{"3", "n"}]], 
        RowBox[{"(", 
         SubscriptBox["W", "n"], ")"}]}], "=", 
       RowBox[{
        SubscriptBox["l", 
         RowBox[{"3", "n"}]], 
        RowBox[{"(", 
         SubscriptBox["\[Theta]", "n"], ")"}], 
        SubscriptBox["I", "n"], 
        RowBox[{"(", 
         SubscriptBox["h", "n"], ")"}]}]}],
      FontFamily->"Times"]]]]],
    FontSize->14], 
   StyleBox[",",
    FontSize->14], 
   StyleBox[
    RowBox[{
     StyleBox[" ",
      FontSize->14], " "}]]}], "Text",
  FontFamily->"Times"]], "Input",
 CellChangeTimes->{{3.565729157488332*^9, 3.565729210441391*^9}, {
  3.5657292756566*^9, 3.5657292782159224`*^9}, {3.565729396190639*^9, 
  3.565729509692094*^9}, {3.565729581848571*^9, 3.565729603224696*^9}},
 TextAlignment->Center,
 FontWeight->"Plain"],

Cell[TextData[StyleBox["The transfer matrix can now be written as",
 FontWeight->"Plain"]], "Text",
 CellChangeTimes->{{3.565729943140991*^9, 3.565729955240953*^9}},
 FontWeight->"Bold"],

Cell[BoxData[
 RowBox[{
  RowBox[{"T", 
   RowBox[{"(", "W", ")"}]}], "=", 
  RowBox[{
   RowBox[{
    UnderscriptBox["Tr", "o"], "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["L", "No"], 
       RowBox[{"(", "W", ")"}], 
       SubscriptBox["L", 
        RowBox[{"N", "-", 
         RowBox[{"1", "o"}]}]], 
       RowBox[{"(", "W", ")"}]}], "..."}], 
     SubscriptBox["L", 
      RowBox[{"1", "o"}]], 
     RowBox[{"(", "W", ")"}]}], "]"}], "."}]}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.489333392735273*^9, 3.489333498133288*^9}, {
  3.5657215927988586`*^9, 3.565721632423533*^9}, {3.56572989184337*^9, 
  3.5657299334051867`*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{
  RowBox[{"With", " ", "these", " ", "definitions"}], ",", 
  RowBox[{
   RowBox[{
   "the", " ", "transfer", " ", "matrix", " ", "forms", " ", "a", " ", 
    "commuting", " ", 
    RowBox[{"family", " ", "[", 
     RowBox[{
      RowBox[{"T", 
       RowBox[{"(", 
        SubscriptBox["W", "1"], ")"}]}], ",", 
      RowBox[{"T", 
       RowBox[{"(", 
        SubscriptBox["W", "2"], ")"}]}]}], "]"}]}], "=", 
   RowBox[{
    RowBox[{
    "0", " ", "provided", " ", "the", " ", "scattering", " ", "matrix", "  ", 
     "is", " ", "proven", " ", "to", " ", "satisfy", " ", "the", " ", 
     "Yang"}], "-", 
    RowBox[{"Baxter", " ", "equation", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["W", "n"], "\[Congruent]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Theta]", "n"], ",", 
         SubscriptBox["h", "n"]}], ")"}]}], ")"}]}]}]}]}]], "Text",
 CellChangeTimes->{{3.4893324948896914`*^9, 3.489332565118*^9}, {
  3.565721190696644*^9, 3.5657212051803923`*^9}, {3.565728466984169*^9, 
  3.5657284719600677`*^9}, {3.56572961146954*^9, 3.565729619128441*^9}, {
  3.5657299802049103`*^9, 3.565730067945759*^9}, {3.5660463186540594`*^9, 
  3.5660463203901587`*^9}},
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["L", "31"], 
   RowBox[{"(", 
    SubscriptBox["W", "1"], ")"}], 
   SubscriptBox["L", "32"], 
   RowBox[{"(", 
    SubscriptBox["W", "2"], ")"}], 
   SubscriptBox["S", "12"], 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["W", "2"], "|", 
     SubscriptBox["W", "1"]}], ")"}]}], "=", 
  RowBox[{
   SubscriptBox["S", "12"], 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["W", "2"], "|", 
     SubscriptBox["W", "1"]}], ")"}], 
   SubscriptBox["L", "32"], 
   RowBox[{"(", 
    SubscriptBox["W", "2"], ")"}], 
   SubscriptBox["L", "31"], 
   RowBox[{
    RowBox[{"(", 
     SubscriptBox["W", "1"], ")"}], "."}]}]}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.4893326076230917`*^9, 3.4893326157493773`*^9}, {
   3.489332990008946*^9, 3.4893330493451157`*^9}, {3.565729523679639*^9, 
   3.565729563102584*^9}, 3.565730089581232*^9},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell["The  S operator found by Shastry is given by:", "Text",
 CellChangeTimes->{{3.489333071163238*^9, 3.489333072112135*^9}, {
  3.565721260155407*^9, 3.565721286355809*^9}, {3.5657296290532227`*^9, 
  3.565729631951641*^9}, {3.565730094441824*^9, 3.565730109166815*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["S", "12"], 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["W", "2"], "|", 
     SubscriptBox["W", "1"]}], ")"}]}], "=", 
  RowBox[{
   RowBox[{"\[Rho]", "[", 
    RowBox[{
     RowBox[{"cos", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Theta]", "2"], "+", 
        SubscriptBox["\[Theta]", "1"]}], ")"}], "cos", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "2"], "-", 
        SubscriptBox["h", "1"]}], ")"}], 
      SubscriptBox["l", "12"], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Theta]", "2"], "-", 
        SubscriptBox["\[Theta]", "1"]}], ")"}]}], "+", 
     RowBox[{"cos", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Theta]", "2"], "-", 
        SubscriptBox["\[Theta]", "1"]}], ")"}], "sinh", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "2"], "-", 
        SubscriptBox["h", "1"]}], ")"}], 
      SubscriptBox["l", "12"], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Theta]", "2"], "+", 
        SubscriptBox["\[Theta]", "1"]}], ")"}], 
      SubsuperscriptBox["\[Sigma]", "2", "z"], 
      SubsuperscriptBox["\[Tau]", "2", "z"]}]}], "]"}], 
   "."}]}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.489333078984861*^9, 3.489333090600279*^9}, {
  3.489333145899724*^9, 3.489333170961636*^9}, {3.56572963928004*^9, 
  3.565729644410172*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
  "An", " ", "extra", "  ", "and", "  ", "crucial", " ", "sufficiency", " ", 
   "condition", " ", "is", "  ", "that", "  ", "the", " ", "fields", " ", 
   SubscriptBox["h", "n"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"n", "=", "1"}], ",", "2"}], ")"}], " ", "must", " ", 
   "satisfy"}]}]], "Text",
 CellChangeTimes->{{3.489332353021078*^9, 3.489332375600018*^9}, {
  3.48933246843111*^9, 3.4893324901827106`*^9}, {3.5657212940184593`*^9, 
  3.56572133845883*^9}, {3.565730121332312*^9, 3.5657301385952063`*^9}},
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox[
    RowBox[{"Sinh", "[", 
     RowBox[{"2", " ", 
      SubscriptBox["h", "n"]}], "]"}], 
    RowBox[{
     SubscriptBox["a", "n"], 
     SubscriptBox["b", "n"]}]], "=", 
   FractionBox["U", "2"]}], ","}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.489332383837482*^9, 3.489332418264*^9}, {
  3.4893324547594013`*^9, 3.489332457638693*^9}, {3.565729007075255*^9, 
  3.5657290076251297`*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{
   "where", " ", "U", " ", "is", " ", "the", " ", "interaction", " ", "term", 
    " ", "in", " ", "the", " ", "Hubbard", " ", 
    RowBox[{"Hamiltonian", ".", "  ", "The"}], " ", "peculiarity", " ", "of", 
    " ", "this", " ", "model", " ", "is", " ", "that", "  ", "S", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["W", "2"], "|", 
      SubscriptBox["W", "1"]}], ")"}], " ", "depends", " ", "on", " ", "the", 
    " ", "spectral", " ", "parameters", " ", 
    SubscriptBox["W", "1"], "and", " ", 
    SubscriptBox["W", "2"], " ", 
    RowBox[{"separately", ".", " ", "It"}], 
    StyleBox[
     RowBox[{" ", "  "}]], "cannot", " ", "be", " ", "written", " ", "in", "  ",
     "terms", " ", "of", " ", "the", " ", "difference", " ", "of", " ", "the",
     " ", "spectral", " ", "parameters"}], ",", " ", 
   RowBox[{"unlike", " ", "most", " ", "other", " ", "standard", " ", 
    RowBox[{"models", ".", " ", "This"}], " ", "relation", " ", "was", "  ", 
    "proved", " ", "by", " ", "Shastry", " ", "analytically", " ", "in", " ", 
    
    RowBox[{"B", ".", " ", "S", ".", " ", "Shastry"}]}], ",", " ", 
   RowBox[{"J", ".", " ", "Stat", ".", " ", "Phys", ".", " ", "50"}], ",", 
   " ", 
   RowBox[{"57", " ", 
    RowBox[{
     RowBox[{"(", "1988", ")"}], ".", "\[IndentingNewLine]"}]}]}],
  FontFamily->"Arial"]], "Text",
 CellChangeTimes->{{3.489333210051049*^9, 3.489333382053822*^9}, {
  3.489345502424097*^9, 3.48934550355847*^9}, {3.490458429488907*^9, 
  3.4904584441876287`*^9}, {3.4904584970108557`*^9, 3.4904585067045317`*^9}, {
  3.5657213516078787`*^9, 3.565721583384212*^9}, {3.5657290107871532`*^9, 
  3.565729028337867*^9}, {3.565729059988598*^9, 3.565729062852069*^9}, {
  3.565729664508366*^9, 3.565729722879571*^9}, {3.5657297590073633`*^9, 
  3.565729762881518*^9}, {3.565730150207234*^9, 3.5657302137212973`*^9}, {
  3.5662170047632117`*^9, 3.566217017357932*^9}, {3.648223540109664*^9, 
  3.648223604184374*^9}, {3.648223639284934*^9, 3.648223720838372*^9}},
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{"T", "=", 
  RowBox[{
   RowBox[{
    UnderscriptBox["Tr", "o"], "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["S", "No"], 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["W", "1"], "|", 
         SubscriptBox["W", "2"]}], ")"}], 
       SubscriptBox["S", 
        RowBox[{"N", "-", 
         RowBox[{"1", "o"}]}]], 
       RowBox[{"(", 
        RowBox[{"W", "|", 
         SubscriptBox["W", 
          RowBox[{"N", "-", "1"}]]}], ")"}]}], "..."}], 
     SubscriptBox["S", 
      RowBox[{"1", "o"}]], 
     RowBox[{"(", 
      RowBox[{"W", "|", 
       SubscriptBox["W", "1"]}], ")"}]}], "]"}], 
   "."}]}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.489333392735273*^9, 3.489333498133288*^9}, {
  3.5657215927988586`*^9, 3.565721632423533*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{
  RowBox[{"so", " ", "that", " ", "on", " ", "setting", " ", 
   SubscriptBox["W", "j"], " ", "to", " ", "0"}], ",", 
  RowBox[{
  "we", " ", "get", " ", "back", " ", "the", " ", "Hubbard", " ", "transfer", 
   " ", "matrix", "  ", 
   RowBox[{"above", ".", "   ", "The"}], " ", "inhomogeneous", " ", "matrix", 
   " ", "T", " ", "commutes", " ", "for", " ", "different", " ", "weights", 
   " ", "according", " ", "to", " "}]}]], "Text",
 CellChangeTimes->{{3.489333513360654*^9, 3.489333558139255*^9}, {
  3.4893336469389687`*^9, 3.489333649234634*^9}, {3.565721645084589*^9, 
  3.5657217059353437`*^9}, {3.565730273871415*^9, 3.565730278158256*^9}, {
  3.565730759649493*^9, 3.565730780711832*^9}},
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"[", 
    RowBox[{
     RowBox[{"T", 
      RowBox[{"(", 
       RowBox[{"W", "|", 
        RowBox[{
         RowBox[{
          SubscriptBox["W", "1"], "..."}], 
         SubscriptBox["W", "N"]}]}], ")"}]}], ",", 
     RowBox[{"T", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"W", "'"}], "|", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["W", "1"], "..."}], 
          SubscriptBox["W", "N"]}], ")"}]}]}]}]}]}], "]"}], "=", 
  "0."}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.4893335648711433`*^9, 3.489333641251338*^9}, 
   3.489333736097467*^9, {3.565721712761664*^9, 3.565721730773368*^9}, {
   3.5657302925692368`*^9, 3.56573029535711*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell["\<\
The sufficiency condition for this is the more general Yang Baxter relation\
\>", "Text",
 CellChangeTimes->{{3.565716778759037*^9, 3.565716799584461*^9}, {
   3.565721757423324*^9, 3.565721789496964*^9}, {3.565730307545129*^9, 
   3.56573030851819*^9}, 3.5660475222228994`*^9, 3.566217026018428*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  StyleBox[
   RowBox[{
    RowBox[{
     SubscriptBox["S", "31"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["W", "1"], "|", 
       SubscriptBox["W", "3"]}], ")"}], 
     SubscriptBox["S", "32"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["W", "2"], "|", 
       SubscriptBox["W", "3"]}], ")"}], 
     SubscriptBox["S", "12"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["W", "2"], "|", 
       SubscriptBox["W", "1"]}], ")"}]}], "=", 
    RowBox[{
     SubscriptBox["S", "12"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["W", "2"], "|", 
       SubscriptBox["W", "1"]}], ")"}], 
     SubscriptBox["S", "32"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["W", "2"], "|", 
       SubscriptBox["W", "3"]}], ")"}], 
     SubscriptBox["S", "31"], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["W", "1"], "|", 
        SubscriptBox["W", "3"]}], ")"}], ".", " "}]}]}],
   FontFamily->"Times"]}]], "DisplayFormulaNumbered",
 CellGroupingRules->{GroupTogetherGrouping, 15001.},
 CellChangeTimes->{{3.483738652595137*^9, 3.483738744751387*^9}, {
   3.483738896173262*^9, 3.483738925767012*^9}, {3.483739028938887*^9, 
   3.483739113001387*^9}, {3.483739156157637*^9, 3.483739356548262*^9}, {
   3.483739711923262*^9, 3.483739817329512*^9}, {3.483971477953125*^9, 
   3.483971489875*^9}, {3.483971692546875*^9, 3.483971709765625*^9}, {
   3.48397179759375*^9, 3.483971806125*^9}, {3.483971851234375*^9, 
   3.483971904640625*^9}, {3.48397242009375*^9, 3.4839724281875*^9}, {
   3.484336166390625*^9, 3.484336188078125*^9}, 3.4891789656271772`*^9, {
   3.4892439399826736`*^9, 3.4892439469302177`*^9}, {3.489331150679132*^9, 
   3.4893311514197493`*^9}, {3.565713289853262*^9, 3.5657133113383913`*^9}, {
   3.565713349578238*^9, 3.5657134768650427`*^9}, {3.565713515716501*^9, 
   3.565713728029298*^9}, {3.565713766255362*^9, 3.565713793769643*^9}, {
   3.5657138537948837`*^9, 3.565713961134357*^9}, {3.565713998758895*^9, 
   3.565714027780023*^9}, {3.565714059860623*^9, 3.565714510314746*^9}, {
   3.5657146098536673`*^9, 3.5657146240297537`*^9}, {3.565714654149926*^9, 
   3.565714888389523*^9}, {3.565715313295261*^9, 3.565715328407378*^9}, {
   3.5657154341703243`*^9, 3.5657156320469837`*^9}, {3.565715662919568*^9, 
   3.565715779684214*^9}, {3.565715834261265*^9, 3.565715858138712*^9}, 
   3.565715902162878*^9, 3.565716556205474*^9, 3.5657166236760683`*^9, {
   3.565716755776536*^9, 3.5657167748299303`*^9}, 3.56571682963888*^9, {
   3.565730794596929*^9, 3.565730807482008*^9}},
 TextAlignment->Center,
 LineSpacing->{1.5, 3}],

Cell[TextData[{
 "This is Eq( 4.14) of the paper and several typical matrix elements were \
verified by Shastry by explicit calculation, who conjectured that this \
equation is true for all matrix elements. Since the number of terms is very \
large (see below), an exhaustive enumerative proof  was not given. We will \
provide such an enumerative proof using DiracQ next.\n  For ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"non", " ", "zero", " ", 
     SubscriptBox["W", "3"]}], ",", " ", 
    RowBox[{"an", "  ", "analytical", "  ", "proof", " "}]}], 
   TraditionalForm]],
  FontFamily->"Times"],
 "was actually found much later in the work of M. Shiroishi and M. Wadati - \
Journal of the Physical Society of Japan, Vol 64, 57-63 (1995), and thus the \
following enumerative proof is not the first proof. However, it does \
demonstrate the power of DiracQ quite well. The large number of terms \
involved make the algebra prohibitive by a hand calculation for most of us! \
Using the package however we are able to compute the product of three S \
operators in a number of minutes."
}], "Text",
 CellGroupingRules->{GroupTogetherGrouping, 15001.},
 CellChangeTimes->{{3.483738652595137*^9, 3.483738744751387*^9}, {
   3.483738896173262*^9, 3.483738925767012*^9}, {3.483739028938887*^9, 
   3.483739113001387*^9}, {3.483739156157637*^9, 3.483739356548262*^9}, {
   3.483739711923262*^9, 3.483739817329512*^9}, {3.483971477953125*^9, 
   3.483971489875*^9}, {3.483971692546875*^9, 3.483971709765625*^9}, {
   3.48397179759375*^9, 3.483971806125*^9}, {3.483971851234375*^9, 
   3.483971904640625*^9}, {3.48397242009375*^9, 3.4839724281875*^9}, {
   3.484336166390625*^9, 3.484336188078125*^9}, 3.4891789656271772`*^9, {
   3.4892439399826736`*^9, 3.4892439469302177`*^9}, {3.489331150679132*^9, 
   3.4893311514197493`*^9}, {3.565713289853262*^9, 3.5657133113383913`*^9}, {
   3.565713349578238*^9, 3.5657134768650427`*^9}, {3.565713515716501*^9, 
   3.565713728029298*^9}, {3.565713766255362*^9, 3.565713793769643*^9}, {
   3.5657138537948837`*^9, 3.565713961134357*^9}, {3.565713998758895*^9, 
   3.565714027780023*^9}, {3.565714059860623*^9, 3.565714510314746*^9}, {
   3.5657146098536673`*^9, 3.5657146240297537`*^9}, {3.565714654149926*^9, 
   3.565714888389523*^9}, {3.565715313295261*^9, 3.565715328407378*^9}, {
   3.5657154341703243`*^9, 3.5657156320469837`*^9}, {3.565715662919568*^9, 
   3.565715779684214*^9}, {3.565715834261265*^9, 3.565715858138712*^9}, 
   3.565715902162878*^9, 3.565716556205474*^9, 3.5657166236760683`*^9, {
   3.565716755776536*^9, 3.5657167748299303`*^9}, {3.56571682963888*^9, 
   3.5657168492981033`*^9}, {3.565716883784154*^9, 3.565716904173532*^9}, {
   3.5657169530991163`*^9, 3.565716953633337*^9}, {3.565730867988309*^9, 
   3.565730973148388*^9}, {3.565731017725053*^9, 3.5657310231616497`*^9}, {
   3.565731114399273*^9, 3.5657311146758223`*^9}, {3.566047496812446*^9, 
   3.5660475196897545`*^9}, {3.5662170342859*^9, 3.566217066960769*^9}, {
   3.5665782703831*^9, 3.566578275293947*^9}},
 LineSpacing->{1.5, 3}]
}, Closed]],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{"This", " ", "is", " ", "Eq", 
    RowBox[{"(", " ", "4.14", ")"}], " ", "of", " ", "the", " ", "paper", " ",
     "and", "   ", "several", " ", "typical", "  ", "matrix", " ", "elements",
     " ", "were", " ", "verified", " ", "by", " ", "Shastry", " ", "by", " ", 
    "explicit", " ", "calculation"}], ",", " ", 
   RowBox[{
   "who", " ", "conjectured", " ", "that", " ", "this", " ", "equation", " ", 
    "is", " ", "true", " ", "for", " ", "all", " ", "matrix", " ", 
    RowBox[{"elements", ".", "   ", "Since"}], " ", "the", " ", "number", " ",
     "of", " ", "terms", " ", "is", " ", "very", " ", "large", " ", 
    RowBox[{"(", 
     RowBox[{"see", " ", "below"}], ")"}]}], ",", " ", 
   RowBox[{
   "an", " ", "exhaustive", " ", "enumerative", " ", "proof", "  ", "was", 
    " ", "not", " ", 
    RowBox[{"given", ".", " ", "We"}], " ", "will", " ", "provide", " ", 
    "such", " ", "an", " ", "enumerative", " ", "proof", " ", "using", " ", 
    "DiracQ", " ", 
    RowBox[{"next", ".", "\n", "  ", "\n", "  ", "For"}], " ", 
    FormBox[
     RowBox[{
      RowBox[{"non", " ", "zero", " ", 
       SubscriptBox["W", 
        RowBox[{"3", " "}]]}], ",", "  ", 
      RowBox[{
      "analytical", "  ", "results", " ", "in", " ", "this", " ", "direction",
        " "}]}],
     TraditionalForm], "were", "   ", "found", "  ", "much", " ", "later", 
    " ", "in", " ", "the", " ", "work", " ", "of", "  ", "M", " ", 
    "Shiroishi", " ", "and", "  ", "M", " ", "Wadati"}], " ", ",", 
   RowBox[{
   "Journal", " ", "of", "  ", "Physical", " ", "Society", " ", "of", " ", 
    "Japan"}], ",", " ", 
   RowBox[{"Vol", " ", "64"}], ",", " ", 
   RowBox[{"2795", " ", 
    RowBox[{
     RowBox[{"(", "1995", ")"}], ".", 
     StyleBox[
      RowBox[{" ", " "}]], "The"}], " ", "calculation", " ", "below", " ", 
    "demonstrates", " ", "the", " ", "power", " ", "of", " ", "DiracQ", " ", 
    "quite", " ", 
    RowBox[{"well", ".", " ", "The"}], " ", "large", " ", "number", " ", "of",
     " ", "terms", " ", "involved", " ", "make", " ", "the", " ", "algebra", 
    " ", "prohibitive", " ", "by", " ", "a", " ", "hand", " ", "calculation", 
    " ", "for", " ", "most", " ", "of", " ", 
    RowBox[{"us", "!"}], " ", "Using", " ", "the", " ", "package", " ", 
    "however", " ", "we", " ", "are", " ", "able", " ", "to", " ", "compute", 
    " ", "the", " ", "product", " ", "of", " ", "three", " ", "S", " ", 
    "operators", " ", "in", " ", "a", " ", "number", " ", "of", " ", 
    RowBox[{"minutes", "."}]}]}],
  FontFamily->"Arial"]], "Text",
 CellChangeTimes->{{3.6482231720412397`*^9, 3.64822325418452*^9}, {
  3.6482234190022182`*^9, 3.648223435883769*^9}, {3.648223489064961*^9, 
  3.6482234994095373`*^9}, {3.648223731840888*^9, 3.6482237424747334`*^9}},
 TextJustification->1.],

Cell["\<\
 The first step is to input the form for the scattering matrix and for the S \
operators. Here, we denote the two separate species of Pauli matrices not by \
use of another symbol, but rather by using a third index to denote the \
species of the operator, as shown below.\
\>", "Text",
 CellChangeTimes->{{3.489345624263249*^9, 3.4893458107607603`*^9}, {
   3.4893458497200212`*^9, 3.4893458597676*^9}, 3.565721833463765*^9, {
   3.5657310325979548`*^9, 3.565731074860546*^9}, 3.5657311121752*^9, 
   3.566047534902624*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox["\[Sigma]", "1", "z"], "\[Rule]", 
   RowBox[{"\[Sigma]", "[", 
    RowBox[{"1", ",", "z", ",", "1"}], "]"}]}], " ", ",", " ", 
  RowBox[{
   SubsuperscriptBox["\[Tau]", "1", "z"], "\[Rule]", 
   RowBox[{"\[Sigma]", "[", 
    RowBox[{"1", ",", "z", ",", "2"}], "]"}]}]}]], "DisplayFormulaNumbered",
 CellChangeTimes->{{3.488916325515625*^9, 3.488916367875*^9}},
 TextAlignment->Center,
 FontFamily->"Times"],

Cell["\<\
In this notation the scattering matrix and the S operator are written below \
as general functions. \
\>", "Text",
 CellChangeTimes->{{3.488916383921875*^9, 3.488916438984375*^9}, {
   3.48891649396875*^9, 3.488916645609375*^9}, {3.48891736425*^9, 
   3.48891737396875*^9}, 3.48917950106975*^9, {3.489179874194969*^9, 
   3.489179874282641*^9}, {3.4893458702570066`*^9, 3.489345877599724*^9}, 
   3.565731146901017*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"A", ",", "l", ",", "S"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["l", 
     RowBox[{"u_", ",", "v_"}]], "[", 
    RowBox[{"n_", ",", "p_", ",", "q_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"A", "[", 
       RowBox[{"n", ",", "1", ",", "p", ",", "q"}], "]"}], "+", 
      RowBox[{
       RowBox[{"A", "[", 
        RowBox[{"n", ",", 
         RowBox[{"-", "1"}], ",", "p", ",", "q"}], "]"}], 
       RowBox[{
        RowBox[{"\[Sigma]", "[", 
         RowBox[{"v", ",", "z", ",", "1"}], "]"}], "**", 
        RowBox[{"\[Sigma]", "[", 
         RowBox[{"u", ",", "z", ",", "1"}], "]"}]}]}], "+", 
      RowBox[{
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"v", ",", "x", ",", "1"}], "]"}], "**", 
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"u", ",", "x", ",", "1"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"v", ",", "y", ",", "1"}], "]"}], "**", 
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"u", ",", "y", ",", "1"}], "]"}]}]}], ")"}], "\[CircleTimes]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"A", "[", 
       RowBox[{"n", ",", "1", ",", "p", ",", "q"}], "]"}], "+", 
      RowBox[{
       RowBox[{"A", "[", 
        RowBox[{"n", ",", 
         RowBox[{"-", "1"}], ",", "p", ",", "q"}], "]"}], 
       RowBox[{
        RowBox[{"\[Sigma]", "[", 
         RowBox[{"v", ",", "z", ",", "2"}], "]"}], "**", 
        RowBox[{"\[Sigma]", "[", 
         RowBox[{"u", ",", "z", ",", "2"}], "]"}]}]}], "+", 
      RowBox[{
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"v", ",", "x", ",", "2"}], "]"}], "**", 
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"u", ",", "x", ",", "2"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"v", ",", "y", ",", "2"}], "]"}], "**", 
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"u", ",", "y", ",", "2"}], "]"}]}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["S", 
   RowBox[{"u_", ",", "v_"}]], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{
      SubscriptBox["\[Theta]", "v"], "+", 
      SubscriptBox["\[Theta]", "u"]}], "]"}], 
    RowBox[{"Cosh", "[", 
     RowBox[{
      SubscriptBox["h", "v"], "-", 
      SubscriptBox["h", "u"]}], "]"}], 
    RowBox[{
     SubscriptBox["l", 
      RowBox[{"u", ",", "v"}]], "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", 
      SubscriptBox["\[Theta]", "v"], ",", 
      SubscriptBox["\[Theta]", "u"]}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{
      SubscriptBox["\[Theta]", "v"], "-", 
      SubscriptBox["\[Theta]", "u"]}], "]"}], 
    RowBox[{"Sinh", "[", 
     RowBox[{
      SubscriptBox["h", "v"], "-", 
      SubscriptBox["h", "u"]}], "]"}], 
    RowBox[{
     SubscriptBox["l", 
      RowBox[{"u", ",", "v"}]], "[", 
     RowBox[{"1", ",", 
      SubscriptBox["\[Theta]", "v"], ",", 
      SubscriptBox["\[Theta]", "u"]}], "]"}], 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"v", ",", "z", ",", "1"}], "]"}], " ", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"v", ",", "z", ",", "2"}], "]"}]}]}]}]}], "Input"],

Cell[TextData[{
 "The Boltzmann weights in the scattering matrix are not written explicitly \
because this increases the number of terms involved in the product and their \
definition will not be relevant until after the S operator products have been \
calculated. They are therefore represented by the function A, which will be \
defined after the product of the three S operators on each side of (36) has \
been computed. We will set ",
 Cell[BoxData[
  RowBox[{"rulesimplify", "=", " ", 
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["h", "i_"], "->", 
      RowBox[{
       RowBox[{"1", "/", "2"}], 
       RowBox[{"ArcSinh", "[", 
        RowBox[{
         RowBox[{"U", "/", "2"}], " ", 
         RowBox[{"Cos", "[", 
          SubscriptBox["\[Theta]", "i"], "]"}], " ", 
         RowBox[{"Sin", "[", 
          SubscriptBox["\[Theta]", "i"], "]"}]}], "]"}]}]}], ",", " ", 
     RowBox[{
      RowBox[{"A", "[", 
       RowBox[{"n_", ",", "m_", ",", "p_", ",", "q_"}], "]"}], "->", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"p", "+", 
         RowBox[{"n", " ", "q"}]}], "]"}], "+", 
       RowBox[{"m", " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"p", "+", 
          RowBox[{"n", " ", "q"}]}], "]"}]}]}]}]}], "}"}]}]],
  CellChangeTimes->{{3.565710345339984*^9, 3.565710381064516*^9}},
  FontSize->14],
 ",\nafter the calculation with the operators is performed, and it is easy to \
see that this replacement reproduces the scattering operator above with a \
scale factor of 2 which cancels out. Here we compute the product of three S , \
matrices. Each S matrix contains about thirty terms, we we wexpect the \
product to contain on the order of 27,000 terms. This expression is so large \
as to be prohibitive to compute by hand."
}], "Text",
 CellChangeTimes->{{3.490547138002001*^9, 3.490547221770771*^9}, {
  3.565731151537969*^9, 3.565731153276984*^9}, {3.565731212464575*^9, 
  3.565731315288718*^9}, {3.565731464866725*^9, 3.5657314932388983`*^9}, {
  3.5660468462792377`*^9, 3.566046851020509*^9}, {3.566047902868671*^9, 
  3.566047912946247*^9}, {3.566047952247495*^9, 3.5660479680944014`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"time", ",", "LHS"}], "}"}], "=", 
   RowBox[{"Timing", "[", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{"3", ",", "1"}]], "\[CircleTimes]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"3", ",", "2"}]], "\[CircleTimes]", 
       SubscriptBox["S", 
        RowBox[{"1", ",", "2"}]]}], ")"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"time", "/", "60"}]], "Input"],

Cell[BoxData["2.791116666666667`"], "Output",
 CellChangeTimes->{{3.566047202024585*^9, 3.566047222149736*^9}, 
   3.566132721768622*^9, 3.566133430146139*^9}]
}, Closed]],

Cell[TextData[StyleBox["The timing function informs us that this operation \
takes around three minutes on our testing machine.",
 FontSize->12,
 FontWeight->"Plain"]], "Text",
 CellChangeTimes->{{3.48891697115625*^9, 3.488916991703125*^9}, {
  3.489165526865614*^9, 3.4891655292390747`*^9}, {3.4948703769430313`*^9, 
  3.4948703776622343`*^9}, {3.5657315194657307`*^9, 3.56573152286477*^9}},
 FontSize->14,
 FontWeight->"Bold"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "LHS", "]"}]], "Input"],

Cell[BoxData["27360"], "Output",
 CellChangeTimes->{3.5660471935320992`*^9, 3.5661327271369286`*^9, 
  3.566133431713228*^9}]
}, Closed]],

Cell["\<\
We see that there are approximately 32,000 terms in the final product. Below \
an arbitrary example of a single term is shown. We expect similar timing and \
length for the RHS.\
\>", "Text",
 CellChangeTimes->{{3.488917002953125*^9, 3.488917102109375*^9}, {
  3.490547251921204*^9, 3.4905472557629747`*^9}, {3.4905473022964354`*^9, 
  3.490547309862192*^9}, {3.566046880731208*^9, 3.5660468858585014`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LHS", "[", 
  RowBox[{"[", "6", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5660468896257167`*^9, 3.5660469076107454`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
  SuperscriptBox[
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], "2"], " ", 
  SuperscriptBox[
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
  RowBox[{"Cos", "[", 
   RowBox[{
    SubscriptBox["\[Theta]", "1"], "+", 
    SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
  RowBox[{"Cos", "[", 
   RowBox[{
    SubscriptBox["\[Theta]", "1"], "+", 
    SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
  RowBox[{"Cos", "[", 
   RowBox[{
    SubscriptBox["\[Theta]", "2"], "+", 
    SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
  RowBox[{"Cosh", "[", 
   RowBox[{
    SubscriptBox["h", "1"], "-", 
    SubscriptBox["h", "2"]}], "]"}], " ", 
  RowBox[{"Cosh", "[", 
   RowBox[{
    SubscriptBox["h", "1"], "-", 
    SubscriptBox["h", "3"]}], "]"}], " ", 
  RowBox[{"Cosh", "[", 
   RowBox[{
    SubscriptBox["h", "2"], "-", 
    SubscriptBox["h", "3"]}], "]"}]}]], "Output",
 CellChangeTimes->{{3.566046896385103*^9, 3.566046908441793*^9}, 
   3.566132736141444*^9, 3.5661334378505793`*^9}]
}, Closed]],

Cell["We now compute the RHS", "Text",
 CellChangeTimes->{{3.490547267721418*^9, 3.490547274454812*^9}, {
  3.492974860003249*^9, 3.492974868247252*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"RHS", "=", 
   RowBox[{
    SubscriptBox["S", 
     RowBox[{"1", ",", "2"}]], "\[CircleTimes]", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"3", ",", "2"}]], "\[CircleTimes]", 
      SubscriptBox["S", 
       RowBox[{"3", ",", "1"}]]}], ")"}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "RHS", "]"}]], "Input",
 CellChangeTimes->{{3.5661330894676533`*^9, 3.5661330933518753`*^9}}],

Cell[BoxData["27360"], "Output",
 CellChangeTimes->{3.566133093716896*^9, 3.566133613431622*^9}]
}, Closed]],

Cell["\<\
We have now computed both the left and right hand sides of equation 13. If \
the equation holds, they are equal. We verify this by subtracting the two and \
returnign zero.\
\>", "Text",
 CellChangeTimes->{{3.566047711978753*^9, 3.566047769225027*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"subtraction", "=", 
   RowBox[{"LHS", "-", "RHS"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5661330999002495`*^9, 3.566133108781758*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "subtraction", "]"}]], "Input",
 CellChangeTimes->{{3.566133115268129*^9, 3.5661331228155603`*^9}}],

Cell[BoxData["13472"], "Output",
 CellChangeTimes->{3.5661331234095945`*^9, 3.5661336843226767`*^9}]
}, Closed]],

Cell["\<\
We would have hoped that when we subtracted the LHS from the RHS that the \
result was zero. It appears we will have to do more manipulation to show that \
the LHS is equal to the RHS. We now have a very large expression, and we need \
to manipulate it in several ways. We need to find all distinct strings of \
operators and then find the coefficient of all of each string. We must then \
show that every one of these coefficients vanishes. DiracQ contains functions \
that will perform the first two. However, were we to use the functions \
outright the computation time would be very long. Every expression input into \
a DiracQ function is first organized into lists that can be manipulated (see \
the \[OpenCurlyQuote] Brief Explanation of Form\[CloseCurlyQuote] section). \
For a large expression such as the one we are working with, organizing the \
expression takes a very long time. The expression must then be recompiled \
into and understandable form. We can circumvent some of these steps by \
organizing an expression once and specifiying to subsequent functions that \
the input has already been organized and that this step should be skipped. \
This is done by specifying\[CloseCurlyQuote] Organized Expression -> True\
\[CloseCurlyQuote] as an option. This process is demonstrated below.\
\>", "Text",
 CellChangeTimes->{{3.566134048824525*^9, 3.5661341089199624`*^9}, {
  3.5662171214878883`*^9, 3.5662171649963765`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Organize"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"Organize is the function that enables the DiracQ package to \
understand user input. Organize takes a mathematical expression as input and \
yields a nested list that contains the atoms of the input ordered according \
to their properties. Numbers, summed indices, c numbers, and q numbers are \
separated into groups. Each term of the input separated by plus sign \
constitutes a separate list of items in the output. Example:\\nOrganize[(#) \
\\!\\(\\*UnderscriptBox[\\\"\[Sum]\\\", \\\"index\\\"]\\)(c #)*(q \
#)]\\n={{#,{index},{c #},{q #}}}\\n\\nOrganize[(\\!\\(\\*SubscriptBox[\\\"#\\\
\", \\\"1\\\"]\\)) \\!\\(\\*UnderscriptBox[\\\"\[Sum]\\\", \
SubscriptBox[\\\"index\\\", \\\"1\\\"]]\\)(c \\!\\(\\*SubscriptBox[\\\"#\\\", \
\\\"1\\\"]\\))*(q \\!\\(\\*SubscriptBox[\\\"#\\\", \
\\\"1\\\"]\\))+(\\!\\(\\*SubscriptBox[\\\"#\\\", \\\"2\\\"]\\)) \
\\!\\(\\*UnderscriptBox[\\\"\[Sum]\\\", SubscriptBox[\\\"index\\\", \
\\\"2\\\"]]\\)(c \\!\\(\\*SubscriptBox[\\\"#\\\", \\\"2\\\"]\\))*(q \
\\!\\(\\*SubscriptBox[\\\"#\\\", \
\\\"2\\\"]\\))]\\n={{\\!\\(\\*SubscriptBox[\\\"#\\\", \
\\\"1\\\"]\\),{\\!\\(\\*SubscriptBox[\\\"index\\\", \\\"1\\\"]\\)},{c \
\\!\\(\\*SubscriptBox[\\\"#\\\", \\\"1\\\"]\\)},{q \
\\!\\(\\*SubscriptBox[\\\"#\\\", \\\"1\\\"]\\)}},{\\!\\(\\*SubscriptBox[\\\"#\
\\\", \\\"2\\\"]\\),{\\!\\(\\*SubscriptBox[\\\"index\\\", \\\"2\\\"]\\)},{c \
\\!\\(\\*SubscriptBox[\\\"#\\\", \\\"2\\\"]\\)},{q \
\\!\\(\\*SubscriptBox[\\\"#\\\", \\\"2\\\"]\\)}}}\\nFor a more in depth \
explanation see the DiracQ writeup notebook.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.566134131639262*^9},
 CellTags->"Info3566105331-6477710"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"time", ",", "organizedsubtraction"}], "}"}], "=", 
   RowBox[{"Timing", "[", 
    RowBox[{"Organize", "[", "subtraction", "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5661337181076093`*^9, 3.566133774228819*^9}, 
   3.566133902800173*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"time", "/", "60"}]], "Input",
 CellChangeTimes->{{3.56613400206085*^9, 3.566134003894955*^9}}],

Cell[BoxData["1.3767166666666668`"], "Output",
 CellChangeTimes->{3.566134004286978*^9}]
}, Closed]],

Cell["\<\
We see that on our test machine the Organize function takes  a few minutes to \
evaluate and is computationally quite expensive. Below we use the TakeQPart \
function to find the strings of operators that appear in \
OrganizedSubtraction, specifying that the input is already an organized \
expression to save some computational time.\
\>", "Text",
 CellChangeTimes->{{3.5661340189798183`*^9, 3.5661340255471935`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"OperatorTerms", "=", 
   RowBox[{"Union", "[", 
    RowBox[{"TakeQPart", "[", 
     RowBox[{"organizedsubtraction", ",", 
      RowBox[{"OrganizedExpression", "\[Rule]", "True"}]}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5661341422518687`*^9, 3.5661341443249874`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "OperatorTerms", "]"}]], "Input"],

Cell[BoxData["472"], "Output",
 CellChangeTimes->{3.5661341729796267`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OperatorTerms", "[", 
  RowBox[{"[", "300", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Sigma]", "[", 
   RowBox[{"1", ",", "y", ",", "1"}], "]"}], "**", 
  RowBox[{"\[Sigma]", "[", 
   RowBox[{"2", ",", "z", ",", "1"}], "]"}], "**", 
  RowBox[{"\[Sigma]", "[", 
   RowBox[{"3", ",", "x", ",", "1"}], "]"}], "**", 
  RowBox[{"\[Sigma]", "[", 
   RowBox[{"2", ",", "z", ",", "2"}], "]"}], "**", 
  RowBox[{"\[Sigma]", "[", 
   RowBox[{"3", ",", "z", ",", "2"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5661341829121943`*^9}]
}, Closed]],

Cell["\<\
The above output shows that there are 472 distinct strings of operators found \
in the expression and an example of one such string. We can now use the \
QCoefficient function to find the coefficients of a string of operators.\
\>", "Text",
 CellChangeTimes->{{3.491837431585552*^9, 3.491837458973323*^9}, {
   3.494865573174604*^9, 3.494865573761682*^9}, {3.4948668546461563`*^9, 
   3.4948668763812313`*^9}, {3.565734325378162*^9, 3.565734334525016*^9}, 
   3.566134229074835*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QCoefficient"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"QCoeffficient[expression,form] will scan the expression for \
terms containing a string of operators that match 'form'. The function will \
output the coefficient of the operator(s) specified by 'form'. Only exact \
matches of the string of operators are found. If several terms are found in \
expression containing terms that match 'form' the output will be a sum of the \
coefficients of the specified operators.\"\>", "MSG"]], "Print", "PrintUsage",
 
 CellChangeTimes->{3.5661342323220205`*^9},
 CellTags->"Info3566105432-6477710"]
}, Closed]],

Cell["\<\
The result is not printed due to the length and complexity, though we do show \
the first term in the expression. See the appendix section, or remove the \
semicolon at the end to see the expression.\
\>", "Text",
 CellChangeTimes->{{3.5661346313988466`*^9, 3.5661347061091194`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sum", "[", "1", "]"}], "=", 
   RowBox[{"QCoefficient", "[", 
    RowBox[{"organizedsubtraction", ",", 
     RowBox[{"OperatorTerms", "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"OrganizedExpression", "\[Rule]", "True"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.566134255771362*^9, 3.5661342583375087`*^9}, 
   3.5661346075694838`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"sum", "[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5661346096156006`*^9, 3.5661346157299504`*^9}}],

Cell[BoxData["24"], "Output",
 CellChangeTimes->{3.5661346161009717`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sum", "[", "1", "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5661346817357254`*^9, 3.5661346845118847`*^9}}],

Cell[BoxData[
 RowBox[{"4", " ", 
  RowBox[{"A", "[", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", "1"}], ",", 
    SubscriptBox["\[Theta]", "2"], ",", 
    SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
  RowBox[{"A", "[", 
   RowBox[{"1", ",", 
    RowBox[{"-", "1"}], ",", 
    SubscriptBox["\[Theta]", "1"], ",", 
    SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
  RowBox[{"Cos", "[", 
   RowBox[{
    SubscriptBox["\[Theta]", "1"], "-", 
    SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
  RowBox[{"Cos", "[", 
   RowBox[{
    SubscriptBox["\[Theta]", "1"], "-", 
    SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
  RowBox[{"Cos", "[", 
   RowBox[{
    SubscriptBox["\[Theta]", "2"], "+", 
    SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
  RowBox[{"Cosh", "[", 
   RowBox[{
    SubscriptBox["h", "2"], "-", 
    SubscriptBox["h", "3"]}], "]"}], " ", 
  RowBox[{"Sinh", "[", 
   RowBox[{
    SubscriptBox["h", "1"], "-", 
    SubscriptBox["h", "2"]}], "]"}], " ", 
  RowBox[{"Sinh", "[", 
   RowBox[{
    SubscriptBox["h", "1"], "-", 
    SubscriptBox["h", "3"]}], "]"}]}]], "Output",
 CellChangeTimes->{3.5661346847929*^9}]
}, Closed]],

Cell[TextData[StyleBox["We now apply the definition of the Boltzmann weights.",
 FontSize->12,
 FontWeight->"Plain"]], "Text",
 CellChangeTimes->{{3.488917146890625*^9, 3.48891719559375*^9}, {
   3.488917436546875*^9, 3.4889174371875*^9}, 3.489179502028809*^9, 
   3.494866910757257*^9},
 FontSize->10,
 FontWeight->"Bold"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rulesimplify", "=", " ", 
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["h", "i_"], "->", 
      RowBox[{
       RowBox[{"1", "/", "2"}], 
       RowBox[{"ArcSinh", "[", 
        RowBox[{
         RowBox[{"U", "/", "2"}], " ", 
         RowBox[{"Cos", "[", 
          SubscriptBox["\[Theta]", "i"], "]"}], " ", 
         RowBox[{"Sin", "[", 
          SubscriptBox["\[Theta]", "i"], "]"}]}], "]"}]}]}], ",", " ", 
     RowBox[{
      RowBox[{"A", "[", 
       RowBox[{"n_", ",", "m_", ",", "p_", ",", "q_"}], "]"}], "->", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"p", "+", 
         RowBox[{"n", " ", "q"}]}], "]"}], "+", 
       RowBox[{"m", " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"p", "+", 
          RowBox[{"n", " ", "q"}]}], "]"}]}]}]}]}], "}"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"sum", "[", "1", "]"}], "/.", "rulesimplify"}], "]"}]], "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.566134325512351*^9}]
}, Closed]],

Cell["\<\
We see that the coefficients of this one string of operators simplify to \
zero. We must show that each such term vanishes for all 472 strings of \
operators. The command below will evaluate each such term. \
\>", "Text",
 CellChangeTimes->{{3.494865817235071*^9, 3.494865933144792*^9}, {
  3.565733076994487*^9, 3.565733125090851*^9}, {3.565734007323831*^9, 
  3.565734010034779*^9}, {3.5661343623644586`*^9, 3.5661343875719004`*^9}}],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sum", "[", "i", "]"}], "=", 
    RowBox[{"Simplify", "[", 
     RowBox[{
      RowBox[{"QCoefficient", "[", 
       RowBox[{"organizedsubtraction", ",", 
        RowBox[{"OperatorTerms", "[", 
         RowBox[{"[", "i", "]"}], "]"}], ",", 
        RowBox[{"OrganizedExpression", "\[Rule]", "True"}]}], "]"}], "/.", 
      "rulesimplify"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "472"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.566134393112217*^9, 3.5661344055049257`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"sum", "[", "i", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "472"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5661345415047045`*^9}]
}, Closed]],

Cell["\<\
Thus we see that every term simplifies to zero and thus proves the result.\
\>", "Text",
 CellChangeTimes->{{3.566134461634136*^9, 3.5661344679364967`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["(VII) Hubbard Model Currents", "Subsection",
 CellChangeTimes->{{3.491143566063478*^9, 3.491143572370645*^9}, {
   3.5304120854544487`*^9, 3.530412086523321*^9}, 3.565753731131138*^9, {
   3.565836403825985*^9, 3.565836416286583*^9}, {3.5670078911606827`*^9, 
   3.567007928212603*^9}},
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{"\t", 
  RowBox[{
   RowBox[{
   "Below", " ", "we", " ", "define", " ", "the", " ", "Hamiltonian", " ", 
    "of", " ", "the", " ", "Hubbard", " ", "model", " ", "using", " ", 
    "fermionic", " ", 
    RowBox[{"operators", ".", " ", "The"}], " ", "system", " ", "is", " ", 
    "taken", " ", "to", " ", "have", " ", "periodic", " ", "boundary", " ", 
    "conditions", " ", "such", " ", "that", " ", 
    RowBox[{"(", 
     RowBox[{"N", "+", "a"}], ")"}], " ", "a", " ", "where", " ", "N", " ", 
    "is", " ", "the", " ", "number", " ", "of", " ", 
    RowBox[{"sites", ".", " ", "We"}], " ", "also", " ", "write", " ", "two", 
    " ", "currents", " ", "which", " ", "have", " ", "been", " ", "shown", 
    " ", "to", " ", "commute", " ", "with", " ", "the", " ", "Hamiltonian", 
    " ", "in", " ", "the", " ", "papers", " ", 
    RowBox[{"B", ".", "S", ".", "Shastry"}], " ", 
    RowBox[{"Phys", ".", " ", "Rev", ".", " ", "Letts", ".", " ", "vol"}], 
    " ", "56"}], ",", " ", "1529", ",", 
   RowBox[{"2453", " ", 
    RowBox[{"(", "1986", ")"}], " ", "and", " ", "especially", " ", 
    RowBox[{"J", ".", "Stat", ".", "Phys"}]}], ",", " ", 
   RowBox[{"vol", " ", "50"}], ",", 
   RowBox[{"57", " ", 
    RowBox[{
     RowBox[{"(", "1988", ")"}], ".", " ", "Here"}], " ", "it", " ", "is", 
    " ", "shown", " ", "that", " ", "for", " ", "a", " ", "system", " ", 
    "with", " ", "odd", " ", "number", " ", "of", " ", "sites", " ", "we", 
    " ", "have", " ", "a", " ", "conservation", " ", "law", " ", "termed", 
    " ", 
    SubscriptBox["j", "A"], 
    RowBox[{"(", 
     RowBox[{"defined", " ", "below"}], ")"}], " ", "that", " ", "stands", 
    " ", "apart", " ", "from", " ", "the", " ", "other", " ", "conservation", 
    " ", 
    RowBox[{"laws", ".", " ", "A"}], " ", "current", " ", 
    SubscriptBox["j", "B"], " ", "is", " ", "also", " ", "found", " ", 
    RowBox[{"(", 
     RowBox[{"defined", " ", "below"}], ")"}], " ", "and", "  ", "unlike", 
    " ", 
    SubscriptBox["j", "A"], " ", "generalizes", " ", "to", " ", "many", " ", 
    "more", " ", "currents", " ", "since", " ", "it", " ", "is", " ", "the", 
    " ", "first", " ", "non", " ", "trivial", " ", "member", " ", "of", " ", 
    "the", " ", "currents", " ", "contained", " ", "in", " ", "the", " ", 
    "transfer", " ", 
    RowBox[{"matrix", ".", " ", "Note"}], " ", "that", " ", 
    SubscriptBox["j", "A"], " ", "corresponds", " ", "to", " ", "infinite", 
    " ", "ranged", " ", "hopping", " ", "of", " ", "the", " ", "doubly", " ", 
    "occupied", " ", "sites", " ", "adding", " ", "to", " ", "short", " ", 
    "ranged", " ", "hops", " ", "of", " ", "fermions"}], ",", " ", 
   RowBox[{"whereas", " ", 
    SubscriptBox["j", "B"], " ", "corresponds", " ", "to", " ", "second", " ",
     "neighbor", " ", 
    RowBox[{"hopping", ".", "\[IndentingNewLine]", "We"}], " ", "will", " ", 
    "now", " ", "verify", "  ", "the", " ", "commutation", " ", "of", " ", 
    SubscriptBox["j", "A"], " ", "for", " ", "odd", " ", "number", " ", "of", 
    " ", "sites", "  ", "using", " ", "DiracQ", " ", "and", " ", "the", " ", 
    "commutation", " ", "of", " ", 
    SubscriptBox["j", 
     RowBox[{"B", " "}]], "for", " ", "both", " ", "odd", " ", "and", " ", 
    "even", " ", 
    RowBox[{"sites", ".", " ", "Notice"}], " ", "that", " ", "we", " ", 
    "write", " ", "the", " ", "Hamiltonian", " ", "as", " ", "well", " ", 
    "as", " ", "the", " ", "currents", " ", "as", " ", "functions", " ", "of",
     " ", "the", " ", "number", " ", "of", " ", "sites", " ", "NN", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
      "N", " ", "is", " ", "a", " ", "predefined", " ", "symbol", " ", "in", 
       " ", "Mathematica"}], ")"}], ".", " ", "This"}], " ", "will", " ", 
    "allow", " ", "us", " ", "to", " ", "vary", " ", "the", " ", "number", 
    " ", "of", " ", "sites", " ", "between", " ", "even", " ", "and", " ", 
    "odd", " ", 
    RowBox[{"values", "."}]}]}]}]], "Text",
 CellChangeTimes->{{3.491152184966669*^9, 3.4911521861525307`*^9}, {
  3.4911523840579233`*^9, 3.491152484701282*^9}, {3.565753781377686*^9, 
  3.565754044450807*^9}, {3.5657541178683567`*^9, 3.5657542407687597`*^9}, {
  3.5661289001700387`*^9, 3.5661289109446554`*^9}, {3.5662172248398*^9, 
  3.566217362492673*^9}},
 TextAlignment->Left,
 FontFamily->"Times"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "H", "]"}], ";", 
  RowBox[{"Remove", "[", "j", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"H", "[", "NN_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "t"}], 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"n", "=", "1"}], "NN"], 
     RowBox[{"Sum", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"f\[Dagger]", "[", 
           RowBox[{
            RowBox[{"n", "+", "1"}], ",", "\[Sigma]"}], "]"}], "**", 
          RowBox[{"f", "[", 
           RowBox[{"n", ",", "\[Sigma]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"f\[Dagger]", "[", 
           RowBox[{"n", ",", "\[Sigma]"}], "]"}], "**", 
          RowBox[{"f", "[", 
           RowBox[{
            RowBox[{"n", "+", "1"}], ",", "\[Sigma]"}], "]"}]}]}], ")"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Sigma]", ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "2"}], "}"}]}], "]"}]}]}], "+", 
   RowBox[{"U", " ", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"m", "=", "1"}], "NN"], 
     RowBox[{
      RowBox[{
       SubscriptBox["n", "f"], "[", 
       RowBox[{"m", ",", "1"}], "]"}], "**", 
      RowBox[{
       SubscriptBox["n", "f"], "[", 
       RowBox[{"m", ",", 
        RowBox[{"-", "1"}]}], "]"}]}]}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["j", "A"], "[", "NN_", "]"}], ":=", 
   RowBox[{
    RowBox[{"\[ImaginaryI]", " ", "t", " ", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"l", "=", "1"}], "NN"], 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"f\[Dagger]", "[", 
             RowBox[{"l", ",", "\[Sigma]"}], "]"}], "**", 
            RowBox[{"f", "[", 
             RowBox[{
              RowBox[{"1", "+", "l"}], ",", "\[Sigma]"}], "]"}]}]}], "+", 
          RowBox[{
           RowBox[{"f\[Dagger]", "[", 
            RowBox[{
             RowBox[{"1", "+", "l"}], ",", "\[Sigma]"}], "]"}], "**", 
           RowBox[{"f", "[", 
            RowBox[{"l", ",", "\[Sigma]"}], "]"}]}]}], ")"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[Sigma]", ",", 
          RowBox[{"-", "1"}], ",", "1", ",", "2"}], "}"}]}], "]"}]}]}], "+", 
    RowBox[{"\[ImaginaryI]", " ", "U", " ", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"r", "=", "1"}], "NN"], 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"l", "=", "1"}], 
        RowBox[{"NN", "-", "1"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "l"], " ", 
        RowBox[{
         RowBox[{"f\[Dagger]", "[", 
          RowBox[{
           RowBox[{"l", "+", "r"}], ",", "1"}], "]"}], "**", 
         RowBox[{"f\[Dagger]", "[", 
          RowBox[{
           RowBox[{"l", "+", "r"}], ",", 
           RowBox[{"-", "1"}]}], "]"}], "**", 
         RowBox[{"f", "[", 
          RowBox[{"r", ",", 
           RowBox[{"-", "1"}]}], "]"}], "**", 
         RowBox[{"f", "[", 
          RowBox[{"r", ",", "1"}], "]"}]}]}]}]}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["j", "B"], "[", "NN_", "]"}], ":=", 
   RowBox[{
    RowBox[{"\[ImaginaryI]", " ", "t", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"o", "=", "1"}], "NN"], 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"f\[Dagger]", "[", 
           RowBox[{
            RowBox[{"o", "+", "2"}], ",", "\[Sigma]"}], "]"}], "**", 
          RowBox[{"f", "[", 
           RowBox[{"o", ",", "\[Sigma]"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{"f\[Dagger]", "[", 
           RowBox[{"o", ",", "\[Sigma]"}], "]"}], "**", 
          RowBox[{"f", "[", 
           RowBox[{
            RowBox[{"o", "+", "2"}], ",", "\[Sigma]"}], "]"}]}]}], ",", 
        RowBox[{"{", 
         RowBox[{"\[Sigma]", ",", 
          RowBox[{"-", "1"}], ",", "1", ",", "2"}], "}"}]}], "]"}]}]}], "+", 
    RowBox[{"\[ImaginaryI]", " ", "U", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"o", "=", "1"}], "NN"], 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"f\[Dagger]", "[", 
           RowBox[{
            RowBox[{"o", "+", "1"}], ",", "\[Sigma]"}], "]"}], "**", 
          RowBox[{"f", "[", 
           RowBox[{"o", ",", "\[Sigma]"}], "]"}]}], "-", 
         RowBox[{
          RowBox[{"f\[Dagger]", "[", 
           RowBox[{"o", ",", "\[Sigma]"}], "]"}], "**", 
          RowBox[{"f", "[", 
           RowBox[{
            RowBox[{"o", "+", "1"}], ",", "\[Sigma]"}], "]"}]}]}], ",", 
        RowBox[{"{", 
         RowBox[{"\[Sigma]", ",", 
          RowBox[{"-", "1"}], ",", "1", ",", "2"}], "}"}]}], "]"}]}]}], "+", 
    RowBox[{"\[ImaginaryI]", " ", "U", " ", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"o", "=", "1"}], "NN"], 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"f\[Dagger]", "[", 
             RowBox[{"o", ",", "\[Sigma]"}], "]"}], "**", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"f", "[", 
               RowBox[{
                RowBox[{"o", "+", "1"}], ",", "\[Sigma]"}], "]"}], "-", 
              RowBox[{"f", "[", 
               RowBox[{
                RowBox[{"o", "-", "1"}], ",", "\[Sigma]"}], "]"}]}], ")"}]}], 
           "-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"f\[Dagger]", "[", 
               RowBox[{
                RowBox[{"o", "+", "1"}], ",", "\[Sigma]"}], "]"}], "-", 
              RowBox[{"f\[Dagger]", "[", 
               RowBox[{
                RowBox[{"o", "-", "1"}], ",", "\[Sigma]"}], "]"}]}], ")"}], "**", 
            RowBox[{"f", "[", 
             RowBox[{"o", ",", "\[Sigma]"}], "]"}]}]}], ")"}], 
         RowBox[{
          SubscriptBox["n", "f"], "[", 
          RowBox[{"o", ",", 
           RowBox[{"-", "\[Sigma]"}]}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"\[Sigma]", ",", 
          RowBox[{"-", "1"}], ",", "1", ",", "2"}], "}"}]}], "]"}]}]}]}]}], 
  ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{"    ", 
  RowBox[{
   RowBox[{
   "First", " ", "we", " ", "will", " ", "perform", " ", "the", " ", 
    "commutator", " ", "of", " ", "the", " ", "Hamiltonian", " ", "with", " ", 
    SubscriptBox["j", "A"], " ", "for", " ", "a", " ", "system", " ", "with", 
    " ", "5", " ", 
    RowBox[{"sites", ".", " ", "As"}], " ", "the", " ", "number", " ", "of", 
    " ", "sites", " ", "increases", " ", "so", " ", "does", " ", "the", " ", 
    "number", " ", "of", " ", "operations", " ", "performed"}], ",", " ", 
   RowBox[{
   "and", " ", "therefore", " ", "we", " ", "will", " ", "only", " ", 
    "perform", " ", "these", " ", "commutators", " ", "for", " ", 
    "relatively", " ", "small", " ", 
    RowBox[{"systems", ".", " ", "Even"}], " ", "so", " ", "the", " ", 
    "computing", " ", "time", " ", "is", " ", 
    RowBox[{"nonnegligible", ".", " ", "We"}], " ", "must", " ", "include", 
    " ", "the", " ", "periodic", " ", "boundary", " ", "conditions", " ", 
    "of", " ", "our", " ", 
    RowBox[{"system", ".", " ", "These"}], " ", "are", " ", "written", " ", 
    "as", " ", "replacement", " ", 
    RowBox[{"rules", ".", " ", "The"}], " ", "replacement", " ", "rules", " ",
     "must", " ", "be", " ", "specified", " ", "to", " ", "each", " ", 
    "term"}], ",", " ", 
   RowBox[{
   "and", " ", "cannot", " ", "be", " ", "specified", " ", "at", " ", "the", 
    " ", "end", " ", "of", " ", "computation"}], ",", 
   RowBox[{
   "as", " ", "this", " ", "will", " ", "lead", " ", "to", " ", "an", " ", 
    "erroneous", " ", 
    RowBox[{"answer", "."}]}]}]}]], "Text",
 CellChangeTimes->{{3.491151995072997*^9, 3.491152136941374*^9}, {
   3.491152189643985*^9, 3.491152192727055*^9}, {3.4911522354200773`*^9, 
   3.4911523645341053`*^9}, {3.565754273819085*^9, 3.565754281303179*^9}, 
   3.5660480290308867`*^9, {3.5662173759764442`*^9, 3.566217387085079*^9}},
 FontFamily->"Times"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PBCrule", "[", "n_", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"i_", ",", "a_"}], "]"}], "/;", 
      RowBox[{"i", ">", "n"}]}], "\[Rule]", 
     RowBox[{"f", "[", 
      RowBox[{
       RowBox[{"i", "-", "n"}], ",", "a"}], "]"}]}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"i_", ",", "a_"}], "]"}], "/;", 
      RowBox[{"i", "<", "1"}]}], "\[Rule]", 
     RowBox[{"f", "[", 
      RowBox[{
       RowBox[{"i", "+", "n"}], ",", "a"}], "]"}]}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"f\[Dagger]", "[", 
       RowBox[{"i_", ",", "a_"}], "]"}], "/;", 
      RowBox[{"i", ">", "n"}]}], "\[Rule]", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{
       RowBox[{"i", "-", "n"}], ",", "a"}], "]"}]}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"f\[Dagger]", "[", 
       RowBox[{"i_", ",", "a_"}], "]"}], "/;", 
      RowBox[{"i", "<", "1"}]}], "\[Rule]", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{
       RowBox[{"i", "+", "n"}], ",", "a"}], "]"}]}]}], "}"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"H", "[", "5", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "5", "]"}]}], ",", 
    RowBox[{
     RowBox[{
      SubscriptBox["j", "A"], "[", "5", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "5", "]"}]}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5661287594219885`*^9, 3.566128840827645*^9}, {
  3.5661312417489696`*^9, 3.5661312433080587`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"20.778999999999996`", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.566128855466482*^9, 3.5661312646182775`*^9}]
}, Closed]],

Cell["\<\
Performing the same computation with a four site sytem we see that the result \
is nonvanishing, as we expected.\
\>", "Text",
 CellChangeTimes->{{3.49115220142973*^9, 3.491152231873089*^9}, {
  3.5657543482886877`*^9, 3.565754366375931*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"H", "[", "4", "]"}], "/.", 
    RowBox[{"PBCrule", "[", "4", "]"}]}], ",", 
   RowBox[{
    RowBox[{
     SubscriptBox["j", "A"], "[", "4", "]"}], "/.", 
    RowBox[{"PBCrule", "[", "4", "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"4", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"4", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", "1"}], "]"}]}]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"4", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", "1"}], "]"}]}]}], "+", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t", " ", "U"}], ")"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"4", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", "1"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.5661312876685963`*^9}]
}, Closed]],

Cell["\<\
Again, a system with an odd number of sites commutes with the Hamiltonian.\
\>", "Text",
 CellChangeTimes->{{3.491152508032249*^9, 3.49115253006789*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"H", "[", "7", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "7", "]"}]}], ",", 
    RowBox[{
     RowBox[{
      SubscriptBox["j", "A"], "[", "7", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "7", "]"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"59.06199999999998`", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.566131566537546*^9}]
}, Closed]],

Cell[BoxData[
 StyleBox[
  RowBox[{"We", " ", "now", " ", "verify", " ", "that", " ", 
   SubscriptBox["j", "B"], " ", "commutes", " ", "with", " ", "the", " ", 
   "Hamiltonian", " ", "for", " ", "both", " ", "odd", " ", "and", " ", 
   "even", " ", "number", " ", "of", " ", 
   RowBox[{"sites", "."}]}],
  FontSize->12]], "Text",
 CellChangeTimes->{{3.4911525461384077`*^9, 3.491152572510778*^9}},
 FontFamily->"Times",
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"H", "[", "5", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "5", "]"}]}], ",", 
    RowBox[{
     RowBox[{
      SubscriptBox["j", "B"], "[", "5", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "5", "]"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"42.80600000000004`", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.5661316741196995`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"H", "[", "6", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "6", "]"}]}], ",", 
    RowBox[{
     RowBox[{
      SubscriptBox["j", "B"], "[", "6", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "6", "]"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"62.32299999999998`", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.566131630911228*^9}]
}, Closed]],

Cell["\<\
Here we investigate whether the two currents commute with each other. The \
results suggest that for odd N the two currents commute but that they do not \
commute for even N.\
\>", "Text",
 CellChangeTimes->{{3.491153241431958*^9, 3.491153310712201*^9}, {
   3.491153374788842*^9, 3.491153390672567*^9}, 3.565754509415023*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Commutator", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["j", "A"], "[", "5", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "5", "]"}]}], ",", 
    RowBox[{
     RowBox[{
      SubscriptBox["j", "B"], "[", "5", "]"}], "/.", 
     RowBox[{"PBCrule", "[", "5", "]"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"95.36399999999998`", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.5661317715462723`*^9}]
}, Closed]],

Cell["\<\
From the computation below we see that the commutator is non zero for even \
sites.\
\>", "Text",
 CellChangeTimes->{{3.565754849541741*^9, 3.565754939004838*^9}, {
  3.565755016981934*^9, 3.565755032981584*^9}, {3.5661320363894205`*^9, 
  3.566132065317075*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"evensitecommutator", "=", 
   RowBox[{"Commutator", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["j", "A"], "[", "4", "]"}], "/.", 
      RowBox[{"PBCrule", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{
       SubscriptBox["j", "B"], "[", "4", "]"}], "/.", 
      RowBox[{"PBCrule", "[", "4", "]"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.566131832801776*^9, 3.5661318402592025`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{"evensitecommutator", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.5661318454214973`*^9, 3.566131854457014*^9}, {
  3.566132012396048*^9, 3.5661320149371934`*^9}, {3.566132070349363*^9, 
  3.566132074346592*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"t", " ", "U"}], ")"}], "**", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "]"}], "**", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "**", 
     RowBox[{"f", "[", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "]"}], "**", 
     RowBox[{"f", "[", 
      RowBox[{"3", ",", "1"}], "]"}]}]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], "+", 
   RowBox[{"\[LeftSkeleton]", "44", "\[RightSkeleton]"}], "+", 
   RowBox[{"2", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"t", " ", "U"}], ")"}], "**", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"4", ",", 
       RowBox[{"-", "1"}]}], "]"}], "**", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"4", ",", "1"}], "]"}], "**", 
     RowBox[{"f", "[", 
      RowBox[{"4", ",", 
       RowBox[{"-", "1"}]}], "]"}], "**", 
     RowBox[{"f", "[", 
      RowBox[{"2", ",", "1"}], "]"}]}]}]}],
  Short[#, 2]& ]], "Output",
 CellChangeTimes->{{3.56613200735776*^9, 3.566132015453223*^9}, 
   3.5661320748906226`*^9}]
}, Closed]],

Cell[TextData[{
 "In view of the above results, it seems very likely that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", 
     RowBox[{
      SubscriptBox["j", "A"], ",", "T"}], "]"}], "=", "0"}], TraditionalForm]],
  FontSize->12],
 " for odd number of sites, where T is the transfer matrix of the Hubbard \
model in the Fermi representation, whereas the commutator is non zero for \
even sites."
}], "Text",
 CellChangeTimes->{{3.565754849541741*^9, 3.565754939004838*^9}, 
   3.5662174101083965`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
(VIII)  Construction of Cluster Hamiltonians of the Hubbard model using Bras \
and Kets\
\>", "Subsection",
 CellChangeTimes->{{3.567007044101672*^9, 3.5670070937637444`*^9}, {
  3.567007966178505*^9, 3.567007985272533*^9}}],

Cell["\<\
We illustrate the use of the Bra[Vacuum] and Ket[Vacuum] symbols, for \
creating the space of allowed states in the Hubbard model on a small cluster, \
and furhter to operate the Hamiltonian on these to generate its  matrix \
representation.  Let us consider a 4 site cluster defined in the diagram, \
with the Hubbard Hamiltonian hopping parameters (t1, t2 , U). For \
illustration we confine ourselves to the sector with Sz=0 and 4 particles, \
i.e. the half filled state, where the number of basis states  is 36. 



                          (1)             t1                 (2)
                                              
                          t1                t2                t1
                          
                          (3)              t1                 (4)
 
 
                      4-Site Hubbard Cluster with nearest neighbour hops t1, \
and second neighbour hops t2.
 \
\>", "Text",
 CellChangeTimes->{{3.566997656494385*^9, 3.566997670530336*^9}, {
  3.566997733967971*^9, 3.566997882106155*^9}, {3.566997923544196*^9, 
  3.566998004520083*^9}, {3.56699899470232*^9, 3.5669993940941353`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hop", "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", " ", 
  RowBox[{"-", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"f\[Dagger]", "[", 
        RowBox[{"i", ",", "\[Sigma]"}], "]"}], "**", 
       RowBox[{"f", "[", 
        RowBox[{"j", ",", "\[Sigma]"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"f\[Dagger]", "[", 
        RowBox[{"j", ",", "\[Sigma]"}], "]"}], "**", 
       RowBox[{"f", "[", 
        RowBox[{"i", ",", "\[Sigma]"}], "]"}]}]}], " ", ",", 
     RowBox[{"{", 
      RowBox[{"\[Sigma]", ",", 
       RowBox[{"-", "1"}], ",", "1", ",", "2"}], "}"}]}], "]"}]}]}]], "Input",\

 CellChangeTimes->{{3.566940205535891*^9, 3.566940284205268*^9}, {
  3.5669403204191027`*^9, 3.566940405469885*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pot", "[", "i_", "]"}], ":=", " ", 
  RowBox[{"U", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"i", ",", "1"}], "]"}], "**", 
    RowBox[{
     SubscriptBox["n", "f"], "[", 
     RowBox[{"i", ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.566940416725155*^9, 3.566940448612671*^9}, {
  3.566940573791978*^9, 3.566940573938201*^9}, {3.566940621672555*^9, 
  3.566940621908375*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Hcluster", "=", 
   RowBox[{
    RowBox[{"t1", 
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"hop", "[", 
        RowBox[{"1", ",", "2"}], "]"}], "+", 
       RowBox[{"hop", "[", 
        RowBox[{"2", ",", "3"}], "]"}], "+", 
       RowBox[{"hop", "[", 
        RowBox[{"3", ",", "4"}], "]"}], "+", 
       RowBox[{"hop", "[", 
        RowBox[{"4", ",", "1"}], "]"}]}], ")"}]}], "+", 
    RowBox[{"t2", 
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"hop", "[", 
        RowBox[{"1", ",", "4"}], "]"}], "+", 
       RowBox[{"hop", "[", 
        RowBox[{"2", ",", "3"}], "]"}]}], ")"}]}], " ", "+", " ", 
    RowBox[{"pot", "[", "1", "]"}], "+", 
    RowBox[{"pot", "[", "2", "]"}], "+", 
    RowBox[{"pot", "[", "3", "]"}], "+", 
    RowBox[{"pot", "[", "4", "]"}]}]}], " ", ";"}]], "Input",
 CellChangeTimes->{{3.5669404543103237`*^9, 3.566940569900421*^9}, {
   3.5669406040444193`*^9, 3.566940616711664*^9}, 3.56694869280683*^9}],

Cell["\<\
We now define the creation and destruction operators for pairs of sites with \
a given spin projection,\
\>", "Text",
 CellChangeTimes->{{3.566999424304554*^9, 3.566999453341796*^9}, {
   3.56699948676694*^9, 3.566999524366869*^9}, {3.566999623018383*^9, 
   3.566999645968858*^9}, 3.566999945709934*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"twofermi", "[", 
    RowBox[{"i_", ",", "j_", ",", "\[Sigma]_"}], "]"}], "=", 
   RowBox[{
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"i", ",", "\[Sigma]"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"j", ",", "\[Sigma]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"twofermidest", "[", 
    RowBox[{"i_", ",", "j_", ",", "\[Sigma]_"}], "]"}], "=", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"j", ",", "\[Sigma]"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"i", ",", "\[Sigma]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5669441268592987`*^9, 3.566944192380464*^9}, 
   3.566948705436226*^9, 3.56699956997185*^9}],

Cell["\<\
Let us see the explicit form of a basis ket.  The function StandardOrderQ \
organizes a Fermi operator  product,  its convention is to write these with \
increasing site labels from left to right, and puts the down spin blocks to \
the left of the up spin blocks. \
\>", "Text",
 CellChangeTimes->{{3.5669996640328693`*^9, 3.566999837558652*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardOrderQ", "[", 
  RowBox[{
   RowBox[{"twofermi", "[", 
    RowBox[{"1", ",", "2", ",", "1"}], "]"}], "**", 
   RowBox[{"twofermi", "[", 
    RowBox[{"2", ",", "3", ",", 
     RowBox[{"-", "1"}]}], "]"}], "**", 
   RowBox[{"Ket", "[", "Vacuum", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.566944296933358*^9, 3.5669443493440113`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"2", ",", 
    RowBox[{"-", "1"}]}], "]"}], "**", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", "1"}]}], "]"}], "**", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"1", ",", "1"}], "]"}], "**", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"2", ",", "1"}], "]"}], "**", 
  TemplateBox[{"Vacuum"},
   "Ket"]}]], "Output",
 CellChangeTimes->{3.566944350093704*^9, 3.5669487342895327`*^9, 
  3.567004229011414*^9}]
}, Closed]],

Cell["\<\
 We  next  produce the 36 basis kets  {\[CapitalPsi][1], .., \
\[CapitalPsi][36]}      and  their adjoint 36 basis  bras  \
{\[CapitalPsi]B[1], .., \[CapitalPsi]B[36]}\
\>", "Text",
 CellChangeTimes->{{3.566999952573427*^9, 3.567000052023326*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ii", "=", "1"}], ";", 
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalPsi]", "[", "ii", "]"}], "=", 
      RowBox[{"StandardOrderQ", "[", 
       RowBox[{
        RowBox[{"twofermi", "[", 
         RowBox[{"i", ",", "j", ",", "1"}], "]"}], "**", 
        RowBox[{"twofermi", "[", 
         RowBox[{"k", ",", "l", ",", 
          RowBox[{"-", "1"}]}], "]"}], "**", 
        RowBox[{"Ket", "[", "Vacuum", "]"}]}], "]"}]}], ";", 
     RowBox[{"ii", "=", 
      RowBox[{"ii", "+", "1"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "1", ",", 
      RowBox[{"i", "-", "1"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "1", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"l", ",", "1", ",", 
      RowBox[{"k", "-", "1"}]}], "}"}]}], "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ii", "=", "1"}], ";", 
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalPsi]B", "[", "ii", "]"}], "=", 
      RowBox[{"StandardOrderQ", "[", 
       RowBox[{
        RowBox[{"Bra", "[", "Vacuum", "]"}], "**", 
        RowBox[{"twofermidest", "[", 
         RowBox[{"i", ",", "j", ",", "1"}], "]"}], "**", 
        RowBox[{"twofermidest", "[", 
         RowBox[{"k", ",", "l", ",", 
          RowBox[{"-", "1"}]}], "]"}]}], "]"}]}], ";", 
     RowBox[{"ii", "=", 
      RowBox[{"ii", "+", "1"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "1", ",", 
      RowBox[{"i", "-", "1"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "1", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"l", ",", "1", ",", 
      RowBox[{"k", "-", "1"}]}], "}"}]}], "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.566944449331581*^9, 3.5669445508206882`*^9}, {
   3.566944604623892*^9, 3.566944663471974*^9}, 3.566948413489739*^9, 
   3.567000094145464*^9}],

Cell["It is useful to inspect a typical basis state pair", "Text",
 CellChangeTimes->{{3.567000122438596*^9, 3.567000141998784*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalPsi]", "[", "9", "]"}]], "Input",
 CellChangeTimes->{{3.567000148038847*^9, 3.567000156563529*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"2", ",", 
    RowBox[{"-", "1"}]}], "]"}], "**", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", "1"}]}], "]"}], "**", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"1", ",", "1"}], "]"}], "**", 
  RowBox[{"f\[Dagger]", "[", 
   RowBox[{"3", ",", "1"}], "]"}], "**", 
  TemplateBox[{"Vacuum"},
   "Ket"]}]], "Output",
 CellChangeTimes->{3.5670001589816523`*^9, 3.567004229197774*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalPsi]B", "[", "18", "]"}]], "Input",
 CellChangeTimes->{{3.567000148038847*^9, 3.567000176901103*^9}}],

Cell[BoxData[
 RowBox[{
  TemplateBox[{"Vacuum"},
   "Bra"], "**", 
  RowBox[{"f", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", "1"}]}], "]"}], "**", 
  RowBox[{"f", "[", 
   RowBox[{"4", ",", 
    RowBox[{"-", "1"}]}], "]"}], "**", 
  RowBox[{"f", "[", 
   RowBox[{"2", ",", "1"}], "]"}], "**", 
  RowBox[{"f", "[", 
   RowBox[{"3", ",", "1"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5670001775061274`*^9, 3.567004244418676*^9}]
}, Closed]],

Cell["\<\
We next illustrate the action of the Hubbard Hamiltonian on these states, \
here the function StandardOrderQ  does the job of simplifying expressions bu \
pushing the destruction operators to the extreme right and hitting the \
Ket[Vacuum], which is annihilated, or a similar story for the Bra[Vacuum] \
that is annihilated by creation operators. A brief glance at the output shows \
that we get back  the basis states multiplied by coefficients that depend on \
t1,t2 and U. \
\>", "Text",
 CellChangeTimes->{{3.5670002023242292`*^9, 3.567000382025696*^9}, {
  3.567001156509169*^9, 3.567001221372243*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"StandardOrderQ", "[", 
    RowBox[{"Hcluster", "**", 
     RowBox[{"\[CapitalPsi]", "[", "9", "]"}]}], "]"}], ",", "3"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5670003857262278`*^9, 3.567000414314699*^9}, {
  3.567000530713459*^9, 3.5670005648429537`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"t1", "**", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "]"}], "**", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"3", ",", 
       RowBox[{"-", "1"}]}], "]"}], "**", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "**", 
     RowBox[{"f\[Dagger]", "[", 
      RowBox[{"3", ",", "1"}], "]"}], "**", 
     TemplateBox[{"Vacuum"},
      "Ket"]}]}], "-", 
   RowBox[{"t1", "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}], "-", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "-", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "-", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "-", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
   RowBox[{"U", "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "**", 
    RowBox[{"f\[Dagger]", "[", 
     RowBox[{"3", ",", "1"}], "]"}], "**", 
    TemplateBox[{"Vacuum"},
     "Ket"]}]}],
  Short[#, 3]& ]], "Output",
 CellChangeTimes->{{3.567000552758947*^9, 3.5670005655973597`*^9}, 
   3.5670042445922623`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"StandardOrderQ", "[", 
    RowBox[{
     RowBox[{"\[CapitalPsi]B", "[", "18", "]"}], "**", "Hcluster"}], "]"}], 
   ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.5670003857262278`*^9, 3.567000414314699*^9}, {
  3.5670004479790373`*^9, 3.567000524840982*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{"t1", "**", 
    TemplateBox[{"Vacuum"},
     "Bra"], "**", 
    RowBox[{"f", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", "1"}], "]"}]}], "-", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "-", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "-", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "-", 
   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
   RowBox[{"U", "**", 
    TemplateBox[{"Vacuum"},
     "Bra"], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"4", ",", 
      RowBox[{"-", "1"}]}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "**", 
    RowBox[{"f", "[", 
     RowBox[{"3", ",", "1"}], "]"}]}]}],
  Short[#, 2]& ]], "Output",
 CellChangeTimes->{
  3.567000470749135*^9, {3.5670005010025806`*^9, 3.56700052559303*^9}, 
   3.5670042447135897`*^9}]
}, Closed]],

Cell["\<\
At this point, there are two ways to construct the Hamiltonian matrix.  \
Starting with expressions of the type \
StandardOrderQ[Hcluster**\[CapitalPsi][i]],  we can take the inner product \
with the bras \[CapitalPsi]B[j]. As an alternative, we   can  pick off the \
coefficients using the  QCoefficient function, this is faster and hence \
prefereable in most situatiions. We illustrate both methods and display the \
time saved by the second method. First we need a rule that collapses the \
fundamental innerproduct to unity:\
\>", "Text",
 CellChangeTimes->{{3.567001124460005*^9, 3.5670011470079937`*^9}, {
  3.567001228371913*^9, 3.5670015278748302`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ruleinnerproduct", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Bra", "[", "Vacuum", "]"}], "**", 
      RowBox[{"Ket", "[", "Vacuum", "]"}]}], "\[Rule]", "1"}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5669483795762253`*^9, 3.566948391663114*^9}, 
   3.566948720862483*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"res", "[", "i", "]"}], "=", 
     RowBox[{"StandardOrderQ", "[", 
      RowBox[{"Hcluster", "**", 
       RowBox[{"\[CapitalPsi]", "[", "i", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "36"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5669447133282633`*^9, 3.566944747221569*^9}, {
  3.566948504124261*^9, 3.566948518583954*^9}, {3.567004253584447*^9, 
  3.567004281639509*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4.5608010000000005`", ",", "Null"}], "}"}]], "Output",
 CellChangeTimes->{3.5670042877759132`*^9}]
}, Closed]],

Cell["\<\
The array res[i] stores the resultant of the action of the Hamiltonian on the \
ith ket.
The two methods to creat the Hamiltonian matrix use the innerproduct with the \
\[CapitalPsi]B[j], i.e. the  Bra vector,  or picking out the coefficients of \
\[CapitalPsi][j] in res[i]. These are called matrix1 and matrix2 respectively.\
\>", "Text",
 CellChangeTimes->{{3.567004727080536*^9, 3.5670049192675753`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"matrix1", "[", 
   RowBox[{"j_", ",", "i_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"StandardOrderQ", "[", 
    RowBox[{
     RowBox[{"\[CapitalPsi]B", "[", "j", "]"}], "**", 
     RowBox[{"res", "[", "i", "]"}]}], "]"}], "/.", 
   "ruleinnerproduct"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"matrix2", "[", 
   RowBox[{"j_", ",", "i_"}], "]"}], ":=", 
  RowBox[{"QCoefficient", "[", 
   RowBox[{
    RowBox[{"res", "[", "i", "]"}], ",", 
    RowBox[{"\[CapitalPsi]", "[", "j", "]"}]}], 
   "]"}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.566948468708324*^9, 3.566948494545453*^9}, {
   3.566948529460209*^9, 3.566948577409617*^9}, {3.566948749144498*^9, 
   3.566948751310501*^9}, 3.5670043638387613`*^9, 3.567004948329672*^9, {
   3.567004995031423*^9, 3.567005038892932*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"R1", "=", 
     RowBox[{"Timing", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"matrix1", "[", 
         RowBox[{"k", ",", "i"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "1", ",", "36"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "1", ",", "36"}], "}"}]}], "]"}], "]"}]}], ";"}], 
   "\n", 
   RowBox[{
    RowBox[{"R2", "=", 
     RowBox[{"Timing", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"matrix2", "[", 
         RowBox[{"k", ",", "i"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "1", ",", "36"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "1", ",", "36"}], "}"}]}], "]"}], "]"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.566948468708324*^9, 3.566948494545453*^9}, {
   3.566948529460209*^9, 3.566948577409617*^9}, {3.566948749144498*^9, 
   3.566948751310501*^9}, 3.5670043638387613`*^9, 3.567004948329672*^9, {
   3.567004995031423*^9, 3.5670050584496326`*^9}, {3.5670051742860317`*^9, 
   3.567005237384788*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"R1", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5670063412456913`*^9, 3.567006343362752*^9}}],

Cell[BoxData["401.360128`"], "Output",
 CellChangeTimes->{3.56700634516748*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"R2", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.567006346686228*^9, 3.567006348552733*^9}}],

Cell[BoxData["7.002155999999957`"], "Output",
 CellChangeTimes->{3.567006349072178*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"R1", "[", 
     RowBox[{"[", "2", "]"}], "]"}], "-", 
    RowBox[{"R2", "[", 
     RowBox[{"[", "2", "]"}], "]"}]}], ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.5670063663280163`*^9, 3.56700639135742*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0"}], "}"}], ",", 
    RowBox[{"\[LeftSkeleton]", "34", "\[RightSkeleton]"}], ",", 
    RowBox[{"{", 
     RowBox[{
     "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
      "0"}], "}"}]}], "}"}],
  Short[#, 3]& ]], "Output",
 CellChangeTimes->{{3.567006377551005*^9, 3.567006391928524*^9}}]
}, Closed]],

Cell["\<\
We see that the two methods give the same resulting matrix, but the second \
method is considerably faster. We next define the ClusterHamiltonian and \
check its eigenvalues in some simple cases. For larger systems, it is clearly \
more advantageous to use the sparse matrix notation in such problems, but we \
will not pursue it here.\
\>", "Text",
 CellChangeTimes->{{3.567005096794984*^9, 3.567005125456533*^9}, {
  3.567006431071149*^9, 3.5670064548817883`*^9}, {3.5670064913334293`*^9, 
  3.56700653815803*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ClusterHamiltonian", "[", 
    RowBox[{"t1_", ",", "t2_", ",", "U_"}], "]"}], "=", 
   RowBox[{"R1", "[", 
    RowBox[{"[", "2", "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.566952567664838*^9, 3.566952594314766*^9}, {
  3.567006405808385*^9, 3.567006422984091*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eigenvalues", "[", 
  RowBox[{"ClusterHamiltonian", "[", 
   RowBox[{"1", ",", "0", ",", "0"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.566952604282537*^9, 3.566952618211444*^9}, {
  3.56700646209531*^9, 3.567006470271257*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "4"}], ",", 
   RowBox[{"-", "4"}], ",", 
   RowBox[{"-", "4"}], ",", 
   RowBox[{"-", "4"}], ",", "4", ",", "4", ",", "4", ",", "4", ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "2"}], ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", 
   "2", ",", "2", ",", "2", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.566952619623403*^9, 3.567006473687751*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eigenvalues", "[", 
  RowBox[{"ClusterHamiltonian", "[", 
   RowBox[{"0", ",", "0", ",", "U"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.566952604282537*^9, 3.566952618211444*^9}, {
  3.566952994739654*^9, 3.566952998189769*^9}, {3.567006555498122*^9, 
  3.567006557273445*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "U", ",", "U", 
   ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", 
   "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", 
   ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", "U", ",", 
   
   RowBox[{"2", " ", "U"}], ",", 
   RowBox[{"2", " ", "U"}], ",", 
   RowBox[{"2", " ", "U"}], ",", 
   RowBox[{"2", " ", "U"}], ",", 
   RowBox[{"2", " ", "U"}], ",", 
   RowBox[{"2", " ", "U"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5669530000727873`*^9, 3.567006558761882*^9}]
}, Closed]],

Cell["\<\
The first set of eigenvalues correspond to a non interacting model with only \
nearest neighbour hoppings, and is easily seen to be correct. The second set \
of eigenvalues correspond to the local limit, where we have 6 states in the \
U=0 sector (Lowest Hubbard Band), 6 states in the U=2 sector (Second Hubbard \
band) and the rest with U=1, i.e. the First Hubbard Band.\
\>", "Text",
 CellChangeTimes->{{3.567006597810544*^9, 3.56700674322256*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Appendices", "Section",
 CellChangeTimes->{{3.490643670291956*^9, 3.490643672412842*^9}},
 FontFamily->"Utopia",
 FontSize->16],

Cell[CellGroupData[{

Cell["(I) Explanation of Form", "Subsection",
 CellChangeTimes->{{3.481990715546875*^9, 3.481990718078125*^9}, 
   3.483974623328125*^9, {3.489248487787678*^9, 3.48924850384898*^9}, {
   3.489248579169793*^9, 3.489248580703822*^9}, {3.49055839959177*^9, 
   3.49055840015666*^9}, {3.5304120895746727`*^9, 3.530412090528164*^9}},
 FontFamily->"Times"],

Cell[TextData[{
 StyleBox["    The strength of the system lies in the ability to organize, \
rearrange, and manipulate terms in an expression based on their properties. \
The method of organization provides a simple language for the manipulation of \
terms. An understanding of the language used by the code is not essential for \
most operations. The user is only required to input algebraic expressions in \
a logical and minimally restricted form to effectively use the functions of \
this package. The language of organization will briefly be covered here for \
posterity.\n    ",
  FontWeight->"Plain"],
 StyleBox["Mathematica",
  FontWeight->"Plain",
  FontSlant->"Italic"],
 StyleBox[" organizes all input in terms of lists. Each list has a head. The \
head of a list is the function that is to be applied to the list. The example \
below shows how ",
  FontWeight->"Plain"],
 StyleBox["Mathematica",
  FontWeight->"Plain",
  FontSlant->"Italic"],
 StyleBox[" organizes some algebraic input into lists and functions.",
  FontWeight->"Plain"]
}], "Text",
 CellChangeTimes->{{3.48276572765625*^9, 3.482765759328125*^9}, {
  3.482768749171875*^9, 3.482769002796875*^9}, {3.483221545136125*^9, 
  3.483221589183*^9}, {3.483221628058*^9, 3.483221661542375*^9}, {
  3.483221704292375*^9, 3.4832217861205*^9}, {3.483222623604875*^9, 
  3.48322270633925*^9}, {3.483222743933*^9, 3.48322276890175*^9}, {
  3.483299955046875*^9, 3.483300013953125*^9}, {3.48339762253125*^9, 
  3.483397712109375*^9}, {3.4843365056875*^9, 3.48433650715625*^9}, {
  3.489179024797715*^9, 3.489179026019903*^9}, {3.489244368983735*^9, 
  3.489244370811186*^9}},
 LineSpacing->{1.5, 3},
 FontWeight->"Bold"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullForm", "[", 
  RowBox[{
   RowBox[{"5", " ", 
    RowBox[{
     UnderscriptBox["\[Sum]", "i"], 
     RowBox[{
      RowBox[{"c", "[", "i", "]"}], " ", 
      RowBox[{
       RowBox[{"b", "[", "i", "]"}], "**", 
       RowBox[{"b\[Dagger]", "[", "j", "]"}]}]}]}]}], "+", 
   RowBox[{"2", 
    RowBox[{
     UnderscriptBox["\[Sum]", "i"], 
     RowBox[{
      RowBox[{"d", "[", "i", "]"}], " ", 
      RowBox[{
       RowBox[{"b\[Dagger]", "[", "i", "]"}], "**", 
       RowBox[{"b", "[", "j", "]"}]}]}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.483397726421875*^9, 3.48339773265625*^9}},
 FontFamily->"Batang"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Plus", "[", 
    RowBox[{
     RowBox[{"Times", "[", 
      RowBox[{"5", ",", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{"Times", "[", 
          RowBox[{
           RowBox[{"c", "[", "i", "]"}], ",", 
           RowBox[{"NonCommutativeMultiply", "[", 
            RowBox[{
             RowBox[{"b", "[", "i", "]"}], ",", 
             RowBox[{"b\\[Dagger]", "[", "j", "]"}]}], "]"}]}], "]"}], ",", 
         "i"}], "]"}]}], "]"}], ",", 
     RowBox[{"Times", "[", 
      RowBox[{"2", ",", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{"Times", "[", 
          RowBox[{
           RowBox[{"d", "[", "i", "]"}], ",", 
           RowBox[{"NonCommutativeMultiply", "[", 
            RowBox[{
             RowBox[{"b\\[Dagger]", "[", "i", "]"}], ",", 
             RowBox[{"b", "[", "j", "]"}]}], "]"}]}], "]"}], ",", "i"}], 
        "]"}]}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.483397734921875*^9},
 FontFamily->"Batang"]
}, Closed]],

Cell["\<\
    It is the job of the Organize function to scan through this list and \
extract the relevant information. The Organize function is therefore at the \
heart of every operation of the code. This function reads input such as the \
example above and organizes it based on the properties of the terms \
encountered. Information encountered by the Organize function is stored in \
nested lists. Different types of objects are placed in different places \
within the larger list created by the Organize function. Organize recognizes \
four types of objects : numbers, sums, constants, and operators. Each of \
these types of objects is placed in a separate nested list. The easiest way \
to understand this ordering is through example\
\>", "Text",
 CellChangeTimes->{{3.483397757328125*^9, 3.483397823875*^9}, 
   3.489179027647991*^9, 3.489179605520837*^9, {3.489244379826809*^9, 
   3.489244380231276*^9}},
 LineSpacing->{1.5, 3}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Organize", "[", 
  RowBox[{"5", " ", 
   RowBox[{
    UnderscriptBox["\[Sum]", "i"], 
    RowBox[{
     RowBox[{"c", "[", "i", "]"}], " ", 
     RowBox[{
      RowBox[{"b", "[", "i", "]"}], "**", 
      RowBox[{"b\[Dagger]", "[", "j", "]"}]}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{3.483223255261125*^9},
 FontFamily->"Batang"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"5", ",", 
    RowBox[{"{", "i", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"c", "[", "i", "]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"b", "[", "i", "]"}], ",", 
      RowBox[{"b\[Dagger]", "[", "j", "]"}]}], "}"}]}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.483223255604875*^9, 3.483300130078125*^9},
 FontFamily->"Batang"]
}, Closed]],

Cell[TextData[{
 Cell[BoxData[
  GraphicsBox[
   TagBox[RasterBox[CompressedData["
1:eJztnT2S3DgSRrmxJ5nzTMjZI4wlby7QJ9ioE+gC7cuXLVeuTJnydINabCPE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     "], {{0, 138}, {331,
      0}}, {0, 255},
     ColorFunction->RGBColor],
    BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
    Selectable->False],
   BaseStyle->"ImageGraphics",
   ImageSize->{230.33333333333292`, Automatic},
   ImageSizeRaw->{331, 138},
   PlotRange->{{0, 331}, {0, 138}}]], "Input"],
 "\n",
 StyleBox["Fig. (1): The output of organize is a list of lists. Each list \
corresponds to a specific type of term.",
  FontFamily->"Times"]
}], "Text",
 CellFrame->{{0.5, 3}, {3, 0.5}},
 CellChangeTimes->{{3.483222828823625*^9, 3.483222878058*^9}, {
  3.483223251104875*^9, 3.48322325240175*^9}, {3.483223515698625*^9, 
  3.48322353065175*^9}, {3.48330015409375*^9, 3.483300216453125*^9}},
 TextAlignment->Center,
 FontFamily->"Batang",
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Organize", "[", 
  RowBox[{
   RowBox[{"5", " ", 
    RowBox[{
     UnderscriptBox["\[Sum]", "i"], 
     RowBox[{
      RowBox[{"c", "[", "i", "]"}], " ", 
      RowBox[{
       RowBox[{"b", "[", "i", "]"}], "**", 
       RowBox[{"b\[Dagger]", "[", "j", "]"}]}]}]}]}], "+", 
   RowBox[{"2", 
    RowBox[{
     UnderscriptBox["\[Sum]", "i"], 
     RowBox[{
      RowBox[{"d", "[", "i", "]"}], " ", 
      RowBox[{
       RowBox[{"b\[Dagger]", "[", "i", "]"}], "**", 
       RowBox[{"b", "[", "j", "]"}]}]}]}]}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"5", ",", 
     RowBox[{"{", "i", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"c", "[", "i", "]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"b", "[", "i", "]"}], ",", 
       RowBox[{"b\[Dagger]", "[", "j", "]"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", 
     RowBox[{"{", "i", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"d", "[", "i", "]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"b\[Dagger]", "[", "i", "]"}], ",", 
       RowBox[{"b", "[", "j", "]"}]}], "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5661321584814034`*^9, 3.5661321658468246`*^9}}]
}, Closed]],

Cell[BoxData[{
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3TGS8joWhmHXzErugkhmBVM36mw2QDZpr4ANkHdOTEraYYed9Q6YM5z6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    "], {{0, 131}, {608, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->{346.66666666666424`, Automatic},
  ImageSizeRaw->{608, 131},
  PlotRange->{{0, 608}, {0, 131}}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{"Fig", ".", 
     RowBox[{"(", "2", ")"}]}], ":", 
    RowBox[{
    "When", " ", "the", " ", "input", " ", "of", " ", "Organize", " ", "is", 
     " ", "the", " ", "addition", " ", "of", " ", "several", " ", "terms"}]}],
    ",", " ", 
   RowBox[{
   "each", " ", "term", " ", "is", " ", "contained", " ", "in", " ", "a", " ",
     "list", " ", "within", " ", "the", " ", "larger", " ", 
    RowBox[{"list", "."}]}]}],
  FontFamily->"Times"]}], "Text",
 CellFrame->{{0.5, 3}, {3, 0.5}},
 CellFrameColor->GrayLevel[0],
 CellChangeTimes->{{3.48322389765175*^9, 3.483223898058*^9}, {
  3.48330022503125*^9, 3.483300259859375*^9}, {3.48330031396875*^9, 
  3.483300375234375*^9}},
 TextAlignment->Center,
 LineSpacing->{2, 3},
 FontFamily->"Batang",
 FontSize->14],

Cell["\<\
     When a function is encountered that cannot be decomposed further, the \
Organize function determines whether this function contains any operators. If \
so, the package uses a special notation to signify that it has found a \
function of operators that cannot be decomposed further. The notation \
function[a, {b}] is used, where a is the function that cannot be decomposed, \
and b is a list of the operators on which the function depends. As it \
currently stands not all functions can be read and placed in this notation, \
but any function involving operators to different powers or exponential \
functions of operators can be understood. Using this format the user is free \
to define commutators of more complicated functions of operators as required. \
The notation used is identical to that used to add an operator, with the \
caveat that the definition will be written using the \"function\" notation \
described above. An example of an organized function such as described above \
is given below.\
\>", "Text",
 CellChangeTimes->{{3.490372501471574*^9, 3.490372718965798*^9}, {
  3.4903727494751673`*^9, 3.490372913268581*^9}},
 LineSpacing->{1.5, 3}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Organize", "[", 
  RowBox[{"a", " ", 
   SuperscriptBox["\[ExponentialE]", 
    RowBox[{
     RowBox[{
      RowBox[{"t", "[", "i", "]"}], " ", 
      RowBox[{"q", "[", "i", "]"}]}], "+", 
     RowBox[{"q", "[", "j", "]"}]}]]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"1", ",", 
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "a", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"function", "[", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"q", "[", "j", "]"}]], ",", 
        RowBox[{"{", 
         RowBox[{"q", "[", "j", "]"}], "}"}]}], "]"}], ",", 
      RowBox[{"function", "[", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"q", "[", "i", "]"}], " ", 
          RowBox[{"t", "[", "i", "]"}]}]], ",", 
        RowBox[{"{", 
         RowBox[{"q", "[", "i", "]"}], "}"}]}], "]"}]}], "}"}]}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5661322197429075`*^9}]
}, Closed]],

Cell[TextData[StyleBox["     Humanize is the functional opposite of organize \
in that Humanize reads the language created by the Organize function and \
recreates a mathematical form that a user can understand. This function is \
demonstrated below.",
 FontSize->12]], "Text",
 CellChangeTimes->{{3.483300388890625*^9, 3.4833005295*^9}, {
  3.4903727519510202`*^9, 3.490372752854845*^9}},
 LineSpacing->{1.5, 3},
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Humanize", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"5", ",", 
      RowBox[{"{", "i", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"c", "[", "i", "]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"b", "[", "i", "]"}], ",", 
        RowBox[{"b\[Dagger]", "[", "j", "]"}]}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"{", "i", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"d", "[", "i", "]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"b\[Dagger]", "[", "i", "]"}], ",", 
        RowBox[{"b", "[", "j", "]"}]}], "}"}]}], "}"}]}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"5", " ", 
   RowBox[{
    UnderscriptBox["\[Sum]", "i"], 
    RowBox[{
     RowBox[{"c", "[", "i", "]"}], "**", 
     RowBox[{"b", "[", "i", "]"}], "**", 
     RowBox[{"b\[Dagger]", "[", "j", "]"}]}]}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{
    UnderscriptBox["\[Sum]", "i"], 
    RowBox[{
     RowBox[{"d", "[", "i", "]"}], "**", 
     RowBox[{"b\[Dagger]", "[", "i", "]"}], "**", 
     RowBox[{"b", "[", "j", "]"}]}]}]}]}]], "Output",
 CellChangeTimes->{3.5661322299804935`*^9}]
}, Closed]],

Cell[TextData[StyleBox["     With this simple language the manipulation of \
certain types of terms can be performed with greater ease. Rather than \
manipulating operators as we come across them, we collect every operator in \
the expression. We can then perform algorithms on the lists of operators to \
combine and manipulate them as necessary. It is necessary to collect numbers \
and constants separately so that numbers can be combined as necessary and \
constants, which may depend on indices of summation, can be placed inside a \
sum. Also, the ability to identify summed indices allows us to evaluate delta \
functions. Every function in the package uses this form to organize input.",
 FontWeight->"Plain"]], "Text",
 CellChangeTimes->{{3.483300536640625*^9, 3.483300595859375*^9}, {
  3.483300670296875*^9, 3.48330078465625*^9}, {3.483300851765625*^9, 
  3.483301162484375*^9}, {3.4891796071497297`*^9, 3.4891796113894863`*^9}},
 LineSpacing->{1.5, 3},
 FontWeight->"Bold"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
(II) Demonstration Problem Output (Produces large outputs hence not meant for \
printing)\
\>", "Subsection",
 CellChangeTimes->{{3.490643836168972*^9, 3.4906438480404987`*^9}, {
   3.530412093803885*^9, 3.5304120950282793`*^9}, {3.5657552072232237`*^9, 
   3.565755228857397*^9}, {3.565836362113699*^9, 3.565836366836193*^9}, 
   3.5661347242721586`*^9},
 FontFamily->"Times New Roman"],

Cell[CellGroupData[{

Cell["\<\
Product of three scattering Matrices from the Eight Vertex Yang - Baxter \
Demonstration Problem\
\>", "Subsubsection",
 CellChangeTimes->{{3.489176736806616*^9, 3.489176769482766*^9}},
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LHS", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"n", ",", 
      SubscriptBox["g", "2"]}]], "[", "W", "]"}], "\[CircleTimes]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["L", 
       RowBox[{"n", ",", 
        SubscriptBox["g", "1"]}]], "[", 
      RowBox[{"W", "'"}], "]"}], "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        SubscriptBox["g", "1"], ",", 
        SubscriptBox["g", "2"]}]], "[", 
      RowBox[{"W", "''"}], "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{3.489176778296371*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["a", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["b", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["c", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "y"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "z"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"c", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"d", " ", 
      SuperscriptBox["d", "\[Prime]",
       MultilineFunction->None], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", ",", "1"}]], "[", 
       SuperscriptBox["W", "\[Prime]\[Prime]",
        MultilineFunction->None], "]"}]}], ")"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"1", ",", "y"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"2", ",", "x"}], "]"}], "**", 
    RowBox[{"\[Sigma]", "[", 
     RowBox[{"3", ",", "z"}], "]"}]}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "a", " ", 
   SuperscriptBox["a", "\[Prime]",
    MultilineFunction->None], " ", 
   RowBox[{
    SubscriptBox["S", 
     RowBox[{"2", ",", "1"}]], "[", 
    SuperscriptBox["W", "\[Prime]\[Prime]",
     MultilineFunction->None], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "b", " ", 
   SuperscriptBox["a", "\[Prime]",
    MultilineFunction->None], " ", 
   RowBox[{
    SubscriptBox["S", 
     RowBox[{"2", ",", "1"}]], "[", 
    SuperscriptBox["W", "\[Prime]\[Prime]",
     MultilineFunction->None], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "a", " ", 
   SuperscriptBox["b", "\[Prime]",
    MultilineFunction->None], " ", 
   RowBox[{
    SubscriptBox["S", 
     RowBox[{"2", ",", "1"}]], "[", 
    SuperscriptBox["W", "\[Prime]\[Prime]",
     MultilineFunction->None], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", "b", " ", 
   SuperscriptBox["b", "\[Prime]",
    MultilineFunction->None], " ", 
   RowBox[{
    SubscriptBox["S", 
     RowBox[{"2", ",", "1"}]], "[", 
    SuperscriptBox["W", "\[Prime]\[Prime]",
     MultilineFunction->None], "]"}]}]}]], "Output",
 CellChangeTimes->{{3.489176778987857*^9, 3.4891768285520363`*^9}, 
   3.5661322761011314`*^9, 3.5661323205166717`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Example Sum of Coefficients from 1 D Hubbard Model SSS Demonstration Problem :\
\>", "Subsubsection",
 CellChangeTimes->{{3.489176896485964*^9, 3.4891769252939167`*^9}, 
   3.4891796167564907`*^9},
 FontSize->14],

Cell["\<\
When simplified this result goes to zero. It is however, nontrivial.\
\>", "Text",
 CellChangeTimes->{{3.489177767829632*^9, 3.489177786124*^9}},
 FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sum", "[", "1", "]"}]], "Input",
 CellChangeTimes->{{3.489176939726054*^9, 3.4891769418903627`*^9}, 
   3.566134564415015*^9, 3.5661347105513735`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"4", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"4", " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"4", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}], ",", 
      SubscriptBox["\[Theta]", "2"], ",", 
      SubscriptBox["\[Theta]", "1"]}], "]"}], "2"], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "1", ",", 
      SubscriptBox["\[Theta]", "2"], ",", 
      SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{"1", ",", "1", ",", 
      SubscriptBox["\[Theta]", "1"], ",", 
      SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"4", " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", 
      RowBox[{"-", "1"}], ",", 
      SubscriptBox["\[Theta]", "2"], ",", 
      SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}], ",", 
      SubscriptBox["\[Theta]", "1"], ",", 
      SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{"1", ",", "1", ",", 
      SubscriptBox["\[Theta]", "2"], ",", 
      SubscriptBox["\[Theta]", "1"]}], "]"}], "2"], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"4", " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"4", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}], ",", 
      SubscriptBox["\[Theta]", "2"], ",", 
      SubscriptBox["\[Theta]", "1"]}], "]"}], "2"], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", 
      RowBox[{"-", "1"}], ",", 
      SubscriptBox["\[Theta]", "1"], ",", 
      SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{"1", ",", 
      RowBox[{"-", "1"}], ",", 
      SubscriptBox["\[Theta]", "2"], ",", 
      SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"4", " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"4", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"A", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1", ",", 
     SubscriptBox["\[Theta]", "1"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{"1", ",", "1", ",", 
      SubscriptBox["\[Theta]", "2"], ",", 
      SubscriptBox["\[Theta]", "1"]}], "]"}], "2"], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "1", ",", 
      SubscriptBox["\[Theta]", "1"], ",", 
      SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
   RowBox[{"A", "[", 
    RowBox[{"1", ",", "1", ",", 
     SubscriptBox["\[Theta]", "2"], ",", 
     SubscriptBox["\[Theta]", "1"]}], "]"}], " ", 
   SuperscriptBox[
    RowBox[{"A", "[", 
     RowBox[{"1", ",", "1", ",", 
      SubscriptBox["\[Theta]", "2"], ",", 
      SubscriptBox["\[Theta]", "3"]}], "]"}], "2"], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "-", 
     SubscriptBox["\[Theta]", "2"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "2"], "-", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{
     SubscriptBox["\[Theta]", "1"], "+", 
     SubscriptBox["\[Theta]", "3"]}], "]"}], " ", 
   RowBox[{"Cosh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "3"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "1"], "-", 
     SubscriptBox["h", "2"]}], "]"}], " ", 
   RowBox[{"Sinh", "[", 
    RowBox[{
     SubscriptBox["h", "2"], "-", 
     SubscriptBox["h", "3"]}], "]"}]}]}]], "Output",
 CellChangeTimes->{
  3.4891769426739817`*^9, 3.48917776034289*^9, 3.5661323334664125`*^9, {
   3.5661345577446337`*^9, 3.5661345647810364`*^9}, 3.5661347108953934`*^9}]
}, Closed]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.565836769031426*^9, 3.565836772714136*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.565836788849255*^9, 3.565836794925982*^9}}]
}, Closed]]
}, Closed]]
}, Closed]]
}, Open  ]]
},
WindowSize->{1350, 730},
WindowMargins->{{104, Automatic}, {Automatic, 58}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> False}},
ShowSelection->True,
Magnification->1.5,
FrontEndVersion->"10.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (December 4, \
2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3565243144-4018571"->{
  Cell[15512, 488, 243, 4, 86, "Print",
   CellTags->"Info3565243144-4018571"]},
 "Info3566105331-6477710"->{
  Cell[205962, 6112, 1684, 26, 490, "Print",
   CellTags->"Info3566105331-6477710"]},
 "Info3566105432-6477710"->{
  Cell[210343, 6225, 564, 9, 154, "Print",
   CellTags->"Info3566105432-6477710"]},
 "Info3648196344-2372840"->{
  Cell[2937, 88, 498, 8, 87, "Print",
   CellTags->"Info3648196344-2372840"],
  Cell[3519, 104, 498, 8, 87, "Print",
   CellTags->"Info3648196344-2372840"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3565243144-4018571", 363170, 10622},
 {"Info3566105331-6477710", 363280, 10625},
 {"Info3566105432-6477710", 363395, 10628},
 {"Info3648196344-2372840", 363508, 10631}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 212, 5, 91, "Title"],
Cell[CellGroupData[{
Cell[816, 31, 400, 7, 70, "Section"],
Cell[1219, 40, 454, 9, 76, "Text"],
Cell[1676, 51, 639, 15, 69, "Input"],
Cell[CellGroupData[{
Cell[2340, 70, 233, 4, 61, "Subsection"],
Cell[2576, 76, 289, 5, 76, "Text"],
Cell[CellGroupData[{
Cell[2890, 85, 44, 1, 43, "Input"],
Cell[2937, 88, 498, 8, 87, "Print",
 CellTags->"Info3648196344-2372840"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3472, 101, 44, 1, 43, "Input"],
Cell[3519, 104, 498, 8, 87, "Print",
 CellTags->"Info3648196344-2372840"]
}, Open  ]],
Cell[4032, 115, 1085, 18, 194, "Text"],
Cell[CellGroupData[{
Cell[5142, 137, 144, 4, 43, "Input"],
Cell[5289, 143, 219, 5, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5545, 153, 207, 6, 43, "Input"],
Cell[5755, 161, 279, 7, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[6071, 173, 194, 6, 32, "Input"],
Cell[6268, 181, 219, 5, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6524, 191, 86, 2, 43, "Input"],
Cell[6613, 195, 89, 1, 43, "Output"]
}, Open  ]],
Cell[6717, 199, 405, 7, 76, "Text"],
Cell[7125, 208, 81, 2, 43, "Input"],
Cell[7209, 212, 333, 12, 73, "Input"],
Cell[CellGroupData[{
Cell[7567, 228, 115, 3, 43, "Input"],
Cell[7685, 233, 271, 7, 50, "Output"]
}, Open  ]],
Cell[7971, 243, 170, 3, 46, "Text"],
Cell[8144, 248, 953, 32, 155, "Input"],
Cell[CellGroupData[{
Cell[9122, 284, 153, 4, 43, "Input"],
Cell[9278, 290, 94, 1, 43, "Output"]
}, Open  ]],
Cell[9387, 294, 170, 5, 44, "Input"],
Cell[9560, 301, 457, 8, 76, "Text"],
Cell[CellGroupData[{
Cell[10042, 313, 90, 2, 43, "Input"],
Cell[10135, 317, 552, 19, 108, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10724, 341, 239, 7, 43, "Input"],
Cell[10966, 350, 92, 1, 43, "Output"]
}, Open  ]],
Cell[11073, 354, 252, 5, 76, "Text"],
Cell[CellGroupData[{
Cell[11350, 363, 262, 7, 43, "Input"],
Cell[11615, 372, 92, 1, 43, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[11756, 379, 496, 12, 52, "Subsection"],
Cell[12255, 393, 537, 9, 105, "Text"],
Cell[CellGroupData[{
Cell[12817, 406, 156, 3, 45, "Input"],
Cell[12976, 411, 429, 11, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[13442, 427, 294, 7, 34, "Input"],
Cell[13739, 436, 724, 16, 62, "Output"]
}, Closed]],
Cell[14478, 455, 890, 25, 111, "Input"],
Cell[CellGroupData[{
Cell[15393, 484, 116, 2, 43, "Input"],
Cell[15512, 488, 243, 4, 86, "Print",
 CellTags->"Info3565243144-4018571"]
}, Closed]],
Cell[CellGroupData[{
Cell[15792, 497, 702, 16, 42, "Input"],
Cell[16497, 515, 1398, 42, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[17932, 562, 325, 8, 42, "Input"],
Cell[18260, 572, 762, 11, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[19059, 588, 324, 8, 42, "Input"],
Cell[19386, 598, 655, 9, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[20078, 612, 367, 9, 42, "Input"],
Cell[20448, 623, 732, 11, 57, "Output"]
}, Closed]],
Cell[21195, 637, 710, 11, 128, "Text"],
Cell[CellGroupData[{
Cell[21930, 652, 340, 9, 43, "Input"],
Cell[22273, 663, 1202, 35, 108, "Output"]
}, Closed]],
Cell[23490, 701, 697, 15, 47, "Text"],
Cell[24190, 718, 1325, 32, 88, "DisplayFormula"],
Cell[25518, 752, 330, 6, 46, "Text"],
Cell[25851, 760, 1918, 46, 93, "Input"],
Cell[27772, 808, 162, 2, 46, "Text"],
Cell[CellGroupData[{
Cell[27959, 814, 142, 4, 48, "Input"],
Cell[28104, 820, 3433, 104, 292, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[31574, 929, 584, 12, 42, "Input"],
Cell[32161, 943, 9531, 291, 937, "Output"]
}, Closed]],
Cell[41707, 1237, 808, 12, 157, "Text"],
Cell[42518, 1251, 343, 9, 44, "Input"],
Cell[42864, 1262, 323, 10, 44, "Input"],
Cell[43190, 1274, 259, 8, 44, "Input"],
Cell[CellGroupData[{
Cell[43474, 1286, 125, 2, 43, "Input"],
Cell[43602, 1290, 165, 2, 57, "Output"]
}, Closed]],
Cell[43782, 1295, 160, 3, 39, "Text"],
Cell[CellGroupData[{
Cell[43967, 1302, 230, 5, 43, "Input"],
Cell[44200, 1309, 5439, 169, 314, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[49676, 1483, 127, 2, 32, "Input"],
Cell[49806, 1487, 163, 2, 57, "Output"]
}, Closed]],
Cell[49984, 1492, 292, 5, 39, "Text"],
Cell[50279, 1499, 394, 10, 44, "Input"],
Cell[50676, 1511, 376, 11, 44, "Input"],
Cell[51055, 1524, 329, 9, 44, "Input"],
Cell[CellGroupData[{
Cell[51409, 1537, 154, 3, 43, "Input"],
Cell[51566, 1542, 190, 3, 57, "Output"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[51805, 1551, 280, 4, 61, "Subsection"],
Cell[52088, 1557, 207, 5, 43, "Item"],
Cell[52298, 1564, 403, 7, 76, "Text"],
Cell[52704, 1573, 1039, 33, 103, "Input"],
Cell[CellGroupData[{
Cell[53768, 1610, 286, 9, 42, "Subsubsection"],
Cell[54057, 1621, 1064, 16, 223, "Text"],
Cell[CellGroupData[{
Cell[55146, 1641, 147, 4, 43, "Input"],
Cell[55296, 1647, 1274, 42, 134, "Output"]
}, Open  ]],
Cell[56585, 1692, 171, 6, 46, "Text"],
Cell[56759, 1700, 376, 6, 76, "Text"],
Cell[CellGroupData[{
Cell[57160, 1710, 161, 5, 43, "Input"],
Cell[57324, 1717, 1012, 33, 74, "Output"]
}, Open  ]],
Cell[58351, 1753, 544, 11, 47, "Text"],
Cell[CellGroupData[{
Cell[58920, 1768, 222, 6, 43, "Input"],
Cell[59145, 1776, 489, 16, 74, "Output"]
}, Open  ]],
Cell[59649, 1795, 598, 10, 105, "Text"],
Cell[CellGroupData[{
Cell[60272, 1809, 150, 4, 43, "Input"],
Cell[60425, 1815, 465, 16, 100, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[60927, 1836, 171, 5, 32, "Input"],
Cell[61101, 1843, 563, 19, 100, "Output"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[61713, 1868, 398, 12, 42, "Subsubsection"],
Cell[62114, 1882, 286, 5, 76, "Text"],
Cell[CellGroupData[{
Cell[62425, 1891, 355, 11, 43, "Input"],
Cell[62783, 1904, 1796, 60, 166, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64616, 1969, 270, 9, 43, "Input"],
Cell[64889, 1980, 1365, 46, 137, "Output"]
}, Open  ]],
Cell[66269, 2029, 258, 5, 76, "Text"],
Cell[CellGroupData[{
Cell[66552, 2038, 406, 12, 43, "Input"],
Cell[66961, 2052, 1396, 47, 137, "Output"]
}, Open  ]],
Cell[68372, 2102, 249, 5, 46, "Text"],
Cell[CellGroupData[{
Cell[68646, 2111, 270, 9, 43, "Input"],
Cell[68919, 2122, 1474, 48, 134, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[70442, 2176, 706, 17, 57, "Subsubsection"],
Cell[71151, 2195, 468, 9, 105, "Text"],
Cell[CellGroupData[{
Cell[71644, 2208, 310, 8, 43, "Input"],
Cell[71957, 2218, 794, 26, 71, "Output"]
}, Open  ]],
Cell[72766, 2247, 159, 3, 46, "Text"],
Cell[CellGroupData[{
Cell[72950, 2254, 235, 6, 43, "Input"],
Cell[73188, 2262, 220, 3, 43, "Output"]
}, Open  ]],
Cell[73423, 2268, 212, 4, 46, "Text"],
Cell[CellGroupData[{
Cell[73660, 2276, 125, 3, 43, "Input"],
Cell[73788, 2281, 150, 2, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[73975, 2288, 398, 10, 43, "Input"],
Cell[74376, 2300, 352, 10, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74765, 2315, 339, 9, 43, "Input"],
Cell[75107, 2326, 821, 27, 71, "Output"]
}, Open  ]],
Cell[75943, 2356, 343, 7, 76, "Text"],
Cell[CellGroupData[{
Cell[76311, 2367, 115, 3, 43, "Input"],
Cell[76429, 2372, 843, 27, 71, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77309, 2404, 163, 3, 43, "Input"],
Cell[77475, 2409, 772, 26, 71, "Output"]
}, Open  ]],
Cell[78262, 2438, 401, 7, 76, "Text"],
Cell[CellGroupData[{
Cell[78688, 2449, 289, 7, 43, "Input"],
Cell[78980, 2458, 1520, 49, 223, "Output"]
}, Closed]],
Cell[80515, 2510, 1735, 31, 130, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[82299, 2547, 284, 4, 61, "Subsection"],
Cell[82586, 2553, 1341, 41, 126, "Input"],
Cell[83930, 2596, 1138, 18, 253, "Text"],
Cell[85071, 2616, 2331, 60, 198, "Text"],
Cell[CellGroupData[{
Cell[87427, 2680, 118, 2, 43, "Subsubsection"],
Cell[CellGroupData[{
Cell[87570, 2686, 403, 12, 57, "Input"],
Cell[87976, 2700, 324, 11, 103, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[88349, 2717, 119, 2, 32, "Subsubsection"],
Cell[CellGroupData[{
Cell[88493, 2723, 672, 19, 57, "Input"],
Cell[89168, 2744, 3972, 132, 428, "Output"]
}, Closed]],
Cell[93155, 2879, 273, 5, 55, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[93465, 2889, 118, 2, 32, "Subsubsection"],
Cell[93586, 2893, 411, 7, 88, "Text"],
Cell[94000, 2902, 1307, 32, 91, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[95356, 2940, 437, 6, 61, "Subsection"],
Cell[95796, 2948, 1450, 34, 123, "Text"],
Cell[97249, 2984, 1905, 61, 219, "Input"],
Cell[CellGroupData[{
Cell[99179, 3049, 118, 2, 55, "Subsubsection"],
Cell[CellGroupData[{
Cell[99322, 3055, 333, 10, 57, "Input"],
Cell[99658, 3067, 507, 17, 61, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[100214, 3090, 193, 3, 55, "Subsubsection"],
Cell[CellGroupData[{
Cell[100432, 3097, 408, 12, 57, "Input"],
Cell[100843, 3111, 2701, 93, 230, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[103593, 3210, 167, 3, 55, "Subsubsection"],
Cell[103763, 3215, 121, 1, 55, "Text"],
Cell[103887, 3218, 554, 16, 57, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[104490, 3240, 445, 8, 47, "Subsection"],
Cell[104938, 3250, 165, 3, 43, "Item"],
Cell[105106, 3255, 609, 11, 88, "Text"],
Cell[105718, 3268, 974, 28, 54, "DisplayFormulaNumbered"],
Cell[106695, 3298, 555, 15, 81, "Text"],
Cell[107253, 3315, 948, 30, 60, "DisplayFormulaNumbered"],
Cell[108204, 3347, 3539, 78, 261, "Text"],
Cell[111746, 3427, 3511, 116, 172, "DisplayFormulaNumbered"],
Cell[115260, 3545, 1598, 34, 216, "Text"],
Cell[CellGroupData[{
Cell[116883, 3583, 130, 4, 43, "Input"],
Cell[117016, 3589, 175, 5, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[117228, 3599, 84, 2, 32, "Input"],
Cell[117315, 3603, 72, 1, 57, "Output"]
}, Closed]],
Cell[117402, 3607, 3807, 135, 255, "Input"],
Cell[121212, 3744, 1200, 19, 130, "Text"],
Cell[122415, 3765, 657, 21, 51, "Input"],
Cell[123075, 3788, 630, 22, 51, "Input"],
Cell[CellGroupData[{
Cell[123730, 3814, 144, 4, 43, "Input"],
Cell[123877, 3820, 172, 4, 57, "Output"]
}, Closed]],
Cell[124064, 3827, 178, 3, 39, "Text"],
Cell[CellGroupData[{
Cell[124267, 3834, 149, 3, 43, "Input"],
Cell[124419, 3839, 2174, 64, 159, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[126630, 3908, 151, 3, 32, "Input"],
Cell[126784, 3913, 2148, 63, 159, "Output"]
}, Closed]],
Cell[128947, 3979, 934, 14, 165, "Text"],
Cell[CellGroupData[{
Cell[129906, 3997, 159, 4, 43, "Input"],
Cell[130068, 4003, 1505, 45, 91, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[131610, 4053, 71, 1, 32, "Input"],
Cell[131684, 4056, 72, 1, 57, "Output"]
}, Closed]],
Cell[131771, 4060, 235, 4, 69, "Text"],
Cell[CellGroupData[{
Cell[132031, 4068, 182, 5, 43, "Input"],
Cell[132216, 4075, 1890, 50, 90, "Output"]
}, Closed]],
Cell[134121, 4128, 438, 7, 98, "Text"],
Cell[CellGroupData[{
Cell[134584, 4139, 551, 17, 69, "Input"],
Cell[CellGroupData[{
Cell[135160, 4160, 1965, 51, 75, "Print"],
Cell[137128, 4213, 1988, 52, 75, "Print"],
Cell[139119, 4267, 1965, 51, 75, "Print"],
Cell[141087, 4320, 1986, 52, 75, "Print"],
Cell[143076, 4374, 1965, 51, 75, "Print"],
Cell[145044, 4427, 1988, 52, 75, "Print"]
}, Closed]]
}, Closed]],
Cell[147059, 4483, 354, 7, 39, "Text"],
Cell[CellGroupData[{
Cell[147438, 4494, 1579, 48, 169, "Input"],
Cell[149020, 4544, 834, 24, 57, "Output"],
Cell[149857, 4570, 814, 23, 57, "Output"],
Cell[150674, 4595, 834, 24, 57, "Output"],
Cell[151511, 4621, 816, 23, 57, "Output"],
Cell[152330, 4646, 816, 23, 57, "Output"],
Cell[153149, 4671, 816, 23, 57, "Output"]
}, Closed]],
Cell[153980, 4697, 895, 15, 123, "Text"],
Cell[154878, 4714, 80, 3, 5, "PageBreak",
 PageBreakBelow->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[154995, 4722, 322, 5, 61, "Subsection"],
Cell[CellGroupData[{
Cell[155342, 4731, 216, 4, 43, "Item"],
Cell[CellGroupData[{
Cell[155583, 4739, 2619, 38, 237, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 15001.}],
Cell[158205, 4779, 1235, 20, 102, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 15001.}]
}, Closed]]
}, Closed]],
Cell[159467, 4803, 1588, 51, 69, "DisplayFormulaNumbered"],
Cell[161058, 4856, 730, 11, 76, "Text"],
Cell[161791, 4869, 2165, 76, 60, "DisplayFormulaNumbered"],
Cell[163959, 4947, 7010, 235, 173, "Text"],
Cell[170972, 5184, 736, 25, 45, "DisplayFormulaNumbered"],
Cell[171711, 5211, 776, 14, 47, "Text"],
Cell[172490, 5227, 955, 32, 45, "Input"],
Cell[173448, 5261, 186, 3, 46, "Text"],
Cell[173637, 5266, 713, 23, 47, "DisplayFormulaNumbered"],
Cell[174353, 5291, 1248, 33, 70, "Text"],
Cell[175604, 5326, 937, 31, 36, "DisplayFormulaNumbered"],
Cell[176544, 5359, 273, 3, 46, "Text"],
Cell[176820, 5364, 1458, 46, 37, "DisplayFormulaNumbered"],
Cell[178281, 5412, 586, 13, 42, "Text"],
Cell[178870, 5427, 494, 15, 64, "DisplayFormulaNumbered"],
Cell[179367, 5444, 2063, 40, 130, "Text"],
Cell[181433, 5486, 855, 29, 47, "DisplayFormulaNumbered"],
Cell[182291, 5517, 747, 14, 73, "Text"],
Cell[183041, 5533, 793, 26, 36, "DisplayFormulaNumbered"],
Cell[183837, 5561, 310, 5, 46, "Text"],
Cell[CellGroupData[{
Cell[184172, 5570, 2644, 61, 36, "DisplayFormulaNumbered",
 CellGroupingRules->{GroupTogetherGrouping, 15001.}],
Cell[186819, 5633, 3070, 49, 327, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 15001.}]
}, Closed]],
Cell[189904, 5685, 2854, 57, 255, "Text"],
Cell[192761, 5744, 533, 9, 105, "Text"],
Cell[193297, 5755, 463, 12, 36, "DisplayFormulaNumbered"],
Cell[193763, 5769, 429, 8, 46, "Text"],
Cell[194195, 5779, 3217, 96, 228, "Input"],
Cell[197415, 5877, 2159, 46, 312, "Text"],
Cell[199577, 5925, 423, 14, 49, "Input"],
Cell[CellGroupData[{
Cell[200025, 5943, 53, 1, 43, "Input"],
Cell[200081, 5946, 159, 2, 57, "Output"]
}, Closed]],
Cell[200255, 5951, 428, 8, 35, "Text"],
Cell[CellGroupData[{
Cell[200708, 5963, 61, 1, 43, "Input"],
Cell[200772, 5966, 125, 2, 57, "Output"]
}, Closed]],
Cell[200912, 5971, 417, 7, 69, "Text"],
Cell[CellGroupData[{
Cell[201354, 5982, 149, 3, 43, "Input"],
Cell[201506, 5987, 1442, 45, 102, "Output"]
}, Closed]],
Cell[202963, 6035, 153, 2, 39, "Text"],
Cell[203119, 6039, 328, 11, 49, "Input"],
Cell[CellGroupData[{
Cell[203472, 6054, 131, 2, 43, "Input"],
Cell[203606, 6058, 96, 1, 57, "Output"]
}, Closed]],
Cell[203717, 6062, 262, 5, 69, "Text"],
Cell[203982, 6069, 173, 4, 43, "Input"],
Cell[CellGroupData[{
Cell[204180, 6077, 137, 2, 43, "Input"],
Cell[204320, 6081, 100, 1, 57, "Output"]
}, Closed]],
Cell[204435, 6085, 1448, 20, 305, "Text"],
Cell[CellGroupData[{
Cell[205908, 6109, 51, 1, 43, "Input"],
Cell[205962, 6112, 1684, 26, 490, "Print",
 CellTags->"Info3566105331-6477710"]
}, Closed]],
Cell[207661, 6141, 313, 8, 32, "Input"],
Cell[CellGroupData[{
Cell[207999, 6153, 118, 2, 43, "Input"],
Cell[208120, 6157, 88, 1, 57, "Output"]
}, Closed]],
Cell[208223, 6161, 428, 7, 98, "Text"],
Cell[208654, 6170, 322, 8, 43, "Input"],
Cell[CellGroupData[{
Cell[209001, 6182, 71, 1, 43, "Input"],
Cell[209075, 6185, 74, 1, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[209186, 6191, 91, 2, 32, "Input"],
Cell[209280, 6195, 473, 12, 57, "Output"]
}, Closed]],
Cell[209768, 6210, 492, 8, 69, "Text"],
Cell[CellGroupData[{
Cell[210285, 6222, 55, 1, 43, "Input"],
Cell[210343, 6225, 564, 9, 154, "Print",
 CellTags->"Info3566105432-6477710"]
}, Closed]],
Cell[210922, 6237, 293, 5, 69, "Text"],
Cell[211218, 6244, 414, 11, 43, "Input"],
Cell[CellGroupData[{
Cell[211657, 6259, 159, 3, 43, "Input"],
Cell[211819, 6264, 73, 1, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[211929, 6270, 177, 4, 32, "Input"],
Cell[212109, 6276, 1145, 37, 60, "Output"]
}, Closed]],
Cell[213269, 6316, 323, 7, 35, "Text"],
Cell[213595, 6325, 859, 26, 49, "Input"],
Cell[CellGroupData[{
Cell[214479, 6355, 127, 3, 43, "Input"],
Cell[214609, 6360, 70, 1, 57, "Output"]
}, Closed]],
Cell[214694, 6364, 445, 7, 69, "Text"],
Cell[215142, 6373, 577, 15, 119, "Input"],
Cell[CellGroupData[{
Cell[215744, 6392, 173, 5, 43, "Input"],
Cell[215920, 6399, 5071, 67, 456, "Output"]
}, Closed]],
Cell[221006, 6469, 166, 3, 39, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[221209, 6477, 314, 5, 61, "Subsection"],
Cell[221526, 6484, 4369, 81, 286, "Text"],
Cell[225898, 6567, 6337, 186, 591, "Input"],
Cell[232238, 6755, 1924, 36, 178, "Text"],
Cell[234165, 6793, 1120, 36, 69, "Input"],
Cell[CellGroupData[{
Cell[235310, 6833, 455, 12, 43, "Input"],
Cell[235768, 6847, 157, 3, 57, "Output"]
}, Closed]],
Cell[235940, 6853, 252, 5, 39, "Text"],
Cell[CellGroupData[{
Cell[236217, 6862, 293, 9, 43, "Input"],
Cell[236513, 6873, 7274, 226, 290, "Output"]
}, Closed]],
Cell[243802, 7102, 163, 3, 39, "Text"],
Cell[CellGroupData[{
Cell[243990, 7109, 334, 10, 43, "Input"],
Cell[244327, 7121, 132, 3, 57, "Output"]
}, Closed]],
Cell[244474, 7127, 437, 10, 32, "Text"],
Cell[CellGroupData[{
Cell[244936, 7141, 334, 10, 43, "Input"],
Cell[245273, 7153, 134, 3, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[245444, 7161, 334, 10, 32, "Input"],
Cell[245781, 7173, 132, 3, 57, "Output"]
}, Closed]],
Cell[245928, 7179, 336, 6, 69, "Text"],
Cell[CellGroupData[{
Cell[246289, 7189, 360, 11, 43, "Input"],
Cell[246652, 7202, 134, 3, 57, "Output"]
}, Closed]],
Cell[246801, 7208, 273, 6, 39, "Text"],
Cell[247077, 7216, 463, 13, 43, "Input"],
Cell[CellGroupData[{
Cell[247565, 7233, 266, 5, 43, "Input"],
Cell[247834, 7240, 1187, 36, 91, "Output"]
}, Closed]],
Cell[249036, 7279, 514, 14, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[249587, 7298, 234, 5, 65, "Subsection"],
Cell[249824, 7305, 1139, 24, 489, "Text"],
Cell[250966, 7331, 793, 23, 44, "Input"],
Cell[251762, 7356, 482, 14, 44, "Input"],
Cell[252247, 7372, 984, 27, 69, "Input"],
Cell[253234, 7401, 314, 6, 46, "Text"],
Cell[253551, 7409, 721, 20, 69, "Input"],
Cell[254275, 7431, 356, 6, 76, "Text"],
Cell[CellGroupData[{
Cell[254656, 7441, 369, 9, 43, "Input"],
Cell[255028, 7452, 492, 15, 68, "Output"]
}, Closed]],
Cell[255535, 7470, 257, 5, 39, "Text"],
Cell[255795, 7477, 2009, 57, 169, "Input"],
Cell[257807, 7536, 132, 1, 46, "Text"],
Cell[CellGroupData[{
Cell[257964, 7541, 132, 2, 43, "Input"],
Cell[258099, 7545, 467, 14, 68, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[258603, 7564, 134, 2, 32, "Input"],
Cell[258740, 7568, 431, 14, 68, "Output"]
}, Closed]],
Cell[259186, 7585, 617, 10, 128, "Text"],
Cell[CellGroupData[{
Cell[259828, 7599, 321, 8, 43, "Input"],
Cell[260152, 7609, 1826, 51, 177, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[262015, 7665, 324, 8, 32, "Input"],
Cell[262342, 7675, 1262, 37, 134, "Output"]
}, Closed]],
Cell[263619, 7715, 672, 11, 128, "Text"],
Cell[264294, 7728, 339, 10, 43, "Input"],
Cell[CellGroupData[{
Cell[264658, 7742, 522, 13, 43, "Input"],
Cell[265183, 7757, 138, 3, 57, "Output"]
}, Closed]],
Cell[265336, 7763, 418, 7, 98, "Text"],
Cell[265757, 7772, 837, 21, 94, "Input"],
Cell[266597, 7795, 1141, 31, 94, "Input"],
Cell[CellGroupData[{
Cell[267763, 7830, 146, 3, 43, "Input"],
Cell[267912, 7835, 79, 1, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[268028, 7841, 144, 3, 32, "Input"],
Cell[268175, 7846, 87, 1, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[268299, 7852, 286, 8, 32, "Input"],
Cell[268588, 7862, 1097, 22, 115, "Output"]
}, Closed]],
Cell[269700, 7887, 526, 9, 98, "Text"],
Cell[270229, 7898, 323, 8, 44, "Input"],
Cell[CellGroupData[{
Cell[270577, 7910, 257, 5, 43, "Input"],
Cell[270837, 7917, 716, 18, 57, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[271590, 7940, 307, 6, 32, "Input"],
Cell[271900, 7948, 642, 14, 57, "Output"]
}, Closed]],
Cell[272557, 7965, 461, 7, 98, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[273067, 7978, 133, 3, 70, "Section"],
Cell[CellGroupData[{
Cell[273225, 7985, 350, 5, 73, "Subsection"],
Cell[273578, 7992, 1683, 33, 283, "Text"],
Cell[CellGroupData[{
Cell[275286, 8029, 641, 20, 102, "Input"],
Cell[275930, 8051, 1114, 34, 102, "Output"]
}, Closed]],
Cell[277059, 8088, 939, 15, 237, "Text"],
Cell[CellGroupData[{
Cell[278023, 8107, 355, 11, 102, "Input"],
Cell[278381, 8120, 412, 13, 61, "Output"]
}, Closed]],
Cell[278808, 8136, 4222, 79, 316, "Text"],
Cell[CellGroupData[{
Cell[283055, 8219, 553, 18, 98, "Input"],
Cell[283611, 8239, 705, 21, 57, "Output"]
}, Closed]],
Cell[284331, 8263, 6241, 116, 327, "Text"],
Cell[290575, 8381, 1175, 18, 283, "Text"],
Cell[CellGroupData[{
Cell[291775, 8403, 269, 8, 69, "Input"],
Cell[292047, 8413, 747, 23, 69, "Output"]
}, Closed]],
Cell[292809, 8439, 428, 8, 102, "Text"],
Cell[CellGroupData[{
Cell[293262, 8451, 691, 22, 57, "Input"],
Cell[293956, 8475, 518, 16, 96, "Output"]
}, Closed]],
Cell[294489, 8494, 985, 14, 237, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[295511, 8513, 397, 8, 76, "Subsection"],
Cell[CellGroupData[{
Cell[295933, 8525, 210, 5, 66, "Subsubsection"],
Cell[CellGroupData[{
Cell[296168, 8534, 595, 20, 133, "Input"],
Cell[296766, 8556, 34198, 1047, 4069, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[331013, 9609, 222, 5, 46, "Subsubsection"],
Cell[331238, 9616, 170, 4, 82, "Text"],
Cell[CellGroupData[{
Cell[331433, 9624, 174, 3, 81, "Input"],
Cell[331610, 9629, 30810, 964, 1756, "Output"]
}, Closed]],
Cell[362435, 10596, 92, 1, 81, "Input"],
Cell[362530, 10599, 92, 1, 81, "Input"]
}, Closed]]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
